/*
*  OV7740 Setup
*/
#include <stdio.h>
#include "r_typedefs.h"                         /* Renesas data types header */
#include "compiler_settings.h"
#include "iodefine.h"                           /* Hardware interface header */
#include "dev_drv.h"                          /* Device Driver common header */
#include "r_ceu.h"
#include "camera.h"
//The 7740 sits on i2c with ID 0x42
#define OV7740_I2C_ADDR 0x42

//typedef struct {
//  uint8_t addr;
//  uint8_t val;
//} OmniRegister_t;

const OmniRegister_t OV7740_Reset_Regs[] = {

  {0x12,0x80}, // - Reset Registers to Default and set Mode to YUV
  //Auto AWB, AEC, AGC
  {0xff, 0xff} //structure end marker
};
const OmniRegister_t OV7740_Common_Regs[] = {

  {0x13,0xff}, //  option for auto mode
  {0x11,0x01}, // 00/01/03/07 for 60/30/15/7.5fps
  {0x12,0x00}, // // - Remove Reset and set Mode to YUV
  {0x0c,0x12}, // 0x12 UV Swap
  {0x1b,0x89}, // Pixel shift = 137
  {0x36,0x3f}, // Analog value (?)
  {0x47,0x42}, // Analog value (?)
  {0x86,0x03}, // AGC base1: sharpness upper limit
  {0x87,0x01}, // AGC base2: sharpness lower limit
  {0x13,0xff}, // select this option for auto gain mode
  {0x15,0xc4}, //Night Mode -> Cumulates several frames
  {0x16,0x00},   // reg16
  {0x17,0x26},   // AHSTART = 152 (2LSB are in reg16)
  {0x18,0xa0},   // AHSIZE  = 640 (2LSB are in reg16)
  {0x19,0x03},   // AVSTART = 6   (LSB is in reg16)
  {0x1a,0xf0},   // AVSIZE  = 480 (LSB is in reg16)
  {0x8d ,0x30},   // LENC
  {0x8f ,0x85},   // LENC
  {0x93 ,0x30},   // LENC
  {0x95 ,0x85},   // LENC
  {0x99 ,0x30},   // LENC
  {0x9b ,0x85},   // LENC
  {0x24, 120},    //LUMINANCE_STABLE_ZONE_HIGH
  {0x25, 104},    //LUMINANCE_STABLE_ZONE_LOW
  {0x26, 0x95},   //LUMINANCE_CONTROL_ZONE
  {0xff, 0xff} //structure end marker
};

const OmniRegister_t OV7740_RGB[] = {
  {0x12,0x01}, // set Mode to RGB only RGB888 is supported !
  {0xff, 0xff} //structure end marker
};

const OmniRegister_t OV7740_VGA[] = {
  {0x31,0xa0}, // HOUTSIZE = 640
  {0x32,0xf0}, // VOUTSIZE = 480
  {0x82,0x32}, // All activated: Yaverage, FIFO, Vscaler, Hscaler, Window crop, Smooth
  {0xE5,0x0C}, //scaleV ctrl
  {0xff, 0xff} //structure end marker
};


const OmniRegister_t OV7740_QVGA[] = {
  {0x31,0x50}, // HOUTSIZE = 320
  {0x32,0x78}, // VOUTSIZE = 240
  {0x82,0x3E},
  {0xE5,0x8C}, //scaleV ctrl
  {0xff, 0xff} //structure end marker
};

const OmniRegister_t OV7740_Picture_Color[] = {
  {0xda,0x00},   // Fixed YU disable
  {0xff, 0xff} //structure end marker
};

const OmniRegister_t OV7740_Picture_WB[] = {
  {0xda,0x18},   // Fixed YU enable
  {0xff, 0xff} //structure end marker
};

static const OmniRegister_t OV7740_Test[] = {
	{0x28,0x02}, //VSYNC Negative Value
	{0xda,0x00},   // Fixed YU disable
	{0xff, 0xff} //structure end marker
	};


static void WriteOmniTable(OmniRegister_t Table[])
{
	uint16_t i;
for(i=0;((Table[i].addr!=0xff)||(Table[i].val!=0xff));i++)
	R_RIIC_CAMERA_write(DEVDRV_CH_1, OV7740_I2C_ADDR, Table[i].addr,Table[i].val );

}

//This function should be replaced by RTOS delay function
#define WAIT_TICK 100000
void msleep_ov7740(uint32_t wait)
{
	uint32_t i;
	for(i=WAIT_TICK;((i>=0)&&(wait!=0));i--)
	{
		if(i==0)
		{
		wait--;
		i=WAIT_TICK;
		}
	}
}
void InitCameraOV7740(void)
{

//Write the default values in the camera module
 WriteOmniTable(OV7740_Reset_Regs);
 msleep_ov7740(100); //The Omnivision chip needs some wait time after reset

// WriteOmniTable(Omni_RachidQVGA);

 //Write the default values in the camera module
 WriteOmniTable(OV7740_Common_Regs);
#ifdef QVGA
 WriteOmniTable(OV7740_QVGA);
#endif
#ifdef VGA
 WriteOmniTable(OV7740_VGA);
#endif
 WriteOmniTable(OV7740_Test);

}
