/******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2013 Renesas Electronics Corporation. All rights reserved.
******************************************************************************/
/******************************************************************************
* File Name     : main.c
* Device(s)     : RZ/A1H RSK+RZA1H
* H/W Platform  : RSK+RZA1H CPU Board
* Tool-Chain    : GNUARM-RZv13.01-EABI
* Description  : RZ/A1H Sample Program - Main
* Operation     : none
* Limitations   : none
******************************************************************************/
/******************************************************************************
* History       : DD.MM.YYYY Version Description
*               : 18.06.2013 1.00
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include <stdio.h>
#include "r_typedefs.h"                         /* Renesas data types header */
#include "compiler_settings.h"
#include "iodefine.h"                           /* Hardware interface header */
#include "dev_drv.h"                          /* Device Driver common header */
#include "devdrv_intc.h"                               /* INTC driver header */
#include "devdrv_riic.h"                               /* RIIC Driver Header */
#include "sample_vdc5_rza1h_rsk_i2c.h"   /* I2C RSK+RZA1H Eval-board header */
#include "sample_riic_rza1h_rsk_i2c.h"   /* I2C RSK+RZA1H Eval-board header */
//#include "sample_vdc5.h"                        /* Sample RIIC r/w functions */
#include "sample_led.h"              /* Sample on-board LED interface header */
#include "logo.h"             /* Renesas logo (used with colour PMod display */
#include "rspi.h"
#include "rza_io_regrw.h"
#include "main.h"                               /* Main sample module header */
#include "ceu_sample.h"
#include "r_ceu.h"
#include "camera.h"
/******************************************************************************
Typedef definitions
******************************************************************************/

/******************************************************************************
Macro definitions
******************************************************************************/

#define SAMPLE_RIIC_CAMERA_ADDR_WRITE       (0x42u)         /* CAMERA device code when writing  */
#define SAMPLE_RIIC_CAMERA_ADDR_READ       	(0x43u)         /* CAMERA device code when reading  */

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

extern void R_RIIC_rza1h_rsk_init(void);

#ifdef CAMERA_OV7740
void InitCameraOV7740(void);
#endif

#ifdef CAMERA_OV7670
void InitCameraOV7670(void);
#endif

void CEU_SAMPLE_CeuInit(void);

unsigned char PingPong = 0;

#ifdef YUV420
extern stVideoBuffer FrameBufferA,FrameBufferB;
#endif
#ifdef RGB565
extern stVideoBuffer FrameBufferA[Y_SIZE*2],FrameBufferB[Y_SIZE];
#endif
/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/
#ifdef YUV420
void CleanBuffer(stVideoBuffer *Frame)
{
	uint32_t i;

	for(i=0;i!=Y_SIZE;i++)
		Frame->Y[i]=0x55;
	for(i=0;i!=UV_SIZE;i++)
		Frame->UV[i]=0xaa;
};
#endif

#ifdef RGB565
void CleanBuffer(stVideoBuffer *Frame)
{
	uint32_t i,j;
	j= Y_SIZE;
	for(i=0;i!=Y_SIZE;i++)
		Frame[i]=0x55AA;
	j=i;
};
#endif

void InitCameraModule(void)
{
	Pekoe_RIIC_init();

	CleanBuffer(&FrameBufferA);
//	CleanBuffer(&FrameBufferB);

	//Initialise the Omnivision internal registers
#ifdef CAMERA_OV7740
	InitCameraOV7740();
#endif

#ifdef CAMERA_OV7670
	InitCameraOV7670();
#endif

	//RZ-CEU Initialization
	CEU_SAMPLE_CeuInit();
}

void StartCameraCapture(void)
{
	while (CEU_CaptureOneFrame(&FrameBufferA)!=CEU_ERROR_OK);
	PingPong = 0;
}

unsigned char *GetCameraFrame(void)
{
	unsigned char *CaptureBuffer;

	if(PingPong==0)
	{
		// Get the YUV420 frame buffer pointer from capture driver
		while (CEU_CaptureOneFrame(&FrameBufferA)!=CEU_ERROR_OK);
		PingPong=1;
		CaptureBuffer = (unsigned char *)&FrameBufferB;
	}
	else
	{
		// Get the YUV420 frame buffer pointer from capture driver
		while (CEU_CaptureOneFrame(&FrameBufferB)!=CEU_ERROR_OK);
		PingPong=0;
		CaptureBuffer = (unsigned char *)&FrameBufferA;
	}

	return CaptureBuffer;
}

/* End of File */
