/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2013 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name   : PekoeSpibsc_init2.c
* Description : Flash boot setting
*******************************************************************************/


/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include "r_typedefs.h"
#include "iodefine.h"
#include "spibsc.h"
#include "r_spibsc_ioset_api.h"
#include "rza_io_regrw.h"
#include "PekoeSpibsc_init2.h"
/******************************************************************************
Typedef definitions
******************************************************************************/


/******************************************************************************
Macro definitions
******************************************************************************/

uint32_t error_sequence[] = {DELAY_20000MS,
	                      	 DELAY_500MS,
		                     DELAY_500MS,
		                     DELAY_500MS,
                             -1u};

char signature[] = ".BootLoad_ValidProgramTest.";

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/
extern void UserProgJmp(uint32_t addr);
/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/
st_spibsc_cfg_t g_spibsc_cfg;

/******************************************************************************
Private global variables and functions
******************************************************************************/
void spibsc_init2(void);

/******************************************************************************
* Function Name: error_image
* Description  : This function is only called if the final boot loader stage
*                cannot recognise that a valid users application is available.
*                It will not exit and cofigures the LED0 as an output and
*                flashes the LED with a pre-defined recoginsable pattern.
* Arguments    : none
* Return Value : none
******************************************************************************/
void Pekoe_error_image(void)
{
	uint32_t delay;
	uint32_t index = 0;
	uint8_t state = 0;

    /* The port pin for LED0 is configured as an output */
    RZA_IO_RegWrite_16(&GPIO.PMC7,  0, GPIO_PMC7_PMC71_SHIFT,  GPIO_PMC7_PMC71);
    RZA_IO_RegWrite_16(&GPIO.P7,  1, GPIO_P7_P71_SHIFT,  GPIO_P7_P71);
    RZA_IO_RegWrite_16(&GPIO.PM7,  0, GPIO_PM7_PM71_SHIFT,  GPIO_PM7_PM71);
    RZA_IO_RegWrite_16(&GPIO.PIPC7, 0, GPIO_PIPC7_PIPC71_SHIFT, GPIO_PIPC7_PIPC71);

    /* This function will never exit - the system has failed to boot. */
    while(1)
    {
    	index = 0;
    	while(error_sequence[index] != -1u)
    	{
    		delay = error_sequence[index++];
		    RZA_IO_RegWrite_16(&GPIO.P7,  state, GPIO_P7_P71_SHIFT,  GPIO_P7_P71);
		    while (--delay)
			{
				__asm__("nop");
			}

		    state = state?0:1;
    	}
    }
}

/******************************************************************************
* Function Name: check_image
* Description  : This function checks that the signature text expected is
*                correct before attempting to jump to the application code.
* Arguments    : none
* Return Value : none
******************************************************************************/
void check_image(void)
{
    uint8_t p    = 0;
    uint8_t *src = (uint8_t *)DEF_USER_SIGNATURE;
    uint16_t len = sizeof(signature);

    while(--len)
    {
    	if( *(src++) != signature[p++])
        {
            break;
        }
    }

    /* If the length remaining is not zero then the signature validation failed. */
    if(!(0 == len))
    {
        /* This function will never return. */
    	Pekoe_error_image();
    }
}



/******************************************************************************
* Function Name: PekoeSpibsc_init2
* Description  :
* Arguments    : none
* Return Value : none
******************************************************************************/
uint8_t PekoeSpibsc_init2(uint32_t dual)
{
	uint8_t status;
	/* Wait TEND=1 for setting change in SPIBSC. */
    R_SFLASH_WaitTend(SPIBSC_CH);

    /* ==== Sets the SPIBSC ==== */
    R_SFLASH_Set_Config(SPIBSC_CH, &g_spibsc_cfg);
    if( R_SFLASH_Exmode_Setting(SPIBSC_CH, dual, &g_spibsc_cfg) != 0 )
    	status=-1;
    else status=NULL;


}


/* End of File */
