
#include "r_typedefs.h"
#include "commun.h"
#include "PekoeSpibsc_init2.h"
#include "spibsc.h"
#include "r_spibsc_ioset_api.h"
extern st_spibsc_cfg_t g_spibsc_cfg;

#define BASE_ADDRESS  (volatile uint16_t *) 0x58000000 /* no cache area */

#define S25FL512S_SECTORSIZE 0x40000
//#define S25FL512S_NB_SECTORS 256
#define S25FL512S_NB_SECTORS 10 /* for test purpose only */
#define NUM_BYTES     (S25FL512S_NB_SECTORS * S25FL512S_SECTORSIZE) /* Testing max 64MBytes = 32M 32bits words */

/**********************************************************************
 *
 * Function:    QspiWritePattern()
 *
 * Description: Test the integrity of a physical memory device by
 *              performing an increment/decrement test over the
 *              entire region.  In the process every storage bit
 *              in the device is tested as a zero and a one.  The
 *              base address and the size of the region are
 *              selected by the caller.
 *
 * Notes:
 *
 * Returns:     NULL if the test succeeds.
 *
 *              A non-zero result is the first address at which an
 *              incorrect value was read back.  By examining the
 *              contents of memory, it may be possible to gather
 *              additional information about the problem.
 *
 **********************************************************************/
uint32_t QspiWritePattern(volatile uint64_t * baseAddress, uint32_t nBytes)
{
	uint32_t  offset,ret;
	uint32_t nWords = nBytes / sizeof(uint64_t);
    uint64_t pattern,val;
    uint64_t *addr;
    uint8_t  index;

    printf("QSPI Flash devices pattern write start \n\r");
	PekoeSpibsc_init2(SPIBSC_CMNCR_BSZ_DUAL); //Just to check the memories contents with debugger
    /*
     * Fill memory with a known pattern.
     */
    for (pattern = 0, offset = 0, index = 0; offset < nWords; pattern++, offset++)
    	{
        /* baseAddress[offset] = pattern; */
    	ret = R_SFLASH_ByteProgram(&baseAddress[offset], &pattern, sizeof(pattern), SPIBSC_CH,SPIBSC_CMNCR_BSZ_DUAL,  g_spibsc_cfg.udef_data_width, g_spibsc_cfg.udef_addr_mode);
//    	ret = R_SFLASH_ByteProgram(&baseAddress[offset], &pattern, sizeof(pattern), SPIBSC_CH,SPIBSC_CMNCR_BSZ_SINGLE,  g_spibsc_cfg.udef_data_width, g_spibsc_cfg.udef_addr_mode);
    	if(index++ == 255)
    		{
    		index = 0;
    		printf("+ ");
    		}
    	if(ret!=NULL)
    		{
    		printf("QSPI write check error at address=0x%x \n\r",&baseAddress[offset]);
    		return((uint32_t) &baseAddress[offset]);
    		}
    	}

    /*
     * Check each location .
     */
    printf("QSPI Flash devices pattern verify start \n\r");
	PekoeSpibsc_init2(SPIBSC_CMNCR_BSZ_DUAL);

    for (pattern = 0, offset = 0, index = 0; offset < nWords; pattern++, offset++)
    	{
    	if(index++ == 255)
    		{
    		index = 0;
    		printf("- ");
    		}
    	addr = (uint32_t *)((uint32_t )&baseAddress[offset] + (uint32_t )BASE_ADDRESS);
    	val = *addr;
        if (val != pattern)
        	{
        	printf("QSPI check error at address=0x%x \n\r",&baseAddress[offset]);
        	return((uint32_t) &baseAddress[offset]);
        	}
    	}
    printf("QSPI Flash devices pattern verify successfully completed \n\r");
    return(NULL);
}
/**********************************************************************
 *
 * Function:    QspiRead()
 *
 * Description: Loop reading into QSPI call this function if you want to check the chronograms
 *
 * Notes:
 *
 * Returns:
 **********************************************************************/
uint32_t QspiRead(volatile uint64_t * baseAddress, uint32_t nBytes)
{
	uint32_t  offset,ret;
	uint32_t nWords = nBytes / sizeof(uint64_t);

    uint64_t pattern,antipattern;
    uint32_t *addr;

	PekoeSpibsc_init2(SPIBSC_CMNCR_BSZ_DUAL);
for(;;)
    for (offset = 0; offset < nWords;  offset++)
    {
    	addr = (uint32_t *)((uint32_t )&baseAddress[offset] + (uint32_t )BASE_ADDRESS);
        pattern = *addr;
    };


    return (NULL);
}
/**********************************************************************
 * Function:    QspiEraseAll()
 * Description: Erases the complete QSPI device
 * Notes: it takes ages (several minutes) on a 64 Mbyte (S25FL512S)
 * Returns:     NULL if the test succeeds.
 **********************************************************************/
uint16_t * QspiEraseAll(void)
{
	uint32_t  i, addr, ret;

    for(i=0;i!=S25FL512S_NB_SECTORS;i++)
    	{
    	printf("Erasing sector %d \n\r",i);
    	addr = i*S25FL512S_SECTORSIZE;
    	ret = R_SFLASH_EraseSector(addr, SPIBSC_CH, SPIBSC_CMNCR_BSZ_DUAL, g_spibsc_cfg.udef_data_width, g_spibsc_cfg.udef_addr_mode);
    	if(ret)
    		{
    		printf("Sector erase error \n\r");
    		return(i+1);
    		}
    	}
    printf("Device successfully erased \n\r");
    return (NULL);
}   /* Erase all device() */

/**********************************************************************
 * Function:    PekoeQspiTest()
 * Description: Erases the complete QSPI device, writes a sequence to it and checks it
 * Notes: it takes ages (several minutes) on a 64 Mbyte (S25FL512S)
 * Returns:     NULL if the test succeeds.
 **********************************************************************/
void PekoeQspiTest(void)
{


uint32_t FailureLocation;
	printf("RENESAS Pekoe RZ-A1L QSPI test start\n\r");
	PekoeSpibsc_init1();
	PekoeSpibsc_init2(SPIBSC_CMNCR_BSZ_DUAL);
#ifdef debug
	QspiEraseAll();
	FailureLocation = QspiWritePattern(0, NUM_BYTES);
	if(FailureLocation != NULL) return(FailureLocation);
	else printf("RENESAS Pekoe RZ-A1L QSPI test successfully finished\n\r");
#endif
}

/* PekoeQspiTest() */


