/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2014 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name     : rspi.c
* Device(s)     : RZ/A1H (R7S721001)
* Tool-Chain    : GNUARM-RZv13.01-EABI
* H/W Platform  : RSK+RZA1H CPU Board
* Description   : This file implements device driver for RSPI module.
*******************************************************************************/
/*******************************************************************************
* History       : DD.MM.YYYY Version Description
*               : 18.06.2013 1.00
*               : 21.03.2014 2.00
*******************************************************************************/

/******************************************************************************
Pragma directive
******************************************************************************/

/******************************************************************************
Includes
******************************************************************************/
/* Default  type definition header */
#include "r_typedefs.h"
/* I/O Register root header */
#include "iodefine.h"
/* INTC Driver Header */
#include "devdrv_intc.h"
/* Device driver for RSPI header */
#include "rspi.h"
/* Low level register read/write header */
#include "rza_io_regrw.h"
/* Interchangeable compiler specific header */
#include "compiler_settings.h"

/******************************************************************************
Global variables and functions
******************************************************************************/
/* RSPI1 transmit buffer address */
uint32_t * gp_rspi1_tx_address;
/* RSPI1 transmit data number */
uint16_t   g_rspi1_tx_count;
/* RSPI1 receive buffer address */
uint32_t * gp_rspi1_rx_address;
/* RSPI1 receive data number */
uint16_t   g_rspi1_rx_count;
/* RSPI1 receive data length */
uint16_t   g_rspi1_rx_length;

/* SCI5 transmit buffer address */
uint8_t * gp_spi_tx_address;
/* SCI5 transmit data number */
uint16_t  g_spi_tx_count;

/*******************************************************************************
* Function Name: R_RSPI1_Create
* Description  : This function initialises the RSPI1 module.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void Pekoe_RSPI1_Create(void)
{
	uint16_t dummy_word  = 0u;
	uint8_t  dummy_byte  = 0u;

    UNUSED_VARIABLE(dummy_word);
    UNUSED_VARIABLE(dummy_byte);

	/* Configure and disable SPI0 transmit interrupts */
	R_INTC_Disable(INTC_ID_SPTI4);
	R_INTC_RegistIntFunc(INTC_ID_SPTI4, &r_rspi1_transmit_interrupt);
	R_INTC_SetPriority(INTC_ID_SPTI4, ADC_PRI_SPTI1);

	RSPI1.SPPCR = 0u;
	dummy_byte = RSPI1.SPPCR;
    /* P1 clock = 66.67MHz, SPI bit rate = 11.11Mbits/s Check Table 16.3 */
    RSPI1.SPBR  = 2u;
    /* P1 clock = 66.67MHz, SPI bit rate = 1.9Mbits/s Check Table 16.3 */
//    RSPI1.SPBR  = 16u;
	dummy_byte = RSPI1.SPBR;
    RSPI1.SPDCR = 0x20u;
    dummy_byte = RSPI1.SPDCR;
    RSPI1.SPSCR = 0u;
	dummy_byte = RSPI1.SPSCR;
    RSPI1.SPCKD = 0;
	dummy_byte = RSPI1.SPCKD;
    RSPI1.SSLND = 0u;
    dummy_byte =  RSPI1.SSLND;
    RSPI1.SPND  = 0u;
    dummy_byte = RSPI1.SPND;
    RSPI1.SSLP  = 0u;
    dummy_byte = RSPI1.SSLP;
    RSPI1.SPSSR  = 0u;
    dummy_byte = RSPI1.SPSSR;
    RSPI1.SPBFCR  |= 0x30u;
    dummy_byte = RSPI1.SPBFCR;
    /* LSB first, use 16 bit data length */
    RSPI1.SPCMD0 = 0x0703u;
    dummy_word = RSPI1.SPCMD0;

    /* Set P6_13 as CS output pin for PMOD */
    RZA_IO_RegWrite_16(&GPIO.PM6,   0, GPIO_PM6_PM613_SHIFT,    GPIO_PM6_PM613);
    RZA_IO_RegWrite_16(&GPIO.PMC6,  0, GPIO_PMC6_PMC613_SHIFT,  GPIO_PMC6_PMC613);
    RZA_IO_RegWrite_16(&GPIO.PFCAE6,0, GPIO_PFCAE6_PFCAE613_SHIFT,GPIO_PFCAE6_PFCAE613);
    RZA_IO_RegWrite_16(&GPIO.PFCE6, 0, GPIO_PFCE6_PFCE613_SHIFT, GPIO_PFCE6_PFCE613);
    RZA_IO_RegWrite_16(&GPIO.PFC6,  0, GPIO_PFC6_PFC613_SHIFT,  GPIO_PFC6_PFC613);
    RZA_IO_RegWrite_16(&GPIO.PIPC6, 0, GPIO_PIPC6_PIPC613_SHIFT, GPIO_PIPC6_PIPC613);

    /* Set P6_12 as RSPCKA (function 3 output) PMC=1 PFCAE=0 PFCE=1 PFC=0 PM=0 */
    RZA_IO_RegWrite_16(&GPIO.PM6,   0, GPIO_PM6_PM612_SHIFT,    GPIO_PM6_PM612);
    RZA_IO_RegWrite_16(&GPIO.PMC6,  1, GPIO_PMC6_PMC612_SHIFT,  GPIO_PMC6_PMC612);
    RZA_IO_RegWrite_16(&GPIO.PFCAE6,0, GPIO_PFCAE6_PFCAE612_SHIFT,GPIO_PFCAE6_PFCAE612);
    RZA_IO_RegWrite_16(&GPIO.PFCE6, 1, GPIO_PFCE6_PFCE612_SHIFT, GPIO_PFCE6_PFCE612);
    RZA_IO_RegWrite_16(&GPIO.PFC6,  0, GPIO_PFC6_PFC612_SHIFT,  GPIO_PFC6_PFC612);
    RZA_IO_RegWrite_16(&GPIO.PIPC6, 1, GPIO_PIPC6_PIPC612_SHIFT, GPIO_PIPC6_PIPC612);

	/* Set P6_14 MOSIA  (function 3 output) PMC=1 PFCAE=0 PFCE=1 PFC=0 PM=0 */
    RZA_IO_RegWrite_16(&GPIO.PM6,  0, GPIO_PM6_PM614_SHIFT,    GPIO_PM6_PM614);
    RZA_IO_RegWrite_16(&GPIO.PMC6,  1, GPIO_PMC6_PMC614_SHIFT,  GPIO_PMC6_PMC614);
    RZA_IO_RegWrite_16(&GPIO.PFCAE6,0, GPIO_PFCAE6_PFCAE614_SHIFT,GPIO_PFCAE6_PFCAE614);
    RZA_IO_RegWrite_16(&GPIO.PFCE6, 1, GPIO_PFCE6_PFCE614_SHIFT, GPIO_PFCE6_PFCE614);
    RZA_IO_RegWrite_16(&GPIO.PFC6,  0, GPIO_PFC6_PFC614_SHIFT,  GPIO_PFC6_PFC614);
    RZA_IO_RegWrite_16(&GPIO.PIPC6, 1, GPIO_PIPC6_PIPC614_SHIFT, GPIO_PIPC6_PIPC614);

    /* Set P6_15 MISOA (function 3 input) PMC=1 PFCAE=0 PFCE=1 PFC=0 PM=1 */
    RZA_IO_RegWrite_16(&GPIO.PM6,   1, GPIO_PM6_PM615_SHIFT,    GPIO_PM6_PM615);
    RZA_IO_RegWrite_16(&GPIO.PMC6,  1, GPIO_PMC6_PMC615_SHIFT,  GPIO_PMC6_PMC615);
    RZA_IO_RegWrite_16(&GPIO.PFCAE6,0, GPIO_PFCAE6_PFCAE615_SHIFT,GPIO_PFCAE6_PFCAE615);
    RZA_IO_RegWrite_16(&GPIO.PFCE6, 1, GPIO_PFCE6_PFCE615_SHIFT, GPIO_PFCE6_PFCE615);
    RZA_IO_RegWrite_16(&GPIO.PFC6,  0, GPIO_PFC6_PFC615_SHIFT,  GPIO_PFC6_PFC615);
    RZA_IO_RegWrite_16(&GPIO.PIPC6, 1, GPIO_PIPC6_PIPC615_SHIFT, GPIO_PIPC6_PIPC615);

	/* Set P1_9 as PMOD IRQ1 pin (function 2 input) PMC=1 PFCAE=0 PFCE=0 PFC=1 PM=1 */
/*   	RZA_IO_RegWrite_16(&GPIO.PM1,   1, GPIO_PM1_PM9_SHIFT,    GPIO_PM1_PM19);
    RZA_IO_RegWrite_16(&GPIO.PMC1,  1, GPIO_PMC1_PMC9_SHIFT,  GPIO_PMC1_PMC19);
    RZA_IO_RegWrite_16(&GPIO.PFCAE1,0, GPIO_PFCAE1_PFCAE9_SHIFT,GPIO_PFCAE1_PFCAE19);
    RZA_IO_RegWrite_16(&GPIO.PFCE1, 0, GPIO_PFCE1_PFCE9_SHIFT, GPIO_PFCE1_PFCE19);
    RZA_IO_RegWrite_16(&GPIO.PFC1,  1, GPIO_PFC1_PFC9_SHIFT,  GPIO_PFC1_PFC19);
    RZA_IO_RegWrite_16(&GPIO.PIPC1, 1, GPIO_PIPC1_PIPC9_SHIFT, GPIO_PIPC1_PIPC19);
*/
	/* Set P3_15 as PMOD pin8 (output) Reset for PMOD connectors */
    PORT3.Pn.BIT.Pn15 = 1;
   	RZA_IO_RegWrite_16(&GPIO.PM3,   0, GPIO_PM3_PM315_SHIFT,    GPIO_PM3_PM315);
    RZA_IO_RegWrite_16(&GPIO.PMC3,  0, GPIO_PMC3_PMC315_SHIFT,  GPIO_PMC3_PMC315);
    RZA_IO_RegWrite_16(&GPIO.PFCAE3,0, GPIO_PFCAE3_PFCAE315_SHIFT,GPIO_PFCAE3_PFCAE315);
    RZA_IO_RegWrite_16(&GPIO.PFCE3, 0, GPIO_PFCE3_PFCE315_SHIFT, GPIO_PFCE3_PFCE315);
    RZA_IO_RegWrite_16(&GPIO.PFC3,  0, GPIO_PFC3_PFC315_SHIFT,  GPIO_PFC3_PFC315);
    RZA_IO_RegWrite_16(&GPIO.PIPC3, 0, GPIO_PIPC3_PIPC315_SHIFT, GPIO_PIPC3_PIPC315);


   	/* Enable master mode */
	RSPI1.SPCR |= 0x28u;
	dummy_byte = RSPI1.SPCR;
}

/*******************************************************************************
* Function Name: R_RSPI1_Start
* Description  : This function starts the RSPI1 module operation.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void R_RSPI1_Start(void)
{
    volatile uint8_t dummy = 0u;

    UNUSED_VARIABLE(dummy);

    /* Enable RSPI interrupts */
	R_INTC_Enable(INTC_ID_SPTI4);
	R_INTC_Enable(INTC_ID_SPRI4);

    /* Clear error sources */
    dummy = RSPI1.SPSR;
    RSPI1.SPSR = 0x00U;

    if (0 == RZA_IO_RegRead_8( &(RSPI1.SPSR),
                               RSPIn_SPSR_MODF_SHIFT,
    		                   RSPIn_SPSR_MODF))
	{
		/* test */
		RSPI1.SPCR |= 0x40U;
		dummy = RSPI1.SPCR;
	}
	else
	{
		/* clear the MODF flag then set the SPE bit */
		dummy = RSPI1.SPSR;

		RSPI1.SPCR |= 0x40U;
		dummy = RSPI1.SPCR;
	}
}

/*******************************************************************************
* Function Name: R_RSPI1_Stop
* Description  : This function stops the RSPI1 module operation.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void R_RSPI1_Stop(void)
{
    /* Disable RSPI interrupts */
	R_INTC_Disable(INTC_ID_SPTI4);

    /* Disable RSPI function */
	RZA_IO_RegWrite_8( &(RSPI1.SPCR),
			           0,
			           RSPIn_SPCR_SPE_SHIFT,
			           RSPIn_SPCR_SPE);

}
/*******************************************************************************
* Function Name: R_RSPI1_Send_Receive
* Description  : This function sends and receives CSI#n1 data.
* Arguments    : tx_buf -
*                 transfer buffer pointer (not used when data is handled by DTC)
*                tx_num -
*                    buffer size
*                rx_buf -
*                 receive buffer pointer (not used when data is handled by DTC)
* Return Value : status -
*                    MD_OK or MD_ARGERROR
*******************************************************************************/
uint8_t R_RSPI1_Send_Receive(uint32_t * const tx_buf,
		                              uint16_t tx_num, uint32_t * const rx_buf)
{
	uint8_t status = 0u;

    if (tx_num < 1U)
    {
        status = 1u;
    }
    else
    {
		gp_rspi1_tx_address = tx_buf;
        g_rspi1_tx_count = tx_num;

        gp_rspi1_rx_address = rx_buf;
        g_rspi1_rx_length = tx_num;
        g_rspi1_rx_count = 0U;

        /* Enable transmit interrupt */
    	RZA_IO_RegWrite_8( &(RSPI1.SPCR),
    			           1,
    			           RSPIn_SPCR_SPTIE_SHIFT,
    			           RSPIn_SPCR_SPTIE);

        /* Enable receive interrupt */
    	RZA_IO_RegWrite_8( &(RSPI1.SPCR),
    			           1,
    			           RSPIn_SPCR_SPRIE_SHIFT,
    			           RSPIn_SPCR_SPRIE);

        /* Enable RSPI function */
    	RZA_IO_RegWrite_8( &(RSPI1.SPCR),
    			           1,
    			           RSPIn_SPCR_SPE_SHIFT,
    			           RSPIn_SPCR_SPE);
    }

    return (status);
}

/*******************************************************************************
* Function Name: R_RSPI1_LoopBackReversed
* Description  : This function .
* Arguments    : None
* Return Value : None
*******************************************************************************/
void R_RSPI1_LoopBackReversed(void)
{
	RZA_IO_RegWrite_8( &(RSPI1.SPPCR),
			           (uint8_t)1,
			           RSPIn_SPPCR_SPLP_SHIFT,
			           RSPIn_SPPCR_SPLP);
}

/*******************************************************************************
* Function Name: R_RSPI1_LoopBackDisable
* Description  : This function disable loop-back mode.
* Arguments    : None
* Return Value : None
*******************************************************************************/
void R_RSPI1_LoopBackDisable(void)
{
	RZA_IO_RegWrite_8( &(RSPI1.SPPCR),
			           0,
			           RSPIn_SPPCR_SPLP_SHIFT,
			           RSPIn_SPPCR_SPLP);
}
