/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2014 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name   : sample_riic_command.c
* $Rev: 1121 $
* $Date:: 2014-08-06 17:09:53 +0900#$
* Description : RIIC sample program
*******************************************************************************/


/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include <stdio.h>
#include "r_typedefs.h"
#include "dev_drv.h"                /* Device Driver common header */
#include "devdrv_riic.h"            /* RIIC Driver Header */
#include "sample_riic.h"
#include "command.h"
#include "iodefine.h"
#include "rza_io_regrw.h"

/******************************************************************************
Typedef definitions
******************************************************************************/


/******************************************************************************
Macro definitions
******************************************************************************/
#define SAMPLE_RIIC_DEVICE_CODE (0xA0)  /* EEPROM device code         : b'1010 */
#define SAMPLE_RIIC_DEVICE_ADDR (0x01)  /* EEPROM device address code : b'001  */

#define SAMPLE_RIIC_DATA_SIZE   (128)   /* Data receive RAM size */
#define SAMPLE_RIIC_PAGE_SIZE   (8)    /* Maximum write in page write mode */

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/


/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/


/******************************************************************************
Private global variables and functions
******************************************************************************/
static void    RIIC_InitMemory8bit(uint8_t * addr, uint8_t data, uint32_t count, uint8_t increment);
static int32_t RIIC_VerifyMemory8bit(uint8_t * addr_1, uint8_t * addr_2, uint32_t count);

#ifdef __CC_ARM
#pragma arm section zidata = "BSS_RIIC_SAMPLE"
/* Transmit data buffer */
static uint8_t  riic_trans_data[SAMPLE_RIIC_DATA_SIZE];
/* Receive data buffer  */
static uint8_t  riic_receive_data[SAMPLE_RIIC_DATA_SIZE];
#pragma arm section zidata
#endif
#ifdef __ICCARM__
/* Transmit data buffer */
static uint8_t  riic_trans_data[SAMPLE_RIIC_DATA_SIZE]   @ "BSS_RIIC_SAMPLE";
/* Receive data buffer  */
static uint8_t  riic_receive_data[SAMPLE_RIIC_DATA_SIZE] @ "BSS_RIIC_SAMPLE";
#endif
#ifdef __GNUC__
/* Transmit data buffer */
//static uint8_t  riic_trans_data[SAMPLE_RIIC_DATA_SIZE]   __attribute__ ((section ("BSS_RIIC_SAMPLE")));
/* Receive data buffer  */
//static uint8_t  riic_receive_data[SAMPLE_RIIC_DATA_SIZE] __attribute__ ((section ("BSS_RIIC_SAMPLE")));
/* Transmit data buffer */
static uint8_t  riic_trans_data[SAMPLE_RIIC_DATA_SIZE];
/* Receive data buffer  */
static uint8_t  riic_receive_data[SAMPLE_RIIC_DATA_SIZE];

#endif


/******************************************************************************
* Function Name: Sample_RIIC_EepAccess
* Description  : In the sample code, the transfer speed is set to 100kbps, and 
*              : the data of 64 bytes allocated to the write space is written 
*              : to the EEPROM. Then waits until writing has been completed 
*              : inside the EEPROM using the acknowledge polling processing.
*              : Reads the data of 128 bytes from the EEPROM and writes it into
*              : the large-capacity on-chip RAM space.
* Arguments    : int32_t   argc : The number of command arguments input from 
*              :                : the terminal. Not used in this function.
*              : char_t ** argv : Pointer to the command input from the terminal.
*              :                : Not used in this function.
* Return Value : COMMAND_SUCCESS : Success of RIIC sample code processing
*              : COMMAND_ERROR   : Failure of RIIC sample code processing
******************************************************************************/
int32_t Pekoe_Sample_RIIC_EepTest(int32_t argc, char_t ** argv)
{
    int32_t  ret;
    uint8_t  w_adr,i;

    printf("EEPROM write test start\r\n");

    /* ==== Memory initialization ==== */
    RIIC_InitMemory8bit(riic_trans_data, 0x00, SAMPLE_RIIC_DATA_SIZE, 1);   /* Generate transmit data    */
    RIIC_InitMemory8bit(riic_receive_data, 0x00, SAMPLE_RIIC_DATA_SIZE, 0); /* Clear receive data buffer */
    w_adr=0;
    for(i=SAMPLE_RIIC_DATA_SIZE/SAMPLE_RIIC_PAGE_SIZE;i!=0;i--)
		{
    	/* ==== Write to EEPROM ==== */
    	ret = Sample_RIIC_EepWrite(DEVDRV_CH_0,                 /* RIIC channel        */
                               SAMPLE_RIIC_DEVICE_CODE, /* Device code         */
                               SAMPLE_RIIC_DEVICE_ADDR, /* Device address code */
                               w_adr,                  	/* Write start address */
                               SAMPLE_RIIC_PAGE_SIZE,   /* Write data size     */
                               &riic_trans_data[w_adr]);        /* Write data          */
    	/* When the NACK is received at the time of data transmission */
    	if (DEVDRV_ERROR_RIIC_NACK == ret)
    		{
    		printf("error: EEPROM write error (NACK).\n");
    		printf("\n");
    		return COMMAND_ERROR;
    		}

    	/* ==== Wait until writing has been completed inside the EEPROM ==== */
    	Sample_RIIC_EepAckPolling(DEVDRV_CH_0, SAMPLE_RIIC_DEVICE_CODE, SAMPLE_RIIC_DEVICE_ADDR);
    	w_adr +=SAMPLE_RIIC_PAGE_SIZE;
		}
    /* ==== Read from EEPROM ==== */
    ret = Sample_RIIC_EepRead(DEVDRV_CH_0,                  /* RIIC channel        */
                              SAMPLE_RIIC_DEVICE_CODE,  /* Device code         */
                              SAMPLE_RIIC_DEVICE_ADDR,  /* Device address code */
                              0x0000,                   /* Read start address  */
                              SAMPLE_RIIC_DATA_SIZE,    /* Read data size      */
                              riic_receive_data);       /* Read data           */
    /* When the NACK is received at the time data transmission */
    if (DEVDRV_ERROR_RIIC_NACK == ret)
    {
        printf("error: EEPROM read error (NACK).\n");
        printf("\n");
        return COMMAND_ERROR;
    }



	printf("EEPROM write and EEPROM read complete.\n");
    printf("\n");

    /* ==== Verify ==== */
    ret = RIIC_VerifyMemory8bit(riic_trans_data, riic_receive_data, SAMPLE_RIIC_DATA_SIZE);
    if (-1 == ret)
    {
        printf("error: Verify error.\n");
        printf("\n");
        return COMMAND_ERROR;
    }

    printf("EEPROM Verify complete and OK.\n");
    printf("\n");

    return COMMAND_SUCCESS;
}


/******************************************************************************
* Function Name: RIIC_InitMemory8bit
* Description  : Writes the value specified by the argument data in the 8-bit 
*              : units from the address specified by the argument *addr for 
*              : the times specified by the argument count. 
*              : When the value other than 0 is set for the argument increment, 
*              : increment the argument data every time writing to the memory.
* Arguments    : uint8_t * addr    : Start address of the memory for writing
*              : uint8_t data      : Data to be written
*              : uint32_t count    : Writing count (in the 8-bit units)
*              : uint8_t increment : Increment setting for write data
*              :                       0 : Write the data specified by the 
*              :                         : argument data
*              :            Other than 0 : Write the data incremented from 
*              :                         : the argument data
* Return Value : none
******************************************************************************/
static void RIIC_InitMemory8bit(uint8_t * addr, uint8_t data, uint32_t count, uint8_t increment)
{
    uint32_t offset;

    for (offset = 0; offset < count; offset++)
    {
        if (0 == increment)
        {
            *(addr + offset) = data;
        }
        else
        {
            *(addr + offset) = data++;
        }
    }
}

/******************************************************************************
* Function Name: RIIC_VerifyMemory8bit
* Description  : Compares the area with the size specified by the argument 
*              : count starting with the address specified by the argument 
*              : *addr_1 and the area starting with the address specified by 
*              : the argument *addr_2. Reads it in the8-bit units, and sets 
*              : the compare result for the return value.
* Arguments    : uint8_t * addr_1 : Start address of the memory for comparison
*              : uint8_t * addr_2 : Start address of the memory for comparison
*              : uint32_t count   : Comparison count (in the 8-bit units)
* Return Value :  0 : Verify OK
*              : -1 : Verify Error
******************************************************************************/
static int32_t RIIC_VerifyMemory8bit(uint8_t * addr_1, uint8_t * addr_2, uint32_t count)
{
    uint32_t offset;
    int32_t  ret;

    ret = 0;

    for (offset = 0; offset < count; offset++)
    {
        if (*(addr_1 + offset) != *(addr_2 + offset))
        {
            ret = -1;   /* Verify error */
            break;
        }
    }

    return ret;
}


/* End of File */

