/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2013 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/**************************************************************************//**
* @file         r_vdc5_shared_param.h
* @version      0.10
* @brief        
******************************************************************************/

#ifndef R_VDC5_SHARED_PARAM_H
#define R_VDC5_SHARED_PARAM_H

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include    "r_vdc5.h"
#include    "r_vdc5_user.h"


/******************************************************************************
Macro definitions
******************************************************************************/

/******************************************************************************
Typedef definitions
******************************************************************************/
/*! Color space */
typedef enum
{
    VDC5_COLOR_SPACE_GBR    = 0,    /*!< GBR */
    VDC5_COLOR_SPACE_YCBCR  = 1     /*!< YCbCr */
} vdc5_color_space_t;

/*! Resource state */
typedef enum
{
    VDC5_RESOURCE_ST_INVALID    = 0,    /*!< Invalid */
    VDC5_RESOURCE_ST_VALID      = 1     /*!< Valid */
} vdc5_resource_state_t;

/*! Resource state */
typedef enum
{
    VDC5_RESOURCE_PANEL_CLK = 0,        /*!< Panel clock */
    VDC5_RESOURCE_VIDEO_IN,             /*!< Input video */
    VDC5_RESOURCE_VSYNC,                /*!< Vsync signal */
    VDC5_RESOURCE_LCD_PANEL,            /*!< LCD panel (output video) */
    VDC5_RESOURCE_LVDS_CLK,             /*!< LVDS PLL clock */
    VDC5_RESOURCE_NUM
} vdc5_resource_type_t;


/******************************************************************************
Functions Prototypes
******************************************************************************/
void VDC5_ShrdParam_Init(const vdc5_channel_t ch);

void VDC5_ShrdParam_SetInitParam(const vdc5_channel_t ch, const vdc5_init_t * const param);
void VDC5_ShrdParam_SetTerminate(const vdc5_channel_t ch);
void VDC5_ShrdParam_SetInputParam(const vdc5_channel_t ch, const vdc5_input_t * const param);
void VDC5_ShrdParam_SetSyncParam(const vdc5_channel_t ch, const vdc5_sync_ctrl_t * const param);
void VDC5_ShrdParam_SetOutputParam(const vdc5_channel_t ch, const vdc5_output_t * const param);
void VDC5_ShrdParam_SetWriteParam(
    const vdc5_channel_t        ch,
    const vdc5_scaling_type_t   scaling_id,
    const vdc5_write_t  * const param);
void VDC5_ShrdParam_SetChgWriteParam(
    const vdc5_channel_t            ch,
    const vdc5_scaling_type_t       scaling_id,
    const vdc5_write_chg_t  * const param);
void VDC5_ShrdParam_SetReadParam(
    const vdc5_channel_t        ch,
    const vdc5_graphics_type_t  graphics_id,
    const vdc5_read_t   * const param);
void VDC5_ShrdParam_SetChgReadParam(
    const vdc5_channel_t            ch,
    const vdc5_graphics_type_t      graphics_id,
    const vdc5_read_chg_t   * const param);
void VDC5_ShrdParam_SetCascade(const vdc5_channel_t ch, const vdc5_onoff_t cascade);
void VDC5_ShrdParam_SetUndSel(const vdc5_channel_t ch, const vdc5_onoff_t und_sel);
void VDC5_ShrdParam_SetGr0Scaler(const vdc5_channel_t ch, const vdc5_onoff_t en_gr0_scaler);

vdc5_panel_clksel_t VDC5_ShrdParam_GetPanelClkSel(const vdc5_channel_t ch);
vdc5_onoff_t VDC5_ShrdParam_GetLvdsClkRef(void);
vdc5_color_space_t VDC5_ShrdParam_GetColorSpace(const vdc5_channel_t ch);
vdc5_input_sel_t VDC5_ShrdParam_GetInputSelect(const vdc5_channel_t ch);
vdc5_res_vs_in_sel_t VDC5_ShrdParam_GetVsInSel(const vdc5_channel_t ch);
vdc5_onoff_t VDC5_ShrdParam_GetCascade(const vdc5_channel_t ch);
vdc5_onoff_t VDC5_ShrdParam_GetUndSel(const vdc5_channel_t ch);
vdc5_onoff_t VDC5_ShrdParam_GetGr0Scaler(const vdc5_channel_t ch);
uint32_t VDC5_ShrdParam_GetBgColor(const vdc5_channel_t ch, const vdc5_color_space_t color_space);
vdc5_wr_md_t VDC5_ShrdParam_GetWritingMode(const vdc5_channel_t ch, const vdc5_scaling_type_t scaling_id);
vdc5_res_inter_t VDC5_ShrdParam_GetInterlace(const vdc5_channel_t ch, const vdc5_scaling_type_t scaling_id);
vdc5_color_space_t VDC5_ShrdParam_GetColorSpaceFbWr(const vdc5_channel_t ch, const vdc5_scaling_type_t scaling_id);
void * VDC5_ShrdParam_GetFrBuffBtm(const vdc5_channel_t ch, const vdc5_scaling_type_t scaling_id);

vdc5_gr_flm_sel_t VDC5_ShrdParam_GetSelFbAddrSig(const vdc5_channel_t ch, const vdc5_graphics_type_t graphics_id);
vdc5_gr_format_t VDC5_ShrdParam_GetGraphicsFormat(const vdc5_channel_t ch, const vdc5_graphics_type_t graphics_id);
vdc5_color_space_t VDC5_ShrdParam_GetColorSpaceFbRd(const vdc5_channel_t ch, const vdc5_graphics_type_t graphics_id);
vdc5_onoff_t VDC5_ShrdParam_GetMeasureFolding(const vdc5_channel_t ch, const vdc5_graphics_type_t graphics_id);
vdc5_period_rect_t * VDC5_ShrdParam_GetDisplayArea(const vdc5_channel_t ch, const vdc5_graphics_type_t graphics_id);
vdc5_width_read_fb_t * VDC5_ShrdParam_GetFrBuffWidth_Rd(
    const vdc5_channel_t        ch,
    const vdc5_graphics_type_t  graphics_id);
vdc5_channel_t VDC5_ShrdParam_GetLvdsCh(void);

void VDC5_ShrdParam_SetResource(
    const vdc5_channel_t        ch,
    const vdc5_resource_type_t  rsrc_type,
    const vdc5_resource_state_t rsrc_state);
void VDC5_ShrdParam_SetLayerResource(
    const vdc5_channel_t        ch,
    const vdc5_layer_id_t       layer_id,
    const vdc5_resource_state_t rsrc_state);
vdc5_resource_state_t VDC5_ShrdParam_GetResource(const vdc5_channel_t ch, const vdc5_resource_type_t rsrc_type);
vdc5_resource_state_t VDC5_ShrdParam_GetLayerResource(const vdc5_channel_t ch, const vdc5_layer_id_t layer_id);

void VDC5_ShrdParam_SetRwProcEnable(const vdc5_channel_t ch, const vdc5_layer_id_t layer_id);
void VDC5_ShrdParam_SetRwProcDisable(const vdc5_channel_t ch, const vdc5_layer_id_t layer_id);
vdc5_resource_state_t VDC5_ShrdParam_GetRwProcReady(const vdc5_channel_t ch, const vdc5_layer_id_t layer_id);
vdc5_resource_state_t VDC5_ShrdParam_GetRwProcEnabled(const vdc5_channel_t ch, const vdc5_layer_id_t layer_id);
vdc5_resource_state_t VDC5_ShrdParam_GetOirRwProcEnabled(const vdc5_channel_t ch);


#endif  /* R_VDC5_SHARED_PARAM_H */

