/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2014 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/******************************************************************************
* File Name     : GNU_DS-5-IRAM.ld
* Device(s)     : RZ/A1H (R7S721001)
* Tool-Chain    : GNUARM-RZv13.01-EABI
* H/W Platform  : RSK+RZA1H CPU Board
* Description   : Linker file for projects that require to run from 
*               : Internal device RAM 
******************************************************************************/
/******************************************************************************
* History       : DD.MM.YYYY Version Description
*               : 18.06.2013 1.00
*               : 21.03.2014 2.00    Fixed RAM0L origin address. 
******************************************************************************/
OUTPUT_FORMAT("elf32-littlearm", "elf32-bigarm", "elf32-littlearm")
OUTPUT_ARCH(arm)
ENTRY(start)

/* Base Address RAM Memory Table 3-Mbyte on-chip RAM */  
/* Total RAM available 10112K */
MEMORY 
{
	/* Internal RAM address range H'2000_0000 to H'2001_FFFF is configured as data retention RAM */
	/* Write access to this address range has to be enabled by writing to registers SYSCR1 and SYSCR2 */
	/* RAM0L (rwx)   : ORIGIN = 0x20020000, LENGTH = 0x00060000	*/  /* Page 0 Lower bank (1024KB) H'20000000 to H'200FFFFF */
	RAM1L (rwx)   : ORIGIN = 0x20080000, LENGTH = 0x00080000  /* Page 1 Upper bank (1024KB) H'20100000 to H'201FFFFF */
	RAM2L (rwx)   : ORIGIN = 0x20100000, LENGTH = 0x00080000  /* Page 2 Upper bank (1024KB) H'20200000 to H'202FFFFF */
	RAM3L (rwx)   : ORIGIN = 0x20180000, LENGTH = 0x00080000  /* Page 3 Upper bank (1024KB) H'20300000 to H'203FFFFF */
	RAM4L (rwx)   : ORIGIN = 0x20200000, LENGTH = 0x00100000  /* Page 4 Upper bank (1024KB) H'20400000 to H'204FFFFF */

	/* RAM area for user Program */
	/* RAM01L (rwx) : ORIGIN = 0x20020000, LENGTH = 0x00060000	*/
	RAM0L (rwx) : ORIGIN = 0x20020000, LENGTH = 0x000E0000

	/* VRAM      (rw)  : ORIGIN = 0x60400000, LENGTH = 0x00600000	*/
	/* VRAM      (rw)  : ORIGIN = 0x60100000, LENGTH = 0x00200000	*/
	VRAM      (rw)  : ORIGIN = 0x60080000, LENGTH = 0x00280000
}

PROGRAM_STACK_SIZE    = 0x4000; 	/* Application stack                  */
IRQ_STACK_SIZE	      = 0x1000;     /* IRQ mode stack                     */
FIQ_STACK_SIZE	      = 0x1000;     /* FRQ mode stack                     */
SVC_STACK_SIZE	      = 0x1000;     /* SVC mode stack                     */
ABT_STACK_SIZE	      = 0x1000;     /* ABT mode stack                     */
TTB_SIZE              = 0x4000;     /* Level-1 Translation Table for MMU  */

STACK_BASE            = 0x20074000; /* Stacks and TTB                     */
/* STACK_BASE            = 0x200E8000;	*/ /* Stacks and TTB                     */
/* STACK_BASE            = 0x20700000;	*/ /* Stacks and TTB                     */

EXEC_BASE             = 0x20040000; /* VECTOR_TABLE located here          */
QSPI_BASE             = 0x18080000; /* VECTOR_TABLE located here */	
  
MIRRORED_VECTOR_TABLE = 0x60040000; /* VECTOR_MIRROR_TABLE located here   */ 

SECTIONS
{
	.reset EXEC_BASE : AT ( QSPI_BASE )
	{
		execute = .;
		*start.o (.text)
		.	= ALIGN(0x4);
		*(.text)
		*(.text.startup)
	} > RAM0L

	.rodata EXEC_BASE + SIZEOF (.reset) : AT ( QSPI_BASE + SIZEOF (.reset))
	{
		*(.rodata)
		*(.rodata.*)
		.	= ALIGN(0x8);
		address_start_data_ROM = .;
	    *(.data)
		*(.data.*)
		*(.got.plt) *(.got)
		.	= ALIGN(0x8);
		address_end_data_ROM = .;
         _end = .;
         PROVIDE(end = .); 
	} > RAM0L

	.data  EXEC_BASE + SIZEOF (.reset) + SIZEOF (.rodata) : 
	{
		address_start_data_RAM = .;
		.  += (address_end_data_ROM - address_start_data_ROM);
		address_end_data_RAM = .;
    } > RAM0L

	.bss (NOLOAD) :
	{
         _bss = .;
         PROVIDE(__bss_start__ = .);
         *(.bss)
         *(.bss.**)
         *(COMMON)
		 . = ALIGN(0x4);
         PROVIDE(__bss_end__ = .);
         _ebss = .;
    } >RAM0L
		

	.irq_stack STACK_BASE (NOLOAD) :
	{
		irq_stack_start = .;
		.  += IRQ_STACK_SIZE;
		.	= ALIGN(0x4);
		irq_stack_end = .;
	} >RAM0L

	.fiq_stack (NOLOAD) :
	{
		fiq_stack_start = .;
		.  += FIQ_STACK_SIZE;
		.	= ALIGN(0x4);
		fiq_stack_end = .;
	} >RAM0L

	.svc_stack (NOLOAD) :
	{
		PROVIDE(svc_stack_start = .);
		.  += SVC_STACK_SIZE;
		.	= ALIGN(0x4);
		svc_stack_end = .;
	} >RAM0L

	.abt_stack (NOLOAD) :
	{
		abt_stack_start = .;
		.  += ABT_STACK_SIZE;
		.	= ALIGN(0x4);
		abt_stack_end = .;
	} >RAM0L

	.program_stack (NOLOAD) :
	{
		program_stack_start = .;
		.  += PROGRAM_STACK_SIZE;
		.	= ALIGN(0x4);
		program_stack_end = .;
	} >RAM0L

	/* L1 translation table must be aligned to 16KB Boundary!           */
	/* Please refer to Cortex-A Series Version: 4.0 Programmers Guide, */
	/* section 9.4 First level address translation                      */
	.ttb_mmu1 (NOLOAD) : ALIGN(0x4000)
	{
		ttb_mmu1_base = .;
		.  += TTB_SIZE;
		.	= ALIGN(0x4);
		ttb_mmu1_end = .;
	} >RAM0L

	/* NOLOAD directs linker NOT to fill VRAMx_SECTION with 0.          */
	/* Usage of NOLOAD increases speed of linker and download to target */
	/* .VRAM_SECTION  0x60400000 (NOLOAD) : {} > VRAM */
	.VRAM_SECTION0 0x60080000 (NOLOAD) : {} > VRAM
	.VRAM_SECTION1 0x60100000 (NOLOAD) : {} > VRAM
	.VRAM_SECTION2 0x60200000 (NOLOAD) :
	{
		//*(.text)
	} > VRAM
	/* .BSS_DMAC_SAMPLE_SDRAM    (NOLOAD) : {} > SDRAM */
}