/******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2014 Renesas Electronics Corporation. All rights reserved.
******************************************************************************/
/******************************************************************************
* File Name     : main.c
* Device(s)     : RZ/A1L (3MB RAM)
* Tool-Chain    : KPIT GNUARM-NONE-EABI Toolchain
* H/W Platform  : STREAM-IT (Pekoe)
* Author		: S.Walger
* Description   : This is the RZ/A1L low power mode demontrator main code
* Operation     : Please see the Description.txt file in the doc folder.
* Limitations   : Please see the Description.txt file in the doc folder.
******************************************************************************/
/******************************************************************************
* History       : DD.MM.YYYY Version Description
*               : 24.07.2015 1.00 / S.Walger / Project Clean-up
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
/* System header */
#include <stdio.h>
/* Renesas data types header */
#include "r_typedefs.h"
#include "compiler_settings.h"
#include "iodefine.h"
/* INTC driver header */
#include "devdrv_intc.h"
/* Low level register read/write header */
#include "rza_io_regrw.h"

#include "board.h"
#include "switch_pekoe.h"
#include "l2_cache_pl310.h"
#include "l2_cache.h"

/******************************************************************************
Typedef definitions
******************************************************************************/

/******************************************************************************
Macro definitions
******************************************************************************/

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/
void LedInit(void);
unsigned char rza_mode = ACTIVE;
unsigned long cpt = 0;
unsigned long tempo_button = 0;
unsigned int dummy;

/******************************************************************************
* Function Name: main
* Description  : Initialises the led on Pekoe board and calls function Graphics_GraphicsSample.
* Arguments    : none
* Return Value : 0
******************************************************************************/
int_t main(void)
{
	/* ==== LED initial setting ==== */
	LedInit();

    /* ==== USER SWITCH initial setting ==== */
    Init_Switch_Pekoe();

    PORT7.Pn.BIT.Pn8 = LED_ON;

	while(1)
	{
		if (g_switch_press_flg == SWITCH_USER_SET_FLG)
		{
			if ((rza_mode == 0) || (rza_mode == 2) || (rza_mode == 4))
			{
				rza_mode++;
				PORT7.Pn.BIT.Pn8 = LED_OFF;

				switch (rza_mode)
				{
					case SLEEP_MODE:
					{
						__asm__("nop");
						CPG.STBCR1 &= 0x7F;	/* 0x: Executing WFI instruction puts chip into sleep mode. */
						__asm__("wfi");
						break;
					}
					case SOFTWARE_STANDBY_MODE:
					{
						setL2CachePowerCtrl(0x0001);	/* 1. Set the standby_mode_en bit of the power control register in the PL310 to 1. For the details of this register, see CoreLink Level 2 Cache Controller L2C-310 Technical Reference Manual issued by ARM Ltd. */
						WDT.WTCSR &= 0xDF;				/* 2. Clear the TME bit in the watchdog timer control/status register (WTCSR) of the watchdog timer to 0 to stop the watchdog timer. */
						WDT.WTCNT = 0;					/* 3. Set the timer counter (WTCNT) of the watchdog timer to 0 and the CKS[2:0] bits in WTCSR to appropriate values to secure the specified oscillation settling time. */
						WDT.WTCSR |= 0x07;	/* CKS : Overflow Cycle = 125ms */
						CPG.STBCR1 = 0x80;				/* 4. After setting the STBY and DEEP bits in STBCR1 to 1 and 0 respectively, read STBCR. Then, execute a WFI instruction. */
						__asm__("wfi");
						break;
					}
					case DEEP_STANDBY_MODE:
					{
						setL2CachePowerCtrl(0x0001);	/* 1. Set the standby_mode_en bit of the power control register in the PL310 to 1. For the details of this register, see CoreLink Level 2 Cache Controller L2C-310 Technical Reference Manual issued by ARM Ltd. */
						CPG.RRAMKP = 0x01;				/* 2. The contents of the on-chip data-retention RAM Storage Area 0 are retained in deep standby mode */
						CPG.DSCTR = 0x40;				/* 3. The state of the external memory control pins is not retained when returning from deep standby mode. The program is read from the on-chip data-retention RAM. Instruction fetch from H'20000000 */
						CPG.STBCR1 = 0xC0;				/* 4. After setting the STBY and DEEP bits in STBCR1 to 1 and 1 respectively, read STBCR. Then, execute a WFI instruction. */
						dummy = CPG.STBCR1;				/* 5. read STBCR1 register */
						CPG.DSFR = 0x00;				/* 6.Read out the DSFR register after clearing the flag in the DSFR register */
						dummy = CPG.DSFR;
						__asm__("wfi");					/* 7. execute the WFI instruction */
						break;
					}
					default:
					{
						break;
					}
				}
				__asm__("nop");
				__asm__("nop");
				__asm__("nop");
				__asm__("nop");
				PORT7.Pn.BIT.Pn8 = LED_ON;
				rza_mode++;
			}
			tempo_button = 0;
			while(tempo_button < TEMPO_BUTTON) tempo_button++;
			g_switch_press_flg = 0;
		}
		if ((rza_mode == 0) || (rza_mode == 2) || (rza_mode == 4))
		{
			cpt++;
			if (cpt > TEMPO_LED_BLINKING)
			{
				cpt = 0;
				PORT7.Pn.BIT.Pn8 = ~PORT7.Pn.BIT.Pn8;
			}
		}
	}

    while(1)
    {
    	/* Program should not exit */
		__asm__("nop");
    }

    return (int8_t)0;
}
/******************************************************************************
* End of Function main
******************************************************************************/

/*******************************************************************************
* Function Name: led_init
* Description  : Executes initial setting for the PORT which is connected to
*              : the LEDs on the CPU board.
* Arguments    : none
* Return Value : none
*******************************************************************************/
void LedInit(void)
{
	//Configure LED (P7_8)
	PORT7.PMCn.BIT.PMCn8 = 0;
	PORT7.PIPCn.BIT.PIPCn8 = 0;
	PORT7.PMn.BIT.PMn8 = 0;
	//Clear LED
	PORT7.Pn.BIT.Pn8 = 1;
}

/* End of File */
