/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2013 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name   : spibsc_init1.c
* $Rev: 914 $
* $Date:: 2014-04-09 13:25:35 +0100#$
* Description : Flash boot setting
*******************************************************************************/


/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include "r_typedefs.h"
#include "iodefine.h"
#include "spibsc.h"
#include "rza_io_regrw.h"

/******************************************************************************
Typedef definitions
******************************************************************************/

/******************************************************************************
Macro definitions
******************************************************************************/

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/
void spibsc_init1(void);

/******************************************************************************
* Function Name: spibsc_init1
* Description  : This is the first code that runs in RAM following the load.
*                It stops the QPSI interface, reconfigures it to high speed
*                and then re-starts the access for the copy of the main program
*                loader.
* Arguments    : none
* Return Value : none
******************************************************************************/
void spibsc_init1(void)
{
    /* Temporary (unused) variable for synchronisation read. */
    volatile uint16_t temp = 0;

    /* Used to suppress the 'variable declared but not used' warning */
    UNUSED_VARIABLE(temp);

    /* Release pin function for memory control without changing pin state. */
    if(RZA_IO_RegRead_16((uint16_t *)&CPG.DSFR, CPG_DSFR_IOKEEP_SHIFT,  CPG_DSFR_IOKEEP) == 1)
    {
        RZA_IO_RegWrite_16((uint16_t *)&CPG.DSFR, 0, CPG_DSFR_IOKEEP_SHIFT,  CPG_DSFR_IOKEEP);
        /* Perform a read after the write for synchronisation. */
        temp = RZA_IO_RegRead_16((uint16_t *)&CPG.DSFR, CPG_DSFR_IOKEEP_SHIFT,  CPG_DSFR_IOKEEP);
    }

    /* SPIBSC stop accessing the SPI in bus mode */
    RZA_IO_RegWrite_32(&SPIBSC0.DRCR, 1, SPIBSC_DRCR_SSLN_SHIFT, SPIBSC_DRCR_SSLN);
    /* Wait for the setting to be accepted. */
    while(1)
    {
        if(RZA_IO_RegRead_32(&SPIBSC0.CMNSR, SPIBSC_CMNSR_SSLF_SHIFT, SPIBSC_CMNSR_SSLF) == SPIBSC_SSL_NEGATE)
        {
            break;
        }
    }

    /* Wait for the final transfers to end (TEND=1) for setting change in SPIBSC. */
    while(RZA_IO_RegRead_32(&SPIBSC0.CMNSR, SPIBSC_CMNSR_TEND_SHIFT, SPIBSC_CMNSR_TEND) != SPIBSC_TRANS_END)
    {
        /* wait for transfer-end */
    }

    SPIBSC0.SSLDR = 0; /* The next access delayed setting : 1SPBCLK   */
                       /* SPBSSL negate delayed setting   : 1.5SPBCLK */
                       /* Clock delayed setting           : 1SPBCLK   */

    /* bitrate setting register(SPBCR)  33.33[MHz] */
    RZA_IO_RegWrite_32(&SPIBSC0.SPBCR, 0, SPIBSC_SPBCR_BRDV_SHIFT, SPIBSC_SPBCR_BRDV);
    RZA_IO_RegWrite_32(&SPIBSC0.SPBCR, 2, SPIBSC_SPBCR_SPBR_SHIFT, SPIBSC_SPBCR_SPBR);

    /* Data read control register(DRCR) set to enable the Read Cache */
    RZA_IO_RegWrite_32(&SPIBSC0.DRCR, 1, SPIBSC_DRCR_RBE_SHIFT, SPIBSC_DRCR_RBE);
}


/* End of File */
