/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer*
* Copyright (C) 2013 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/**************************************************************************//**
* @file         ceu_sample.h
* @version      0.04
* $Rev: 11 $
* $Date:: 2013-09-05 15:58:35 +0900#$
* @brief        CEU Sample function header
******************************************************************************/

#ifndef CEU_SAMPLE_H
#define CEU_SAMPLE_H

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include    "r_typedefs.h"


/******************************************************************************
Typedef definitions
******************************************************************************/
/* Camera module(MT9V024) specific (Don't change) */
#define CEU_CONFIG_CAPTURE_SCREEN_HSYNC_NUM_PXL     (320u)//(640u)  /** Active pixels(752H) for Camera module(MT9V024). */
#define CEU_CONFIG_CAPTURE_SCREEN_VSYNC_NUM_PXL     (240u)//(480u)   /** Active pixels(480V) for Camera module(MT9V024). */

#define CEU_INTERRUPT_PRIORITY                      (5u)     /** interrupt priority [0-15]. */
#define CAPTURE_END                                 (1u)     /** 1 frame capture end flag. */

/******************************************************************************
Macro definitions
******************************************************************************/
/* capture size width x 2[byte/pixel],  32 byte aligned */
#define CEU_CAPTUER_BUFF_STRIDE    (( (CEU_CONFIG_CAPTURE_SCREEN_HSYNC_NUM_PXL) + 31u ) & ~0x001Fu )
                                                                                   /* Width of Capture buffer. */
#define CEU_CAPTUER_BUFF_HEIGHT    CEU_CONFIG_CAPTURE_SCREEN_VSYNC_NUM_PXL         /* Height of Capture buffer. */

#define FLAG_VD_SYNC_CEU	PORT7.Pn.BIT.Pn1
#define FLAG_HD_SYNC_CEU	PORT1.Pn.BIT.Pn3

#define PCLK_CAMERA	PORT10.PPRn.BIT.PPRn0
#define HD_CAMERA	PORT10.PPRn.BIT.PPRn2
/******************************************************************************
Variable Externs
******************************************************************************/

/******************************************************************************
Functions Prototypes
******************************************************************************/
void CEU_SAMPLE_CeuSingleCaptureInit( void );
void CEU_SAMPLE_CeuSingleCaptureTerm( void );
void CEU_SAMPLE_CeuSingleCapture( const uint16_t * const pCapBuff );
void CEU_SAMPLE_CeuConvertbitRgbData( const uint16_t * const pSrcBuff, uint8_t * const pDstBuff, const uint16_t stride );

#endif  /* CEU_SAMPLE_H */
