/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer*
* Copyright (C) 2013 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/**************************************************************************//**
* @file         r_ceu.c
* @version      0.04
* $Rev: 11 $
* $Date:: 2013-09-05 15:58:35 +0900#$
* @brief        CEU driver API functions
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/

#include "r_ceu_interrupt.h"
#include "r_ceu_user_config.h"
#include "iodefine.h"
#include "rza_io_regrw.h"
#include "camera.h"
#include "r_ceu.h"
#include "compiler_settings.h"


/******************************************************************************
Typedef definitions
******************************************************************************/
#define NULL 0

/******************************************************************************
Macro definitions
******************************************************************************/
#define STBCR6_INIT_VALUE      (0xBFu)
#define SWRSTCR3_INIT_VALUE    (0xFDu)
#define CAPSR_INIT_VALUE       (0x00010000u)        //(0x00010000u)
#define CEIER_INIT_VALUE       (0x0000000u)
#define CETCR_INIT_VALUE       (0x00000000u)        //(0x00000000u)
#define CRCNTR_INIT_VALUE      (0x0000u)            //(0x0000u)
#define CAPCR_INIT_VALUE       (0x00000000u)        //(0x00000000u)(0x00010000)
#define CAMCR_INIT_VALUE       (0x00000020u)        //(0x00000020u)
#define CMCYR_INIT_VALUE       (0x00000000u)        //(0x00000000u)(0x00780138u)(0x00780620u)
#define CAMOR_A_INIT_VALUE     (0x00000000u)
#define CAPWR_A_INIT_VALUE     (0x00000000u)        //(0x00000000u)(0x00780138u)(0x00780620u)
#define CBDSR_A_INIT_VALUE     (0x00000000u)
#define CAIFR_INIT_VALUE       (0x00000000u)
#define CDWDR_A_INIT_VALUE     (0x00000000u)       //(0x00000000u)(0x00000138u)(0x00000620u)
#define CFLCR_A_INIT_VALUE     (0x00000000u)
#define CFSZR_A_INIT_VALUE     (0x00000000u)       //(0x00000000u)(0x00780620u)
#define CLFCR_A_INIT_VALUE     (0x00000000u)
#define CFWCR_INIT_VALUE       (0x00000000u)
#define CDOCR_A_INIT_VALUE     (0x00000010u)       //(0x00000010u)

#define CDAYR2_A_INIT_VALUE    (0x00000000u)       //(0x00000000u)(0x20600000u)
#define CDACR2_A_INIT_VALUE    (0x00000000u)       //(0x00000000u)(0x20612D00u)
#define CDBYR2_A_INIT_VALUE    (0x00000000u)       //(0x00000000u)(0x20625A00u)
#define CDBCR2_A_INIT_VALUE    (0x00000000u)       //(0x00000000u)(0x20638700u)

#define CDACR_A_INIT_VALUE     (0x00000000u)       //(0x00000000u)
#define CDBYR_A_INIT_VALUE     (0x00000000u)       //(0x00000000u)
#define CDBCR_A_INIT_VALUE     (0x00000000u)       //(0x00000000u)

#define CSTSR_CPTON_BIT_STATUS (0x00000001u)
#define CAPSR_CPKIL_BIT_STATUS (0x00010000u)


#define CAPSR_CE_BIT_SET       (0x00000001u)
#define CAPSR_CE_BIT_STATUS    (0x00000001u)

#define BIT_SHIFT_20 (20u)
#define BIT_SHIFT_2  (2u)
#define BIT_SHIFT_1  (1u)
#define BIT_SHIFT_0  (0u)

#define BOUNDARY_CHECK (0x00000003u)


/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/
//uint32_t FrameBufferCDAYR2[STRIDE * HEIGHT+SAFETY];
//uint32_t FrameBufferCDACR2[STRIDE * HEIGHT+SAFETY];
//uint32_t FrameBufferCDBYR2[(STRIDE * HEIGHT+SAFETY)/2];
//uint32_t FrameBufferCDBCR2[(STRIDE * HEIGHT/2+SAFETY)/2];

//uint8_t VRAM_SECTION FrameBufferCDAYR2[STRIDE * HEIGHT + SAFETY];
//uint8_t VRAM_SECTION FrameBufferCDACR2[STRIDE * HEIGHT + SAFETY];
//uint8_t VRAM_SECTION FrameBufferCDBYR2[STRIDE * HEIGHT + SAFETY];
//uint8_t VRAM_SECTION FrameBufferCDBCR2[STRIDE * HEIGHT + SAFETY];

//uint8_t VRAM_SECTION FrameBufferCDAYR[STRIDE * HEIGHT + SAFETY];
//uint8_t FrameBufferCDBYR[STRIDE * HEIGHT + SAFETY];



//VRAM_SECTION stYUVPlanarBuffer FrameBufferA,FrameBufferB;
stYUVPlanarBuffer FrameBufferA __attribute__((aligned(0x4)));
stYUVPlanarBuffer FrameBufferB __attribute__((aligned(0x4)));
//stYUVPlanarBuffer FrameBufferA __attribute__((section(".BSS_VRAM_SECTION"), aligned(32)));
//stYUVPlanarBuffer FrameBufferB __attribute__((section(".BSS_VRAM_SECTION"), aligned(32)));

int y_size, uv_size;

/**************************************************************************//**
 * Function Name: R_CEU_ModuleInit
 * @brief       CEU module pre-initialization.
 *
 *              Description:<br>
 *              In this function, the operations are performed as below.<br>
 *              - The specified user-defined function is executed.<br>
 *              - Clock supply to CEU is enabled.<br>
 * @param[in]   init_func Pointer to the user-defined function<br>
 * @retval      none.
 *****************************************************************************/
void R_CEU_ModuleInit( void (* const init_func)(void) )
{
    uint32_t reg_data;

    /* call back function */
    if ( init_func != NULL )
    {
        init_func();
    }

    /* CEU Clock is enabled
     * bit6 MSTP66 - Controls clock supply to CEU   : set 0(CEU operates)
     */
    reg_data   = (uint32_t)CPG.STBCR6;
    /*reg_data   = RZA_IO_RegRead_32((uint32_t *)&CPG.STBCR6,
        		                     CPG_STBCR6_MSTP66_SHIFT,
        		                     CPG_STBCR6_MSTP66);*/
    reg_data   &= STBCR6_INIT_VALUE;
    CPG.STBCR6 = (uint8_t)reg_data;

    /* Software reset is canceled
     * bit1 SRST31 - Issue the reset to CEU : set 0(CEU operates)
     */
    reg_data     = (uint32_t)CPG.SWRSTCR3;
    reg_data     &= SWRSTCR3_INIT_VALUE;
    CPG.SWRSTCR3 = (uint8_t)reg_data;


    return;
}   /* End of function R_CEU_ModuleInit() */

/**************************************************************************//**
 * Function Name: R_CEU_ConfigInit
 * @brief       CEU module initialization.
 *
 *              Description:<br>
 *              In this function, the operations are performed as below.<br>
 *              - CEU setting for the Camera module as below.<br>
 *              [Camera (APTINA MT9V024) Spec]<br>
 *              - VD/HD signal polarity = High-active.<br>
 *              - Buswidth = 10bit<br>
 *              - Pixel output timing = Data fetch mode (Blank period = 0HD)<br>
 *              - Active pixels = 752H x 480V<br>
 * @retval      none.
 *****************************************************************************/
#if 0
void CEU_DataFetchInit( void )
{
    /* software reset
     * bit16 CPKIL - Write 1 to this bit to perform a software reset of capturing.  : set 1(Software reset of capturing)
     * bit0 CE - capture enable bit    : set 0 (Stops capturing)
     */
    CEU.CAPSR = CAPSR_INIT_VALUE;

    /* wait software reset complete
     * Complete : CSTSR.CPTON = b'0 & CAPSR.CPKIL=b'0
     *
     * [CSTSR]
     * bit0 CPTON - Indicates that the CEU is operating. : 0=Halted, 1=operating
     *
     * [CAPSR]
     * bit16 CPKIL - Write 1 to this bit to perform a software reset of capturing.  : 0=Normal, 1=Software reset
     */
    while ( 0u != (CEU.CSTSR & CSTSR_CPTON_BIT_STATUS))
    {
        /* wait state :"Halted" */
    }

    while ( 0u != (CEU.CAPSR & CAPSR_CPKIL_BIT_STATUS))
    {
        /* wait state :"Normal" */
    }

    /* CEU interrupt disable */
    /* Capture Event Interrupt Enable Register (CEIER)
     * CEIER enables or disables interrupts of the event flag register that generates CEU interrupts.
     * bitX : b'0=disable, b'1=enable
     */
    CEU.CEIER = CEIER_INIT_VALUE;     /* Disable all */

    /* CEU interrupt flag clear */
    /* Capture Event Flag Clear Register (CETCR)
     * bitX : b'0=clear, b'1=hold
     */
    CEU.CETCR = CETCR_INIT_VALUE;     /* clear all */

    /* CEU Register Control Register (CRCNTR)
     * bit4 RVS - Sets the timing to switch the register plane in both-field capture.
                                                 : set 0 (Switches the register plane every 2 VD)
     * bit1 RS - Specifies which register plane is used by the CEU in synchronization with VD.
                                                 : set 0 (Uses plane A of the register)
     * bit0 RC - Specifies switching of the register plane used by the CEU in synchronization with VD.
                                                 : set 0 (Uses the specified register plane in synchronization with VD)
     * other bit is reserved    : write value should always be 0.
     */
    CEU.CRCNTR = (uint16_t)CRCNTR_INIT_VALUE;

    /* capture control register (CAPCR)
     * bit31-24 FDRP[7:0] - the frame drop interval in continuous frame capture. : set H'00 (Single Capture mode)
     * bit21-20 MTCM[1:0] - These bits specify the unit for transferring data to a bus bridge module. : variable.
     * bit16 CTNCP - Continuous capture bit : b'0 (One-frame capture)
     * other bit is reserved    : write value should always be 0.
     *
     * [Note]
     * Continuous capture operations are possible in only image capture mode.
     */
    CEU.CAPCR = CAPCR_INIT_VALUE;

    /* bit[21:20]=b'00. Transferred to the bus in 4-burst 1-transfer(4QWx1) units */
    /* bit[21:20]=b'01. Transferred to the bus in 4-burst 1-transfer(4QWx2) units */
    /* bit[21:20]=b'10. Transferred to the bus in 4-burst 1-transfer(4QWx4) units */
    /* bit[21:20]=b'11. Transferred to the bus in 4-burst 1-transfer(4QWx8) units */
    CEU.CAPCR |= ( CEU_CONFIG_BURST_MODE << BIT_SHIFT_20 ) ;

    /* Capture Interface Control Register (CAMCR)
     * bit16 FLDPOL - Sets the polarity of the field identification signal (FLD) from an external module.
                                                                                : 0(high-active) or 1(low-active)
     * bit12 DTIF - Sets the digital image input pins from which data is to be captured.
                                                                                : b'0 (8bit).
     * bit9-8 DTARY[1:0] - These bits set the input order of the luminance component and chrominance component.
                                                                                : b'00(data fetch mode)
     * bit5-4 JPG[1:0] - These bits select the fetched data type. : b'10 (Data enable fetch mode)
     * bit1 VDPOL - Sets the polarity for detection of the vertical sync signal input from an external module.
                                                                                : 0(high-active) or 1(low-active)
     * bit0 HDPOL - Sets the polarity for detection of the horizontal sync signal input from an external module.
                                                                                : 0(high-active) or 1(low-active)
     * other bit is reserved    : write value should always be 0.
     */
    CEU.CAMCR = CAMCR_INIT_VALUE;         /* FLD is High-active, 16bit, Data enable fetch mode */

    CEU.CAMCR |= ( ( CEU_CONFIG_CAMERA_SIGNAL_POLARITY_VD << BIT_SHIFT_1 ) |    /* VDPOL=b'0 (high-active) */
                  ( CEU_CONFIG_CAMERA_SIGNAL_POLARITY_HD << BIT_SHIFT_0) );   /* HDPOL=b'0 (high-active) */

    /* Capture Interface Cycle Register (CMCYR)
     * Set 0 in all bits of this register, during data enable fetch mode.
     */
    CEU.CMCYR = CMCYR_INIT_VALUE;

    /* Capture Interface Offset Register (CAMOR)
     * This register is not used, during data enable fetch mode.
     */
    CEU.CAMOR_A = CAMOR_A_INIT_VALUE;

    /* Capture Interface Width Register (CAPWR)
     * This register is not used, during data enable fetch mode.
     */
    CEU.CAPWR_A = CAPWR_A_INIT_VALUE;

    /* Capture Bundle Destination Size Register (CBDSR)
     * bit22-0 CBVS[22:0] -
       These bits select the number of lines or number of bytes for output to the memory in a bundle write.
     * other bit is reserved    : write value should always be 0.
     */
    CEU.CBDSR_A  = CBDSR_A_INIT_VALUE;  /* sample program not support bundle write mode */

    /* Capture Interface Input Format Register (CAIFR)
     * this register not used in data fetch mode.
     */
    CEU.CAIFR = CAIFR_INIT_VALUE;     /* not used */

    /* Capture Destination Width Register (CDWDR)
     * this register not used in data fetch mode.
     */
    CEU.CDWDR_A = CDWDR_A_INIT_VALUE;   /* not used */

    /* Capture Filter Control Register (CFLCR)
     * In data fetch mode, set CFLCR to 0.
     */
    CEU.CFLCR_A = CFLCR_A_INIT_VALUE;

    /* Capture Filter Size Clip Register (CFSZR)
     * this register not used in data enable fetch mode.
     */
    CEU.CFSZR_A = CFSZR_A_INIT_VALUE;   /* not used */

    /* Capture Low-Pass Filter Control Register (CLFCR)
     * In data fetch mode, clear the LPF bit to B'0.
     * bit0 LPF - Enables or disables operation of the low-pass filter. : set b'0
     */
    CEU.CLFCR_A = CLFCR_A_INIT_VALUE;

    /* Firewall Operation Control Register (CFWCR)
     * bit31-5 FWV[26:0] - These bits specify the upper limit of a write address.
                                         Specify the upper 27 bits of the 32-bit address. : set 0
     * bit0 FWE - FW enable/disable. : set 0 (disable)
     */
    CEU.CFWCR = CFWCR_INIT_VALUE;

    /* Capture Data Output Control Register (CDOCR)
     * bit16 CBE - Controls the number of lines of captured data to be written to the memory.
                                                            : set b'0(normal write)
     * bit4 CDS - Sets the image format when outputting the image data captured in the YCbCr 422 format to the memory.
                                                            : set b'1 (In data fetch mode, set the CDS bit to B'1.)
     * bit2 COLS - Controls swapping in 32-bit units for data output from the CEU.
     * bit1 COWS - Controls swapping in 16-bit units for data output from the CEU.
     * bit0 COBS - Controls swapping in 8-bit units for data output from the CEU.
     * other bit is reserved    : write value should always be 0.
     */
    CEU.CDOCR_A = CDOCR_A_INIT_VALUE;       /* normal write, Data fetch mode, swap disable */

    CEU.CDOCR_A |= ( ( CEU_CONFIG_32BIT_DATA_SWAP << BIT_SHIFT_2 ) |    /* COLS=b'1 (32bit swap enable) */
                     ( CEU_CONFIG_16BIT_DATA_SWAP << BIT_SHIFT_1 ) |    /* COWS=b'1 (16bit swap enable) */
                     ( CEU_CONFIG_8BIT_DATA_SWAP  << BIT_SHIFT_0) );   /* COBS=b'1 (8bit swap enable)  */

#if 0
    CEU.CDAYR_A = (uint32_t) &FrameBufferA.Y[0];
    CEU.CDACR_A = (uint32_t) &FrameBufferA.UV[0];
    CEU.CDBYR_A = (uint32_t) &FrameBufferB.Y[0];
    CEU.CDBCR_A = (uint32_t) &FrameBufferB.UV[0];
#else
    CEU.CDAYR_A = (uint32_t) &FrameBufferA.YUV[0];
    CEU.CDACR_A = (uint32_t) (&FrameBufferA.UV[0] + y_size);
    CEU.CDBYR_A = (uint32_t) &FrameBufferB.Y[0];
    CEU.CDBCR_A = (uint32_t) (&FrameBufferB.UV[0] + y_size);
#endif

    /* Memory address for Bundle write */
    CEU.CDAYR2_A = CDAYR2_A_INIT_VALUE; /* not used */
    CEU.CDACR2_A = CDACR2_A_INIT_VALUE; /* not used */
    CEU.CDBYR2_A = CDBYR2_A_INIT_VALUE; /* not used */
    CEU.CDBCR2_A = CDBCR2_A_INIT_VALUE; /* not used */

    return;

}   /* End of function R_CEU_ConfigInit() */
#endif
/**************************************************************************//**
 * Function Name: R_CEU_SetMemoryAddress
 * @brief       The function which sets the address of a capture data buffer.
 *
 *              Description:<br>
 *              The address of a capture data buffer is set.<br>
 *              The buffers must be 4 byte boundaries.<br>
 * @param[in]   addr The address of a capture data buffer. The buffers must be 4 byte boundaries.<br>
 * @retval      Error code.
 *****************************************************************************/
ceu_error_t R_CEU_SetMemoryAddress( const uint32_t addr )
{
    ceu_error_t ceu_error;

    /* boundary check */
    if ( (addr & BOUNDARY_CHECK) != 0u )
    {
        ceu_error = CEU_ERROR_PARAM;     /* Not 4Byte boundary */
    }
    else
    {
        CEU.CDAYR_A = addr;
        CEU.CDACR_A = CDACR_A_INIT_VALUE;       /* not used in data fetch. */
        CEU.CDBYR_A = CDBYR_A_INIT_VALUE;       /* not used in data fetch. */
        CEU.CDBCR_A = CDBCR_A_INIT_VALUE;       /* not used in data fetch. */
        ceu_error = CEU_ERROR_OK;
    }

    return ceu_error;

}   /* End of function R_CEU_SetMemoryAddress() */

/**************************************************************************//**
 * Function Name: R_CEU_CallbackISR
 * @brief       CEU driver set interrupt callback function.
 *
 *              Description:<br>
 *              Setting interrupt callback function.<br>
 * @param[in]   callback Pointer to the user-defined function.<br>
 * @retval      none.
 *****************************************************************************/
void R_CEU_CallbackISR( void (*const callback)(uint32_t) )
{
    if( callback != NULL ){
        CEU_Int_SetInterrupt( callback );
    }

    return;

}   /* End of function R_CEU_CallbackISR() */

/**************************************************************************//**
 * Function Name: R_CEU_CaptureStart
 * @brief       The Function which start capture.
 *
 *              Description:<br>
 *              This function start CEU capture.<br>
 * @retval      none.
 *****************************************************************************/
void R_CEU_CaptureStart( void )
{
    /* start capture */
    /* Capture Start Register (CAPSR)
     * bit0 CE - Capture Enable : set b'1 (Starts capturing)
     * other bit is not change.
     */

     //seb
	//CEU.CETCR = 0x00;

	CEU.CAPSR |= CAPSR_CE_BIT_SET;

    return;

}   /* End of function R_CEU_CaptureStart() */


/**************************************************************************//**
* Function Name: CEU_ImageCaptureInit
* @brief       The Function initialise the CEU registers for image capture.
*
*              Description:
*
* @retval      none.
*****************************************************************************/
void CEU_ImageCaptureInit(int videoWidth, int videoHeight)
{
/* software reset
* bit16 CPKIL - Write 1 to this bit to perform a software reset of capturing.  : set 1(Software reset of capturing)
* bit0 CE - capture enable bit    : set 0 (Stops capturing)
*/
CEU.CAPSR = CAPSR_INIT_VALUE;

/* wait software reset complete
* Complete : CSTSR.CPTON = b'0 & CAPSR.CPKIL=b'0
* [CSTSR]
* bit0 CPTON - Indicates that the CEU is operating. : 0=Halted, 1=operating
* [CAPSR]
* bit16 CPKIL - Write 1 to this bit to perform a software reset of capturing.  : 0=Normal, 1=Software reset
*/
while ( 0u != (CEU.CSTSR & CSTSR_CPTON_BIT_STATUS))
    {
        /* wait state :"Halted" */
    }

while ( 0u != (CEU.CAPSR & CAPSR_CPKIL_BIT_STATUS))
    {
        /* wait state :"Normal" */
    }
//Return to normal state
CEU.CAPSR = 0;
/* CEU.CETCR = 0; */
CEU.CETCR = 0x00000000;

/*
CEU.CAPCR.FDRP = 0; //No frame drop
CEU.CAPCR.MTCM = 0; //Y data and C data are transferred in 32-byte units
CEU.CAPCR.CTNCP = 0; //Single frame capture*/
CEU.CAPCR = 0x00000000;

/*
CEU.CAMCR.FLDPOL = 0; //FLD signal is high-active, the field is detected as the top field
CEU.CAMCR.DTIF = 0; //Data input to 8-bit digital image input pins is captured
CEU.CAMCR.DTARY= 0; //10: Image input data is fetched in the order of Y0, Cb0, Y1, and Cr0
CEU.CAMCR.JPG = 0; //Image capture mode
CEU.CAMCR.VDPOL = 0; //(VD)  is high-active
CEU.CAMCR.HDPOL = 0;  //(HD)  is high-active*/
CEU.CAMCR = 0x00000200;


/*CEU.CMCYR.VCYL = HEIGHT; //Set the buffer limit for storing
  CEU.CMCYR.HCYL = STRIDE; //Horizontal Cycle Count*/
CEU.CMCYR= ((videoHeight<<16)&0x3FFF0000) + (((TOTAL_LINE_WIDTH)*2)&0x3FFF);//((WIDTH_CEU*2)&0x3FFF) ;

/*CEU.CAMOR_A.VOFST = CAMERA_VOFFSET(0); //Set camera input offset
 CEU.CAMOR_A.HOFST = CAMERA_HOFFSET(0); //Set camera input offset*/
CEU.CAMOR_A = CAMOR_A_INIT;

/*CEU.CAPWR.VWDTH = HEIGHT; //Set the number of lines to capture
 CEU.CAPWR.HWDTH = STRIDE; //Set the number of cycles to capture (1 pixel = 2 bytes/cycles)*/
CEU.CAPWR_A = (((videoHeight)<<16)&0x0FFC0000) + (((TOTAL_LINE_WIDTH)*2)&0x1FF8); //((WIDTH_CEU*2)&0x1FFE);

/*CEU.CAIFR.IFS = 0; //Progressive mode
CEU.CAIFR.CIM = 0; //Clear this bit to 0 when the input mode for image capture is progressive
CEU.CAIFR.FCI = 0; //Capture starts from the VD input immediately after the CEU activation*/
CEU.CAIFR = 0x00000000; // Both-field capture (interleaved if switch planes)

/*CEU.CRCNTR.RVS = 1; //b'4=1 Switches the register plane every 1 VD
  CEU.CRCNTR.RS = 0;  //b'1=0 Uses plane A of the register
  CEU.CRCNTR.RC = 0;  //b'0=1 Switches the register plane in synchronization with VD*/
CEU.CRCNTR = 0x00000010;//Switches the register plane every 1 VD


/*CEU.CRCMPR.RA = 1; //b'0 specify plane B of the register using this bit*/
CEU.CRCMPR = 0x00000001;

/*CEU.CFLCR = 0;//No scale down used*/
CEU.CFLCR_A = 0x00000000;

/*CEU.CFSZR.VFCLP = HEIGHT; //Set the number of lines to capture
  CEU.CFSZR.HFCLP = STRIDE; //Set the number of pixels per line to capture*/
CEU.CFSZR_A = ((videoHeight<<16)&0x0FFC0000) + ((videoWidth)&0x0FFC);//((WIDTH_CEU*2)&0x0FFC);

/*CEU.CDWDR_A.CHDW = CEU.CAPWR_A.HWDTH; /:In 8 bit interface mode*/
CEU.CDWDR_A = videoWidth & 0x1FFC; //CEU.CAPWR_A&0x1FFC;

//These values are just for reference, they will be over-write at frame capture


#if 0
CEU.CDAYR_A = (uint32_t) &FrameBufferA.Y[0];
CEU.CDACR_A = (uint32_t) &FrameBufferA.UV[0];
CEU.CDBYR_A = (uint32_t) &FrameBufferA.Y[0];
CEU.CDBCR_A = (uint32_t) &FrameBufferA.UV[0];
#else
y_size = videoWidth * videoHeight;
uv_size = y_size/2;
CEU.CDAYR_A = (uint32_t) &FrameBufferA.YUV[0];
CEU.CDACR_A = (uint32_t) (&FrameBufferA.YUV[0] + y_size);
CEU.CDBYR_A = (uint32_t) &FrameBufferA.YUV[0];
CEU.CDBCR_A = (uint32_t) (&FrameBufferA.YUV[0] + y_size);
#endif
/*
CEU.CBDSR = CAMERA_VERTICAL; //Number of lines for output to the memory in a bundle write */
CEU.CBDSR_A = 0; //Bundle write is not used so any value is fine here

/*CEU.CLFCR_A.LPF = 0;// Low-pass filter not used*/
CEU.CLFCR_A = 0x00000000;

/*CEU.CFWCR = 0; //Firewall is not activated (upper limit of the write addresses)*/
CEU.CFWCR = 0x00000000;

/*CEU.CDOCR_A.CBE = 0;//b'16 No bundle write (image is not splited into 2 buffers)
  CEU.CDOCR_A.CDS = 0;//b'4 Outputs in the YCbCr 420 format to the memory
  CEU.CDOCR_A.COLS =1;//b'2 Data is swapped in 32-bit units?????????????
  CEU.CDOCR_A.COWS =1;//b'1 Data is swapped in 16-bit units?????????????
  CEU.CDOCR_A.COBS =1;//b'0 Data is swapped in 8-bit units ?????????????*/
CEU.CDOCR_A = CDOCR_A_INIT;

#ifdef DEBUG
	CEU.CEIER = 0x0FFFFFFF; //Enable almost all possible interrupts (for debug purpose only)
#else CEU.CEIER = 0x00000000;
#endif

//Check below status registers
CEU.CETCR = 0X00000000;
CEU.CSTSR = 0X00000000;

CEU.CDSSR = 0X00000000;//Not used in image capture mode

/*
CEU.CDAYR2_A = &FrameBufferCDAYR2; //Y One-field image or top field=> used only in a bundle write
CEU.CDACR2_A = &FrameBufferCDACR2; //C One-field image or top field=> used only in a bundle write
CEU.CDBYR2_A = &FrameBufferCDBYR2; //Y bottom field=> used only in a bundle write
CEU.CDBCR2_A = &FrameBufferCDBCR2; //C bottom field=> used only in a bundle write

CEU.CAPSR.CE = 1; //Start capture of a single frame
CEU.CAPSR = 0X00000001; */



}
/**************************************************************************//**
* Function Name: CEU_ImageCaptureInit
* @brief       The Function initialise the CEU registers for image capture.
*
*              Description:
*
 * @param[in]  pYBuff pointer to the luminance (Y) buffer
 * @param[in]  pUVBuff pointer to the chrominance (UV) buffer
 * @return success or error (CEU busy).
*****************************************************************************/
#ifdef DEBUG

#define FLAG_CEU_CE		PORT7.Pn.BIT.Pn1 //LED0
#define FLAG_CEU_CPTON			PORT1.Pn.BIT.Pn3
uint8_t index=0;
uint8_t FlagP,FlagC=0;
int32_t Start,Finish,Table[256];
extern int32_t A_HD, A_IGHS, A_VD,A_VBP;
#endif

ceu_error_t CEU_CaptureOneFrame(stYUVPlanarBuffer *Frame)
{
    ceu_error_t ceu_error;

#ifdef DEBUG
	if(((CEU.CSTSR) & 0x00000001)==0)
    	{
    	FLAG_CEU_CPTON =0;
    	if(FlagC==1)
    		{
    		FlagC = 0;
       		Finish = A_HD-Start;
    		}
    	}
    else
    	{
    	FLAG_CEU_CPTON =1;
    	if(FlagC==0)
    		{
    		FlagC = 1;
    		Start = A_HD;
    		}

    	}

    if(((CEU.CAPSR) & 0x00000001)==0)
    	FLAG_CEU_CE =0;
    else
    	FLAG_CEU_CE =1;
#endif
//Start new capture if CSTSR.CPTON == CAPSR.CE == 0 (=>no capture request and none in progress)
if((((CEU.CSTSR) & 0x00000001)==0)&&((((CEU.CAPSR) & 0X00000001)==0)))
	{
	CEU.CDAYR_A = (uint32_t) &(Frame->YUV[0]);
	CEU.CDACR_A = (uint32_t) (&(Frame->YUV[0]) + y_size);
	CEU.CAPSR = 0X00000001; //CAPSR.CE =1 => Start capture on next VD
	ceu_error = CEU_ERROR_OK;
	}
else
	ceu_error = CEU_ERROR_PARAM;

return (ceu_error);
}

