/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2013 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name   : r_ether.c
* $Rev: $
* $Date::                           $
* Description : ETHER driver
*******************************************************************************/

/******************************************************************************
Includes <System Includes> , ?gProject Includes?h
******************************************************************************/
#include <stdio.h>
#include <string.h>

#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "r_typedefs.h"
#include "dev_drv.h"
#include "devdrv_ostm.h"            /* OSTM Driver header */
#include "devdrv_intc.h"            /* INTC Driver Header */
#include "pfc_iodefine.h"
#include "ether_iodefine.h"
#include "r_phy.h"
#include "r_ether.h"

//#include <trsocket.h>
//#include <trmacro.h>
//#include <trtype.h>

/******************************************************************************
Private global variables and functions
******************************************************************************/
/* ---- Descriptor ---- */
static volatile txrx_descriptor_set_t_ptr   eth_desc_ptr;
/* ---- Buffer ---- */
static volatile txrx_buffer_set_t_ptr       eth_buf_ptr;

int32_t  rx_interrupt_cnt = 0;
int32_t  rx_task_cnt = 0;
static int32_t  link_status;

static char_t *link_message[] = {"Link Down !!\n",
                                 "Link Up !! 10 Mbps / half-duplex\n",
                                 "Link Up !! 10 Mbps / full-duplex\n",
                                 "Link Up !! 100 Mbps / half-duplex\n",
                                 "Link Up !! 100 Mbps / full-duplex\n"};

//extern ttUserInterface tlInterfaceHandle;

static void lan_desc_create(void);
static void lan_reg_reset(void);
static void lan_reg_set(int32_t link);

#define I_DIV_P         (4) /* Ick:Pck0 = 4:1 */
#define PCLK_5CYC       ((5 * I_DIV_P) / 2)

void SetCAMEntryTable(uint32_t mac_h, uint32_t mac_l)
{
	if((ETHER.TSU_ADSBSY & 1) == 0)
	{
		ETHER.TSU_ADRH0 = mac_h;
		ETHER.TSU_ADRL0 = mac_l;
	}

	ETHER.TSU_TEN = 1 << 31;
}

/******************************************************************************
* ID            : ?|
* Outline       : Open the ethernet driver
* Include       : none
* Function Name : R_Ether_Open
* Description   : Initializes the EtherC, E-DMAC, PHY, and buffer memory.
* Argument      : uint32_t ch       ; I : Ethernet channel number
*               : uint8_t mac_addr[]; I : Pointer to MAC address array
* Return Value  : R_ETHER_OK(0)    : Success
*               : R_ETHER_ERROR(-1): Error
******************************************************************************/
int32_t R_Ether_Open (uint32_t ch, uint8_t mac_addr[])
{
    int32_t             link;
    uint32_t            mac_h;
    uint32_t            mac_l;

#ifdef PEKOE_BOARD
   //Configure ET_TXD0 (P8_0)
   PORT8.PMCn.BIT.PMCn0 = 1;
   PORT8.PFCn.BIT.PFCn0 = 1;
   PORT8.PFCEn.BIT.PFCEn0 = 0;
   PORT8.PFCAEn.BIT.PFCAEn0 = 0;
   PORT8.PIPCn.BIT.PIPCn0 = 1;

   //Configure ET_TXD1 (P8_1)
   PORT8.PMCn.BIT.PMCn1 = 1;
   PORT8.PFCn.BIT.PFCn1 = 1;
   PORT8.PFCEn.BIT.PFCEn1 = 0;
   PORT8.PFCAEn.BIT.PFCAEn1 = 0;
   PORT8.PIPCn.BIT.PIPCn1 = 1;

   //Configure ET_TXD2 (P8_2)
   PORT8.PMCn.BIT.PMCn2 = 1;
   PORT8.PFCn.BIT.PFCn2 = 1;
   PORT8.PFCEn.BIT.PFCEn2 = 0;
   PORT8.PFCAEn.BIT.PFCAEn2 = 0;
   PORT8.PIPCn.BIT.PIPCn2 = 1;

   //Configure ET_TXD3 (P8_3)
   PORT8.PMCn.BIT.PMCn3 = 1;
   PORT8.PFCn.BIT.PFCn3 = 1;
   PORT8.PFCEn.BIT.PFCEn3 = 0;
   PORT8.PFCAEn.BIT.PFCAEn3 = 0;
   PORT8.PIPCn.BIT.PIPCn3 = 1;

   //Configure ET_TXCLK (P8_4)
   PORT8.PMCn.BIT.PMCn4 = 1;
   PORT8.PFCn.BIT.PFCn4 = 1;
   PORT8.PFCEn.BIT.PFCEn4 = 0;
   PORT8.PFCAEn.BIT.PFCAEn4 = 0;
   PORT8.PIPCn.BIT.PIPCn4 = 1;

   //Configure ET_TXER (P8_5)
   PORT8.PMCn.BIT.PMCn5 = 1;
   PORT8.PFCn.BIT.PFCn5 = 1;
   PORT8.PFCEn.BIT.PFCEn5 = 0;
   PORT8.PFCAEn.BIT.PFCAEn5 = 0;
   PORT8.PIPCn.BIT.PIPCn5 = 1;

   //Configure ET_TXEN (P8_6)
   PORT8.PMCn.BIT.PMCn6 = 1;
   PORT8.PFCn.BIT.PFCn6 = 1;
   PORT8.PFCEn.BIT.PFCEn6 = 0;
   PORT8.PFCAEn.BIT.PFCAEn6 = 0;
   PORT8.PIPCn.BIT.PIPCn6 = 1;

   //Configure ET_RXD0 (P8_7)
   PORT8.PMCn.BIT.PMCn7 = 1;
   PORT8.PFCn.BIT.PFCn7 = 1;
   PORT8.PFCEn.BIT.PFCEn7 = 0;
   PORT8.PFCAEn.BIT.PFCAEn7 = 0;
   PORT8.PIPCn.BIT.PIPCn7 = 1;

   //Configure ET_RXD1 (P8_8)
   PORT8.PMCn.BIT.PMCn8 = 1;
   PORT8.PFCn.BIT.PFCn8 = 1;
   PORT8.PFCEn.BIT.PFCEn8 = 0;
   PORT8.PFCAEn.BIT.PFCAEn8 = 0;
   PORT8.PIPCn.BIT.PIPCn8 = 1;

   //Configure ET_RXD2 (P8_9)
   PORT8.PMCn.BIT.PMCn9 = 1;
   PORT8.PFCn.BIT.PFCn9 = 1;
   PORT8.PFCEn.BIT.PFCEn9 = 0;
   PORT8.PFCAEn.BIT.PFCAEn9 = 0;
   PORT8.PIPCn.BIT.PIPCn9 = 1;

   //Configure ET_RXD3 (P8_10)
   PORT8.PMCn.BIT.PMCn10 = 1;
   PORT8.PFCn.BIT.PFCn10 = 1;
   PORT8.PFCEn.BIT.PFCEn10 = 0;
   PORT8.PFCAEn.BIT.PFCAEn10 = 0;
   PORT8.PIPCn.BIT.PIPCn10 = 1;

   //Configure ET_COL (P8_14)
   PORT8.PMCn.BIT.PMCn14 = 1;
   PORT8.PFCn.BIT.PFCn14 = 1;
   PORT8.PFCEn.BIT.PFCEn14 = 0;
   PORT8.PFCAEn.BIT.PFCAEn14 = 0;
   PORT8.PIPCn.BIT.PIPCn14 = 1;

   //Configure ET_CRS (P8_15)
   PORT8.PMCn.BIT.PMCn15 = 1;
   PORT8.PFCn.BIT.PFCn15 = 1;
   PORT8.PFCEn.BIT.PFCEn15 = 0;
   PORT8.PFCAEn.BIT.PFCAEn15 = 0;
   PORT8.PIPCn.BIT.PIPCn15 = 1;

   //Configure ET_MDC (P9_0)
   PORT9.PMCn.BIT.PMCn0 = 1;
   PORT9.PFCn.BIT.PFCn0 = 1;
   PORT9.PFCEn.BIT.PFCEn0 = 0;
   PORT9.PFCAEn.BIT.PFCAEn0 = 0;
   PORT9.PIPCn.BIT.PIPCn0 = 1;

   //Configure ET_MDIO (P9_1)
   PORT9.PMCn.BIT.PMCn1 = 1;
   PORT9.PFCn.BIT.PFCn1 = 1;
   PORT9.PFCEn.BIT.PFCEn1 = 0;
   PORT9.PFCAEn.BIT.PFCAEn1 = 0;
   PORT9.PIPCn.BIT.PIPCn1 = 1;

   //Configure ET_RXCLK (P9_2)
   PORT9.PMCn.BIT.PMCn2 = 1;
   PORT9.PFCn.BIT.PFCn2 = 1;
   PORT9.PFCEn.BIT.PFCEn2 = 0;
   PORT9.PFCAEn.BIT.PFCAEn2 = 0;
   PORT9.PIPCn.BIT.PIPCn2 = 1;

   //Configure ET_RXER (P9_3)
   PORT9.PMCn.BIT.PMCn3 = 1;
   PORT9.PFCn.BIT.PFCn3 = 1;
   PORT9.PFCEn.BIT.PFCEn3 = 0;
   PORT9.PFCAEn.BIT.PFCAEn3 = 0;
   PORT9.PIPCn.BIT.PIPCn3 = 1;

   //Configure ET_RXDV (P9_4)
   PORT9.PMCn.BIT.PMCn4 = 1;
   PORT9.PFCn.BIT.PFCn4 = 1;
   PORT9.PFCEn.BIT.PFCEn4 = 0;
   PORT9.PFCAEn.BIT.PFCAEn4 = 0;
   PORT9.PIPCn.BIT.PIPCn4 = 1;

   //Configure Ethernet PHY reset pin (P2_7)
   PORT2.PMCn.BIT.PMCn7 = 0;
   PORT2.PIPCn.BIT.PIPCn7 = 0;
   PORT2.PMn.BIT.PMn7 = 0;

   //Take the PHY out of reset
   vTaskDelay(1000);
   PORT2.Pn.BIT.Pn7 = 1;
   vTaskDelay(1000);
#else
    /* ---- P1_14 : COL ---- */
    PORT1.PMCn.BIT.PMCn14     = 1;   /* ƒ|?[ƒgƒ‚?[ƒh : Œ“—pƒ‚?[ƒh   */
    PORT1.PFCAEn.BIT.PFCAEn14 = 0;   /* ƒ|?[ƒg‹@”\?Ý’è : ET_COL        */
    PORT1.PFCEn.BIT.PFCEn14   = 1;   /*                             */
    PORT1.PFCn.BIT.PFCn14     = 1;   /*                             */
    PORT1.PIPCn.BIT.PIPCn14   = 1;   /* “ü?o—Í?§Œäƒ‚?[ƒh : Žü•Ó‹@”\ */

    /* ---- P2_0 : TXCLK ---- */
    PORT2.PMCn.BIT.PMCn0     = 1;   /* ƒ|?[ƒgƒ‚?[ƒh : Œ“—pƒ‚?[ƒh   */
    PORT2.PFCAEn.BIT.PFCAEn0 = 0;   /* ƒ|?[ƒg‹@”\?Ý’è : ET_TXCLK        */
    PORT2.PFCEn.BIT.PFCEn0   = 0;   /*                             */
    PORT2.PFCn.BIT.PFCn0     = 1;   /*                             */
    PORT2.PIPCn.BIT.PIPCn0   = 1;   /* “ü?o—Í?§Œäƒ‚?[ƒh : Žü•Ó‹@”\ */
//    PORT2.PMn.BIT.PMn0       = 1;   /*                             */

    /* ---- P2_1 : TXER ---- */
    PORT2.PMCn.BIT.PMCn1     = 1;   /* ƒ|?[ƒgƒ‚?[ƒh : Œ“—pƒ‚?[ƒh   */
    PORT2.PFCAEn.BIT.PFCAEn1 = 0;   /* ƒ|?[ƒg‹@”\?Ý’è : ET_TXER        */
    PORT2.PFCEn.BIT.PFCEn1   = 0;   /*                             */
    PORT2.PFCn.BIT.PFCn1     = 1;   /*                             */
    PORT2.PIPCn.BIT.PIPCn1   = 1;   /* “ü?o—Í?§Œäƒ‚?[ƒh : Žü•Ó‹@”\ */

    /* ---- P2_2 : TXEN ---- */
    PORT2.PMCn.BIT.PMCn2     = 1;   /* ƒ|?[ƒgƒ‚?[ƒh : Œ“—pƒ‚?[ƒh   */
    PORT2.PFCAEn.BIT.PFCAEn2 = 0;   /* ƒ|?[ƒg‹@”\?Ý’è : ET_TXEN        */
    PORT2.PFCEn.BIT.PFCEn2   = 0;   /*                             */
    PORT2.PFCn.BIT.PFCn2     = 1;   /*                             */
    PORT2.PIPCn.BIT.PIPCn2   = 1;   /* “ü?o—Í?§Œäƒ‚?[ƒh : Žü•Ó‹@”\ */

    /* ---- P2_3 : CRS ---- */
    PORT2.PMCn.BIT.PMCn3     = 1;   /* ƒ|?[ƒgƒ‚?[ƒh : Œ“—pƒ‚?[ƒh   */
    PORT2.PFCAEn.BIT.PFCAEn3 = 0;   /* ƒ|?[ƒg‹@”\?Ý’è : ET_CRS        */
    PORT2.PFCEn.BIT.PFCEn3   = 0;   /*                             */
    PORT2.PFCn.BIT.PFCn3     = 1;   /*                             */
    PORT2.PIPCn.BIT.PIPCn3   = 1;   /* “ü?o—Í?§Œäƒ‚?[ƒh : Žü•Ó‹@”\ */

    /* ---- P2_4 : TXD0 ---- */
    PORT2.PMCn.BIT.PMCn4     = 1;   /* ƒ|?[ƒgƒ‚?[ƒh : Œ“—pƒ‚?[ƒh   */
    PORT2.PFCAEn.BIT.PFCAEn4 = 0;   /* ƒ|?[ƒg‹@”\?Ý’è : ET_TXD0        */
    PORT2.PFCEn.BIT.PFCEn4   = 0;   /*                             */
    PORT2.PFCn.BIT.PFCn4     = 1;   /*                             */
    PORT2.PIPCn.BIT.PIPCn4   = 1;   /* “ü?o—Í?§Œäƒ‚?[ƒh : Žü•Ó‹@”\ */

    /* ---- P2_5 : TXD1 ---- */
    PORT2.PMCn.BIT.PMCn5     = 1;   /* ƒ|?[ƒgƒ‚?[ƒh : Œ“—pƒ‚?[ƒh   */
    PORT2.PFCAEn.BIT.PFCAEn5 = 0;   /* ƒ|?[ƒg‹@”\?Ý’è : ET_TXD1        */
    PORT2.PFCEn.BIT.PFCEn5   = 0;   /*                             */
    PORT2.PFCn.BIT.PFCn5     = 1;   /*                             */
    PORT2.PIPCn.BIT.PIPCn5   = 1;   /* “ü?o—Í?§Œäƒ‚?[ƒh : Žü•Ó‹@”\ */

    /* ---- P2_6 : TXD2 ---- */
    PORT2.PMCn.BIT.PMCn6     = 1;   /* ƒ|?[ƒgƒ‚?[ƒh : Œ“—pƒ‚?[ƒh   */
    PORT2.PFCAEn.BIT.PFCAEn6 = 0;   /* ƒ|?[ƒg‹@”\?Ý’è : ET_TXD2        */
    PORT2.PFCEn.BIT.PFCEn6   = 0;   /*                             */
    PORT2.PFCn.BIT.PFCn6     = 1;   /*                             */
    PORT2.PIPCn.BIT.PIPCn6   = 1;   /* “ü?o—Í?§Œäƒ‚?[ƒh : Žü•Ó‹@”\ */

    /* ---- P2_7 : TXD3 ---- */
    PORT2.PMCn.BIT.PMCn7     = 1;   /* ƒ|?[ƒgƒ‚?[ƒh : Œ“—pƒ‚?[ƒh   */
    PORT2.PFCAEn.BIT.PFCAEn7 = 0;   /* ƒ|?[ƒg‹@”\?Ý’è : ET_TXD3        */
    PORT2.PFCEn.BIT.PFCEn7   = 0;   /*                             */
    PORT2.PFCn.BIT.PFCn7     = 1;   /*                             */
    PORT2.PIPCn.BIT.PIPCn7   = 1;   /* “ü?o—Í?§Œäƒ‚?[ƒh : Žü•Ó‹@”\ */

    /* ---- P2_8 : RXD0 ---- */
    PORT2.PMCn.BIT.PMCn8     = 1;   /* ƒ|?[ƒgƒ‚?[ƒh : Œ“—pƒ‚?[ƒh   */
    PORT2.PFCAEn.BIT.PFCAEn8 = 0;   /* ƒ|?[ƒg‹@”\?Ý’è : ET_COL        */
    PORT2.PFCEn.BIT.PFCEn8   = 0;   /*                             */
    PORT2.PFCn.BIT.PFCn8     = 1;   /*                             */
    PORT2.PIPCn.BIT.PIPCn8   = 1;   /* “ü?o—Í?§Œäƒ‚?[ƒh : Žü•Ó‹@”\ */

    /* ---- P2_9 : RXD1 ---- */
    PORT2.PMCn.BIT.PMCn9     = 1;   /* ƒ|?[ƒgƒ‚?[ƒh : Œ“—pƒ‚?[ƒh   */
    PORT2.PFCAEn.BIT.PFCAEn9 = 0;   /* ƒ|?[ƒg‹@”\?Ý’è : ET_RXD1        */
    PORT2.PFCEn.BIT.PFCEn9   = 0;   /*                             */
    PORT2.PFCn.BIT.PFCn9     = 1;   /*                             */
    PORT2.PIPCn.BIT.PIPCn9   = 1;   /* “ü?o—Í?§Œäƒ‚?[ƒh : Žü•Ó‹@”\ */

    /* ---- P2_10 : RXD2 ---- */
    PORT2.PMCn.BIT.PMCn10     = 1;   /* ƒ|?[ƒgƒ‚?[ƒh : Œ“—pƒ‚?[ƒh   */
    PORT2.PFCAEn.BIT.PFCAEn10 = 0;   /* ƒ|?[ƒg‹@”\?Ý’è : ET_RXD2        */
    PORT2.PFCEn.BIT.PFCEn10   = 0;   /*                             */
    PORT2.PFCn.BIT.PFCn10     = 1;   /*                             */
    PORT2.PIPCn.BIT.PIPCn10   = 1;   /* “ü?o—Í?§Œäƒ‚?[ƒh : Žü•Ó‹@”\ */

    /* ---- P2_11 : RXD3 ---- */
    PORT2.PMCn.BIT.PMCn11     = 1;   /* ƒ|?[ƒgƒ‚?[ƒh : Œ“—pƒ‚?[ƒh   */
    PORT2.PFCAEn.BIT.PFCAEn11 = 0;   /* ƒ|?[ƒg‹@”\?Ý’è : ET_RXD3        */
    PORT2.PFCEn.BIT.PFCEn11   = 0;   /*                             */
    PORT2.PFCn.BIT.PFCn11     = 1;   /*                             */
    PORT2.PIPCn.BIT.PIPCn11   = 1;   /* “ü?o—Í?§Œäƒ‚?[ƒh : Žü•Ó‹@”\ */

    /* ---- P3_3 : MDIO ---- */
    PORT3.PMCn.BIT.PMCn3     = 1;   /* ƒ|?[ƒgƒ‚?[ƒh : Œ“—pƒ‚?[ƒh   */
    PORT3.PFCAEn.BIT.PFCAEn3 = 0;   /* ƒ|?[ƒg‹@”\?Ý’è : ET_MDIO        */
    PORT3.PFCEn.BIT.PFCEn3   = 0;   /*                             */
    PORT3.PFCn.BIT.PFCn3     = 1;   /*                             */
    PORT3.PIPCn.BIT.PIPCn3   = 1;   /* “ü?o—Í?§Œäƒ‚?[ƒh : Žü•Ó‹@”\ */

    /* ---- P3_4 : RXCLK ---- */
    PORT3.PMCn.BIT.PMCn4     = 1;   /* ƒ|?[ƒgƒ‚?[ƒh : Œ“—pƒ‚?[ƒh   */
    PORT3.PFCAEn.BIT.PFCAEn4 = 0;   /* ƒ|?[ƒg‹@”\?Ý’è : ET_RXCLK        */
    PORT3.PFCEn.BIT.PFCEn4   = 0;   /*                             */
    PORT3.PFCn.BIT.PFCn4     = 1;   /*                             */
    PORT3.PIPCn.BIT.PIPCn4   = 1;   /* “ü?o—Í?§Œäƒ‚?[ƒh : Žü•Ó‹@”\ */

    /* ---- P3_5 : RXER ---- */
    PORT3.PMCn.BIT.PMCn5     = 1;   /* ƒ|?[ƒgƒ‚?[ƒh : Œ“—pƒ‚?[ƒh   */
    PORT3.PFCAEn.BIT.PFCAEn5 = 0;   /* ƒ|?[ƒg‹@”\?Ý’è : ET_RXER        */
    PORT3.PFCEn.BIT.PFCEn5   = 0;   /*                             */
    PORT3.PFCn.BIT.PFCn5     = 1;   /*                             */
    PORT3.PIPCn.BIT.PIPCn5   = 1;   /* “ü?o—Í?§Œäƒ‚?[ƒh : Žü•Ó‹@”\ */

    /* ---- P3_6 : RXDV ---- */
    PORT3.PMCn.BIT.PMCn6     = 1;   /* ƒ|?[ƒgƒ‚?[ƒh : Œ“—pƒ‚?[ƒh   */
    PORT3.PFCAEn.BIT.PFCAEn6 = 0;   /* ƒ|?[ƒg‹@”\?Ý’è : ET_RXDV        */
    PORT3.PFCEn.BIT.PFCEn6   = 0;   /*                             */
    PORT3.PFCn.BIT.PFCn6     = 1;   /*                             */
    PORT3.PIPCn.BIT.PIPCn6   = 1;   /* “ü?o—Í?§Œäƒ‚?[ƒh : Žü•Ó‹@”\ */

    /* ---- P5_9 : MDC ---- */
    PORT5.PMCn.BIT.PMCn9     = 1;   /* ƒ|?[ƒgƒ‚?[ƒh : Œ“—pƒ‚?[ƒh   */
    PORT5.PFCAEn.BIT.PFCAEn9 = 0;   /* ƒ|?[ƒg‹@”\?Ý’è : ET_MDC        */
    PORT5.PFCEn.BIT.PFCEn9   = 0;   /*                             */
    PORT5.PFCn.BIT.PFCn9     = 1;   /*                             */
    PORT5.PIPCn.BIT.PIPCn9   = 1;   /* “ü?o—Í?§Œäƒ‚?[ƒh : Žü•Ó‹@”\ */
#endif

    /* ==== Resets the E-MAC,E-DMAC === */
    lan_reg_reset();

    /* ==== Initialize of buffer memory ==== */
    lan_desc_create();

    /* ==== MAC address setting ==== */
    mac_h = ((uint32_t)mac_addr[0] << 24) |
            ((uint32_t)mac_addr[1] << 16) |
            ((uint32_t)mac_addr[2] << 8 ) |
            (uint32_t)mac_addr[3];

    mac_l = ((uint32_t)mac_addr[4] << 8 ) |
            (uint32_t)mac_addr[5];

    if ((mac_h == 0) && (mac_l == 0))
    {
        /*
         * If 2nd parameter is 0, the MAC address should be acquired from the system,
         * depending on user implementation. (e.g.: EEPROM)
         */
    }
    else
    {
        ETHER.MAHR0 = mac_h;
        ETHER.MALR0 = mac_l;
    }

//    SetCAMEntryTable(mac_h, mac_l);

    /* ==== E-MAC/E-DMAC setting ==== */
    link = phy_autonego();                    /* Detects the communication mode */
    link_status = link;

    if ( NEGO_FAIL == link )
    {
        printf("Link ERROR !!\n");
        return R_ETHER_ERROR;                 /* Failed to initialize */
    }
    else
    {
        printf(link_message[link]);
        lan_reg_set(link);
    }

    return R_ETHER_OK;
}

/******************************************************************************
* ID            : ?|
* Outline       : Close the ethernet driver
* Include       : none
* Function Name : R_Ether_Close
* Description   : Stops the E-MAC/E-DMAC.
* Argument      : uint32_t ch; I : Ethernet channel number
* Return Value  : R_ETHER_OK(0)     : Success
*               : R_ETHER_ERROR(-1) : Error
******************************************************************************/
int32_t R_Ether_Close (uint32_t ch)
{

    /* ==== Resets the E-MAC,E-DMAC === */
    lan_reg_reset();

    /* ==== Disables the E-DMAC interrupt === */
    R_INTC_Disable(INTC_ID_ETHERI);

    return R_ETHER_OK;
}

/******************************************************************************
* ID            : ?|
* Outline       : Read the frame
* Include       : none
* Function Name : R_Ether_Read
* Description   : Copies the received Ethernet frame on the Ethernet channel
*               : specified by channel number to the receive buffer.
* Argument      : uint32_t ch; I : Ethernet channel number
*               : void *buf  ; I : Pointer to Ethernet receive buffer
* Return Value  : Greater than 0   : Success. Returns number of bytes received
*               : R_ETHER_ERROR(-1): Error
*               : R_ETHER_HARD_ERROR(-3): Hardware error. Software reset is necessary to recover
*               : R_ETHER_RECOVERABLE(-4): Recoverable error
*               : R_ETHER_NODATA(-5): No data received
******************************************************************************/
int32_t R_Ether_Read (uint32_t ch, void *buf)
{
    edmac_recv_desc_t * p   = eth_desc_ptr->pRecv_end;   /* Current descriptor */
    int32_t             ret = 0;


    /* ==== No data ==== */
    if (p->rd0.BIT.RACT == 1)
    {
        return R_ETHER_NODATA;
    }

    rx_task_cnt++;

    /* ==== Receives 1 frame ==== */

    /* ---- Receive frame error ---- */
    if ((p->rd0.BIT.RFE == 1)  &&  ((p->rd0.LONG & 0x025f0000) != 0))
    {
        p->rd0.LONG &= 0x70000000;          /* Processes the error flag */
        ret = R_ETHER_ERROR;
    }
    /* ---- Copies the received frame ---- */
    else
    {
        tm_memcpy(buf, p->rd2.RBA, (size_t)p->rd1.RDL);
        ret = p->rd1.RDL;                   /* number of bytes received */
    }

    /* ---- Sets the receive descriptor to receive again ---- */
    p->rd0.BIT.RACT = 1;

    /* ---- Starts receiving frame ---- */
    if( ETHER.EDRRR0 == 0 )
    {
        ETHER.EDRRR0 =  1;
    }

    /* ==== Update the current pointer value ===ifcon
     * = */
    eth_desc_ptr->pRecv_end = p->pNext;

    return ret;
}
/******************************************************************************
* ID            : ?|
* Outline       : Transfer the frame
* Include       : none
* Function Name : R_Ether_Write
* Description   : This function sends an Ethernet frame pointed by Ethernet frame
*               : pointer on the Ethernet channel specified by channel number.
*               : It updates the status of the transmit descriptor as new Ethernet
*               : frame is processed. It doesn?ft check whether Ethernet frame
*               : transmission completed or not.
* Argument      : uint32_t ch; I : Ethernet channel number
*               : void *buf  ; I : Pointer to Ethernet transmit buffer
*               : uint32_t len;I : Ethernet frame length (unit:byte)
* Return Value  : R_ETHER_OK(0)    : Success
*               : R_ETHER_ERROR(-1): Error
******************************************************************************/
int32_t R_Ether_Write (uint32_t ch, void * buf, uint32_t len)
{
    edmac_send_desc_t * p   = eth_desc_ptr->pSend_top;   /* Current descriptor */

    /* ==== link is down ==== */
    if (link_status == NEGO_FAIL)
    {
        return R_ETHER_ERROR;
    }

    /* ==== When the buffer is full ==== */
    if (p->td0.BIT.TACT == 1)
    {
        return R_ETHER_ERROR;
    }

    /* ==== Transfer 1 frame ==== */

    /* ---- Copies the transmit frame ---- */
    tm_memcpy(p->td2.TBA, buf, len);

    /* ---- Padding for the short frame ---- */
    if (len < MIN_FRAME_SIZE)
    {
        tm_memset((p->td2.TBA + len), 0, (MIN_FRAME_SIZE - len));
        len = MIN_FRAME_SIZE;
    }

    /* ---- Sets the frame length ---- */
    p->td1.TDL = (uint16_t)len;

    /* ---- Sets the transmit descriptor to transmit again ---- */
    p->td0.BIT.TACT = 1;

    /* ---- Starts the transmission ---- */
    if (ETHER.EDTRR0 != 3)
    {
        ETHER.EDTRR0 = 3;
    }

    /* ==== Update the current pointer value ==== */
    eth_desc_ptr->pSend_top = p->pNext;

    return R_ETHER_OK;
}

/******************************************************************************
* ID            : ?|
* Outline       : Create the descriptor
* Include       : none
* Function Name : lan_desc_create
* Description   : Ethernet performs initialization of the required transceiver
*               : buffer and the descriptor
* Argument      : none
* Return Value  : none
******************************************************************************/
static void lan_desc_create (void)
{
    int32_t    i;

    /* ==== Descriptor area configuration ==== */
    /* ---- Memory clear ---- */
    eth_desc_ptr = (txrx_descriptor_set_t_ptr)ADDR_OF_TXRX_DESC;
    tm_memset((void *)eth_desc_ptr, 0, sizeof(txrx_descriptor_set_t));

    /* ==== Buffer area configuration ==== */
    /* ---- Clear the area ---- */
    eth_buf_ptr  = (txrx_buffer_set_t_ptr)ADDR_OF_TXRX_BUFF;
    tm_memset((void *)eth_buf_ptr, 0, sizeof(txrx_buffer_set_t));

    /* ---- Transmit descriptor ---- */
    for (i = 0; i < NUM_OF_TX_DESCRIPTOR; i++)
    {
        eth_desc_ptr->dsend[i].td2.TBA  = eth_buf_ptr->bsend[i];  /* TD2 */
        eth_desc_ptr->dsend[i].td1.TDL  = 0;                      /* TD1 */
        eth_desc_ptr->dsend[i].td0.LONG = 0x30000000;        /* TD0:1frame/1buf1buf, transmission disabled */

        if ((NUM_OF_TX_DESCRIPTOR - 1) != i)
        {
            eth_desc_ptr->dsend[i].pNext = &eth_desc_ptr->dsend[i + 1]; /* pNext */
        }
    }

    eth_desc_ptr->dsend[i - 1].td0.BIT.TDLE = 1;
    eth_desc_ptr->dsend[i - 1].pNext        = &eth_desc_ptr->dsend[0];

    /* ---- Receive descriptor ---- */
    for (i = 0; i < NUM_OF_RX_DESCRIPTOR; i++)
    {
        eth_desc_ptr->drecv[i].rd2.RBA = eth_buf_ptr->brecv[i];      /* RD2 */
        eth_desc_ptr->drecv[i].rd1.RBL = (uint16_t)SIZE_OF_BUFFER;   /* RD1 */
        eth_desc_ptr->drecv[i].rd0.LONG= 0xB0000000;        /* RD0:1frame/1buf, reception enabled */
        eth_desc_ptr->drecv[i].rd0.BIT.RACT = 1;
        if ((NUM_OF_RX_DESCRIPTOR - 1) != i )
        {
            eth_desc_ptr->drecv[i].pNext = &eth_desc_ptr->drecv[i + 1]; /* pNext */
        }
    }

    eth_desc_ptr->drecv[i - 1].rd0.BIT.RDLE = 1;                /* Set the last descriptor */
    eth_desc_ptr->drecv[i - 1].pNext        = &eth_desc_ptr->drecv[0];

    /* ---- Initialize descriptor management information ---- */
    eth_desc_ptr->pSend_top = &eth_desc_ptr->dsend[0];
    eth_desc_ptr->pRecv_end = &eth_desc_ptr->drecv[0];
}

/******************************************************************************
* ID            : ?|
* Outline       : Reset registers
* Include       : none
* Function Name : lan_reg_reset
* Description   : E-MAC,E-DMAC registers initialization function
* Argument      : none
* Return Value  : none
******************************************************************************/
static void lan_reg_reset (void)
{
    volatile int32_t    j = 400;            /* Wait for BƒÓ256 cycles */

                                            /*  ((IƒÓ/BƒÓ)*256)/6cyc = 8*256/6 = 342 */
    /* ---- ETHER software reset ---- */
    /* Ethernet Controller (ETHER)
    ARSTR - Software Reset Register
    b31:b1  Reserved   - Read only bit
    b0      ARST       - Software Reset : Reset all blocks of the ETHER (1) */
    ETHER.ARSTR =  1;
    while (j--)
    {
        /* Wait for BƒÓ256 cycles */
    }

    /* ---- E-DMAC software reset ---- */
    /* Ethernet Controller (ETHER)
    EDSR - E-DMAC Start Register
    b31:b2  Reserved   - Read only bit
    b1      ENT        - E-DMAC Transmitting Unit Start : Start (1)
    b0      ENR        - E-DMAC Receiving Unit Start    : Start (1) */
    ETHER.EDSR0  =  0x00000003;

    /* Ethernet Controller (ETHER)
    EDMR - E-DMAC Mode Register
    b31:b7  Reserved   - Read only bit
    b6      DE         - Transmit/Receive Frame Endian
    b5:b4   DL[1:0]    - Transmit/Receive Descriptor Length
    b3:b2   Reserved   - Read only bit
    b1      SWRT       - Software Reset of Transmit FIFO Controller : Reset start (1)
    b0      SWRR       - Software Reset of Receive FIFO Controller  : Reset start (1) */
    ETHER.EDMR0  =  0x00000003;     /* Set SWRR and SWRT simultaneously */

    /* ---- Check clear software reset ---- */
    while(ETHER.EDMR0 !=  0x00000000)
    {
//        nop();
//        nop();
    }
}

/******************************************************************************
* ID            : ?|
* Outline       : Set registers
* Include       : none
* Function Name : lan_reg_set
* Description   : E-DMAC, E-MAC initialization
* Argument      : int32_t link
* Return Value  : none
******************************************************************************/
static void lan_reg_set (int32_t link)
{
    /* ==== EDMAC ==== */
    /* Ethernet Controller (ETHER)
    TDLAR - Transmit Descriptor List Start Address Register
    b31:b0  TDLA[31:0] - Transmit Descriptor Start Address */
    ETHER.TDLAR0        = (uint32_t)&eth_desc_ptr->dsend[0];

    /* Ethernet Controller (ETHER)
    RDLAR - Receive Descriptor List Start Address Register
    b31:b0  RDLA[31:0] - Receive Descriptor Start Address */
    ETHER.RDLAR0        = (uint32_t)&eth_desc_ptr->drecv[0];

    /* Ethernet Controller (ETHER)
    TDFAR - Transmit Descriptor Fetch Address Register
    b31:b0  TDFA[31:0] - Transmit Descriptor Fetch Address */
    ETHER.TDFAR0        = (uint32_t)&eth_desc_ptr->dsend[0];

    /* Ethernet Controller (ETHER)
    RDFAR - Receive Descriptor Fetch Address Register
    b31:b0  RDFA[31:0] - Receive Descriptor Fetch Address */
    ETHER.RDFAR0        = (uint32_t)&eth_desc_ptr->drecv[0];

    /* Ethernet Controller (ETHER)
    TDFXR - Transmit Descriptor Finished Address Register
    b31:b0  TDFX[31:0] - Transmit Descriptor Finished Address */
    ETHER.TDFXR0        = (uint32_t)&eth_desc_ptr->dsend[NUM_OF_TX_DESCRIPTOR - 1];

    /* Ethernet Controller (ETHER)
    RDFXR - Receive Descriptor Finished Address Register
    b31:b0  RDFX[31:0] - Receive Descriptor Finished Address */
    ETHER.RDFXR0        = (uint32_t)&eth_desc_ptr->drecv[NUM_OF_RX_DESCRIPTOR - 1];

    /* Ethernet Controller (ETHER)
    TDFFR - Transmit Descriptor Final Flag Register
    b31:b1  Reserved   - Read only bit
    b0      TDLF       - Transmit Descriptor Queue Last Flag : Last descriptor (1) */
    ETHER.TDFFR0   = 0x00000001;

    /* Ethernet Controller (ETHER)
    RDFFR - Receive Descriptor Final Flag Register
    b31:b1  Reserved   - Read only bit
    b0      RDLF       - Receive Descriptor Queue Last Flag : Last descriptor (1) */
    ETHER.RDFFR0   = 0x00000001;

    /* Ethernet Controller (ETHER)
    EDMR - E-DMAC Mode Register
    b31:b7  Reserved   - Read only bit
    b6      DE         - Transmit/Receive Frame Endian : Little endian (1)
    b5:b4   DL[1:0]    - Transmit/Receive Descriptor Length : 16 bytes (00)
    b3:b2   Reserved   - Read only bit
    b1      SWRT       - Software Reset of Transmit FIFO Controller
    b0      SWRR       - Software Reset of Receive FIFO Controller */
#if defined(_BIG)
    ETHER.EDMR0         = 0x00000000; /* Big endian */
#else
    ETHER.EDMR0         = 0x00000040; /* Little endian */
#endif

    /* Ethernet Controller (ETHER)
    TRSCER - Transmit/Receive Status Copy Enable Register
    b31:b18 Reserved   - Read only bit
    b17     TABTCE     - TABT Bit Copy Direcrive : Reflects in the TFE bit (0)
    b16     RABTCE     - RABT Bit Copy Direcrive : Reflects in the RFE bit (0)
    b15:b11 Reserved   - Read only bit
    b10     DLCCE      - DLC Bit Copy Directive  : Reflects in the TFE bit (0)
    b9      CDCE       - CD  Bit Copy Directive  : Reflects in the TFE bit (0)
    b8      TROCE      - TRO Bit Copy Directive  : Reflects in the TFE bit (0)
    b7      RMAFCE     - RMAF Bit Copy Directive : Reflects in the RFE bit (0)
    b6      CEEFCE     - CEEF Bit Copy Directive : Reflects in the RFE bit (0)
    b5      CELFCE     - CELF Bit Copy Directive : Reflects in the RFE bit (0)
    b4      RRFCE      - RRF  Bit Copy Directive : Reflects in the RFE bit (0)
    b3      RTLFCE     - RTLF Bit Copy Directive : Reflects in the RFE bit (0)
    b2      RTSFCE     - RTSF Bit Copy Directive : Reflects in the RFE bit (0)
    b1      PRECE      - PRE  Bit Copy Directive : Reflects in the RFE bit (0)
    b0      CERFCE     - CERF Bit Copy Directive : Reflects in the RFE bit (0) */
    ETHER.TRSCER0  = 0x00000000;

    /* Ethernet Controller (ETHER)
    TFTR - Transmit FIFO Threshold Register
    b31:b11 Reserved   - Read only bit
    b10:b0  TFT[10:0]  - Transmit FIFO Threshold : Store and forward modes (H'000) */
    ETHER.TFTR0         = 0x00000000;

    /* Ethernet Controller (ETHER)
    FDR - FIFO Depth Register
    b31:b11 Reserved   - Read only bit
    b10:b8  TFD[2:0]   - Transmit FIFO Size : 2048 bytes (H'07)
    b7:b5   Reserved   - Read only bit
    b4:b0   RFD[4:0]   - Receive FIFO Size : 2048 bytes (H'07) */
//    EDMAC.FDR0.BIT.TFD  = 0x07;
//    EDMAC.FDR0.BIT.RFD  = 0x07;
    ETHER.FDR0  = 0x0707;


    /* Ethernet Controller (ETHER)
    RMCR - Receiving Method Control Register
    b31:b1  Reserved   - Read only bit
    b0      RNC        - Receive Enable Control : Continuous reception enabled (1) */
    ETHER.RMCR0 = 1;

    /* Ethernet Controller (ETHER)
    FCFTR - Overflow Alert FIFO Threshold Register
    b31:b21 Reserved   - Read only bit
    b20:b16 RFF[4:0]   - Receive FIFO Overflow Alert Signal Output Threshold : 8 receive frams (H'07)
    b15:b8  Reserved   - Read only bit
    b7:b0   RFD[7:0]   - Receive FIFO Overflow Alert Signal Output Threshold : (2048-64) bytes (H'07) */
    ETHER.FCFTR0   = 0x00070007; /* Flow control is turned on when either of the condition is satisfied.*/

    /* Ethernet Controller (ETHER)
    RPADIR - Receive Data Padding Insert Register
    b31:b21 Reserved   - Read only bit
    b20:b16 PADS[4:0]  - Padding Size : No padding insertion (H'00)
    b15:b0  RADR[15:0] - Padding Slot : Inserts at first byte (H'0000) */
    ETHER.RPADIR0  = 0x00000000;


    /* ==== E-MAC ==== */
    /* Ethernet Controller (ETHER)
    ECMR - E-MAC Mode Register
    b31:b27 Reserved   - Read only bit
    b26     TRCCM      - Counter Clear Mode : Counter register is cleared to 0 by writing H'11111111 (0)
    b25:b24 Reserved   - Read only bit
    b23     RCSC       - Checksum Calculation : not automatically (0)
    b22     Reserved   - Read only bit
    b21     DPAD       - Data Padding : Enable insert (0)
    b20     RZPF       - PAUSE Frame Receptin with TIME = 0 : Disable (0)
    b19     ZPF        - PAUSE Frame Usage with TIME = 0 Enable : Disable
                         /Lost Carrier Error Detection Enable : Enable (0)
    b18     PFR        - PAUSE Frame Receive Mode : Not transferred to E-DMAC (0)
    b17     RXF        - Operating Mode for Receiving Port Flow Control : PAUSE frame detection is disabled (0)
    b16     TXF        - Operating Mode for Transmitting Port Flow Control : Automatic PAUSE frame is not transmitted (0)
    b15:b14 Reserved   - Read only bit
    b13     MCT        - Multicast Address Frame Receive Mode : When CAM is disabled, all multicast frames are received. (0)
    b12:b10 Reserved   - Read only bit
    b9      MPDE       - Magic Packet Detection Enable : Disable (0)
    b8:b7   Reserved   - Read only bit
    b6      RE         - Reception Enable : Disable (0)
    b5      TE         - Transmission Enable : Disable (0)
    b4      Reserved   - Read only bit
    b3      ILB        - Internal Loop Back Mode : Disable loop back (0)
    b2      Reserved   - Read only bit
    b1      DM         - Duplex Mode
    b0      PRM        - Promiscuous Mode : Normal operation (0) */
    ETHER.ECMR0      = 0x00000000;

    /* Ethernet Controller (ETHER)
    RFLR - Receive Frame Length Register
    b31:b18 Reserved   - Read only bit
    b17:b0  RFL[17:0]  - Receive Frame Length : 1518 bytes (H'00000) */
    ETHER.RFLR0           = 0x00000;

    /* Ethernet Controller (ETHER)
    APR - Automatic PAUSE Frame Register
    b31:b16 Reserved   - Read only bit
    b15:b0  AP[15:0]   - Automatic PAUSE : Flow control is disabled (H'0000)*/
    ETHER.APR0            = 0x00000000;

    /* Ethernet Controller (ETHER)
    MPR - Manual PAUSE Frame Register
    b31:b16 Reserved   - Read only bit
    b15:b0  MP[15:0]   - Manual PAUSE : Flow control is disabled (H'0000)*/
    ETHER.MPR0            = 0x00000000;

    /* Ethernet Controller (ETHER)
    TPAUSER - Automatic PAUSE Frame Retransmit Count Register
    b31:b16 Reserved   - Read only bit
    b15:b0  TPAUSE[15:0] - Upper Limit for Automatic PAUSE Frame : Retransmit count is unlimited (H'0000) */
    ETHER.TPAUSER0        = 0x00000000;

    /* Ethernet Controller (ETHER)
    ECMR - E-MAC Mode Register
    b1      DM         - Duplex Mode : Full-duplex (1) or Half-duplex (0) */
    if ((FULL_TX == link) || (FULL_10M == link))
    {
        ETHER.ECMR0 = ETHER.ECMR0 | 0x00000002;               /* Set to full-duplex mode */
    }
    else
    {
        ETHER.ECMR0 = ETHER.ECMR0 & 0xfffffffd;               /* Set to full-duplex mode */
    }


    /* ==== Interrupt-related ==== */
    /* Ethernet Controller (ETHER)
    EESR - E-MAC/E-DMAC Status Register
    b31:b30 TWB[1:0]   - Write-Back Complete                          : Has not completed, or no transmission directive (00)
    b29     TC[1]      - Frame Transmission Complete(TC[1:0]=11)      : Has not completed, or no transmission directive (00)
    b28     TUC        - Transmit Underflow Frame Write-Back Complete : Has not completed for the frame causing transmit underflow (0)
    b27     ROC        - Receive Overflow Frame Write-Back Complete   : Has not completed for the frame causing transmit overflow (0)
    b26     TABT       - Transmit Abort Detect                        : Has not aborted or no transmission direcrive (0)
    b25     RABT       - Receive Abort Detect                         : Has not aborted or no reception direcrive (0)
    b24     RFCOF      - Receive Frame Counter Overflow               : Has not overflowed (0)
    b23     Reserved   - Read only bit
    b22     ECI        - E-MAC Status Register Source                 : E-MAC status interrupt source has not been detected (0)
    b21     TC[0]      - Frame Transmission Complete(TC[1:0]=11)      : Has not completed, or no transmission directive (00)
    b20     TDE        - Transmit Descriptor Empty                    : TDE error has not occurred (0)
    b19     TFUF       - Transmit FIFO Underflow                      : Has not underflowed (0)
    b18     FR         - Frame Reception                              : Has not received (0)
    b17     RDE        - Receive Descriptor Empty                     : RDE error has not occurred (0)
    b16     RFOF       - Receive FIFO Overflow                        : Has not overflowed (0)
    b15:b11 Reserved   - Read only bit
    b10     DLC        - Detect Loss of Carrier                       : Has not detected (0)
    b9      CD         - Delayed Collision Detect                     : Has not detected (0)
    b8      TRO        - Transmit Retry Over                          : Has not detected (0)
    b7      RMAF       - Receive Multicast Address Frame              : Has not received (0)
    b6      CEEF       - Carrier Extension Error                      : Has not occurred (0)
    b5      CELF       - Carrier Extension Loss                       : Has not occurred (0)
    b4      RRF        - Receive Residual-Bit Frame                   : Has not received (0)
    b3      RTLF       - Receive Too-Long Frame                       : Has not received (0)
    b2      RTSF       - Receive Too-Short Frame                      : Has not received (0)
    b1      PRE        - PHY-LSI Receive Error                        : Has not detected (0)
    b0      CERF       - CRC Error on Received Frame                  : Has not detected (0) */
    ETHER.EESR0          = 0xFF7F07FF;      /* Clear all status (by writing 1) */

    /* Ethernet Controller (ETHER)
    EESIPR - E-MAC/E-DMAC Status Interrupt Permission Register
    b31:b0             - Refer to EESR comment : Enable/Disable (1/0) */
    ETHER.EESIPR0   = 0x00040000;      /* b18     FR         - Frame Reception (1) */

    /* Ethernet Controller (ETHER)
    CheckSum Mode Register */
    ETHER.CSMR      = 0x00000000;

    /* Ethernet Controller (ETHER)
    ECSR - E-MAC Status Register
    b31:b5  Reserved   - Read only bit
    b4      PFROI      - PAUSE Frame Retransmit Retry Over            : Has not exceeded the upper limit (0)
    b3:b1   Reserved   - Read only bit
    b0      ICD        - Illegal Carrier Detection                    : Has not detected (0) */
    ETHER.ECSR0            = 0x00000011;      /* Clear all status (clear by writing 1) */

    /* Ethernet Controller (ETHER)
    ECSIPR - E-MAC Interrupt Permission Register
    b31:b0             - Refer to ECSR comment : Enable (1) or Disable (0) */
    ETHER.ECSIPR0     = 0;

    /* ==== EthernetŠ„‚è?ž‚Ýƒnƒ“ƒhƒ‰“o˜^ ==== */
    R_INTC_RegistIntFunc(INTC_ID_ETHERI, INT_Ether);

    /* ==== EthernetŠ„‚è?ž‚Ý—D?æ“x=6?Ý’è ==== */
    R_INTC_SetPriority(INTC_ID_ETHERI, 6);

    /* ==== Enables the E-DMAC interrupt === */
    R_INTC_Enable(INTC_ID_ETHERI);

    /* Ethernet Controller (ETHER)
    ECMR - E-MAC Mode Register
    b6      RE         - Reception Enable    : Enable (1)
    b5      TE         - Transmission Enable : Enable (1) */
//    MAC.ECMR0.BIT.RE    = 1;
//    MAC.ECMR0.BIT.TE    = 1;
	ETHER.ECMR0 = ETHER.ECMR0 | 0x00000060;


    /* ==== Enable transmission/reception ==== */
    /* Ethernet Controller (ETHER)
    EDRRR - E-DMAC Receive Request Register
    b31:b1  Reserved   - Read only bit
    b0      RR         - Receive Request : The E-DMAC starts receive operation (1) */
    if(ETHER.EDRRR0 == 0)
    {
        ETHER.EDRRR0    = 1;
    }
}

/******************************************************************************
* ID            : ?|
* Outline       : Interrupt handler (Transmission)
* Include       : none
* Function Name : lan_send_handler_isr
* Description   : Interrupt handler related to transmission regarding E-DMAC(EESR)
* Argument      : uint32_t status
* Return Value  : none
******************************************************************************/
void lan_send_handler_isr (uint32_t status)
{

    /* Please add the process as needed */

}

/******************************************************************************
* ID            : ?|
* Outline       : Interrupt handler (Reception)
* Include       : none
* Function Name : lan_recv_handler_isr
* Description   : Interrupt handler related to reception regarding E-DMAC (EESR)
* Argument      : uint32_t status
* Return Value  : none
******************************************************************************/
extern xSemaphoreHandle xEMACRxEventSemaphore;
void lan_recv_handler_isr (uint32_t status)
{

//    tfNotifyInterfaceIsr( tlInterfaceHandle, 1, 0, 0, 0 );
	xSemaphoreGiveFromISR( xEMACRxEventSemaphore, NULL );
    rx_interrupt_cnt++;
//	printf("ISR %d\n", rx_interrupt_cnt);
    /* Please add the process as needed */

}

/******************************************************************************
* ID            : ?|
* Outline       : Interrupt handler (E-MAC)
* Function Name : lan_etherc_handler_isr
* Description   : E-MAC interrupt function
* Argument      : uint32_t status
* Return Value  : none
******************************************************************************/
void lan_etherc_handler_isr (uint32_t status)
{

    /* Please add the process as needed */

}


/* H'CA0 Ether */
void INT_Ether(uint32_t dummy)
{
    uint32_t stat_edmac;
    uint32_t stat_EtherC;
    volatile uint32_t w;

       /* ---- Clear the interrupt request flag ---- */
    stat_edmac = ETHER.EESR0 & ETHER.EESIPR0;    /* Targets are restricted to allowed interrupts */
    ETHER.EESR0 = stat_edmac;
       /* ==== Transmission-related ==== */
    if (stat_edmac & EDMAC_EESIPR_INI_SEND)
    {
        lan_send_handler_isr(stat_edmac & EDMAC_EESIPR_INI_SEND);
    }
       /* ==== Reception-related ==== */
    if (stat_edmac & EDMAC_EESIPR_INI_RECV)
    {
        lan_recv_handler_isr( stat_edmac & EDMAC_EESIPR_INI_RECV );
    }
    /* ==== E-MAC-related ==== */
    if (stat_edmac & EDMAC_EESIPR_INI_EtherC)
    {
        /* ---- Clear the interrupt request flag ---- */
        stat_EtherC = ETHER.ECSR0 & ETHER.ECSIPR0;    /* Targets are restricted to allowed interrupts */
        ETHER.ECSR0   = stat_EtherC;
        lan_etherc_handler_isr(stat_EtherC);
    }
    /* ---- Wait the priority judging time ---- */
    for (w = 0; w < PCLK_5CYC; w++)
    {
        /* DO NOTHING */
    }
}

/******************************************************************************
* ID            : ?|
* Outline       : link status check
* Function Name : lan_link_check
* Description   : PHY Link Status Check function
* Argument      : none
* Return Value  : none
******************************************************************************/
void lan_link_check(void)
{
    int32_t             link;

    link = phy_linkcheck();

    if (link != link_status)
    {
        if ((FULL_TX == link) || (FULL_10M == link))
        {
        	ETHER.ECMR0 = ETHER.ECMR0 | 0x00000002;               /* Set to full-duplex mode */
        }
        else
        {
	        ETHER.ECMR0 = ETHER.ECMR0 & 0xfffffffd;               /* Set to full-duplex mode */
        }
        link_status = link;
        printf(link_message[link]);
    }
}

/* End of file */
