OUTPUT_FORMAT("elf32-littlearm", "elf32-bigarm", "elf32-littlearm")
OUTPUT_ARCH(arm)
ENTRY(start)

/* Base Address RAM Memory Table 10-Mbyte on-chip RAM */  
/* Total RAM available 10112K */

MEMORY {
	IRAM  (rwx)   : ORIGIN = 0x20000000, LENGTH = 0x00300000
	SDRAM (rwx)   : ORIGIN = 0x48000000, LENGTH = 0x02000000
	ROM   (rw)    : ORIGIN = 0x18000000, LENGTH = 0x02000000
}

PROGRAM_STACK_SIZE    = 0x1000; 	/* Application stack                  */
IRQ_STACK_SIZE	      = 0x0100;     /* IRQ mode stack                     */
FIQ_STACK_SIZE	      = 0x0100;     /* FRQ mode stack                     */
SVC_STACK_SIZE	      = 0x0100;     /* SVC mode stack                     */
ABT_STACK_SIZE	      = 0x0100;     /* ABT mode stack                     */
TTB_SIZE              = 0x8000;     /* Level-1 Translation Table for MMU  */
	 
EXEC_BASE             = 0x20000000; /* VECTOR_TABLE located here */
MIRRORED_VECTOR_TABLE = 0x20000000; /* VECTOR_MIRROR_TABLE located here */	 
	 
SECTIONS
{
    .mvt MIRRORED_VECTOR_TABLE :
    {
         mvt_start = .;
         *mirrortable.o (.text)
         . = ALIGN(0x4);
         *(.text)
    } >IRAM

	.rodata /* EXEC_BASE + 0x35000*/ :
	{
		*(.state_mem)
		*(.state_mem.*)	
		*(.rodata)
		*(.rodata.*)
		.	= ALIGN(0x8);

   		address_start_data_ROM = .;
	    *(.data)
		*(.data.*)
		*(.got.plt) *(.got)        
		.	= ALIGN(0x8);
		address_end_data_ROM = .;
        PROVIDE(end = .);
	} >IRAM

	.data address_end_data_ROM : 
	{
		address_start_data_RAM = . ;
		.  += (address_end_data_ROM - address_start_data_ROM);
		address_end_data_RAM = .;
	} >IRAM
	
	.bss  ALIGN(0x4):
	{
         _bss = .;
         PROVIDE(__bss_start__ = .);         
         *(.bss)
         *(.bss.**)
         *(COMMON)
         PROVIDE(__bss_end__ = .);
		.	= ALIGN(0x8);         
         _ebss = .;
         _end = .;
	} >IRAM
     
	.irq_stack ALIGN(0x4) :
	{
		irq_stack_start = .;
		.  += IRQ_STACK_SIZE;
		.	= ALIGN(0x4);
		irq_stack_end = .;
	} >IRAM
		
	.fiq_stack ALIGN(0x4) :
	{
		fiq_stack_start = .;
		.  += FIQ_STACK_SIZE;
		.	= ALIGN(0x4);
		fiq_stack_end = .;
	} >IRAM
		
	.svc_stack ALIGN(0x4) :
	{
		PROVIDE(svc_stack_start = .);		
		.  += SVC_STACK_SIZE;
		.	= ALIGN(0x4);
		svc_stack_end = .;
	} >IRAM
		
	.abt_stack ALIGN(0x4) :
	{
		abt_stack_start = .;
		.  += ABT_STACK_SIZE;
		.	= ALIGN(0x4);
		abt_stack_end = .;
	} >IRAM
      
	.program_stack ALIGN(0x4):
	{
		program_stack_start = .;
		.  += PROGRAM_STACK_SIZE;
		.	= ALIGN(0x4);
		program_stack_end = .; 
	} >IRAM	
    
	.ttb_mmu1 ALIGN(0x4000):
	{
		ttb_mmu1_base = .;
		.  += TTB_SIZE;
		.	= ALIGN(0x4);
		ttb_mmu1_end = .;
	} >IRAM

	.BSS_DMAC_SAMPLE_SDRAM (NOLOAD) : {} >SDRAM
	.BSS_DMAC_SAMPLE_INTERNAL_RAM 0x20500000 (NOLOAD) : {}
}