/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
*******************************************************************************/
/*******************************************************************************
* Copyright (C) 2013 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/******************************************************************************
* File Name     : sample_riic_rza1h_rsk.c
* Device(s)     : RZ/A1H RSK+RZA1H
* H/W Platform  : RSK+RZA1H CPU Board
* Description   : RSK+RZA1H RI2C Access sample,
*               :
*               : DEVDRV_CH_0 RSK TFT APP BOARD YROK77210C000BE devices
*               :  Capacitive touch panel controller
*               :  PanelBus~(TM) flat panel display
*               :
*               : DEVDRV_CH_0 RSK TFT APP BOARD YROK77210C000BE devices
*               :  MAX9856 is a high-performance, low-power stereo audio CODEC
*               :  2 * CMOS 8 bit parallel input/output port expansion for I2C
*               :  Renesas two-wire serial interface EEPROM
******************************************************************************/
/*******************************************************************************
* History       : DD.MM.YYYY Version Description
*               : 18.06.2013 1.00
*******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include <stdio.h>
#include <string.h>

#include "r_typedefs.h"                   /* Default  type definition header */
#include "iodefine.h"                            /* I/O Register root header */
#include "dev_drv.h"                                 /* Device driver header */
#include "devdrv_riic.h"                               /* RIIC Driver Header */
#include "devdrv_intc.h"                               /* INTC Driver Header */
#include "sample_riic_rza1h_rsk_i2c.h"   /* I2C RSK+RZA1H Eval-board header */
#include "rza_io_regrw.h"            /* Low level register read/write header */

/******************************************************************************
Typedef definitions
******************************************************************************/

/******************************************************************************
Macro definitions
******************************************************************************/
#define SAMPLE_RIIC_RWCODE_W        (0)               /* IIC data write code */
#define SAMPLE_RIIC_RWCODE_R        (1)               /* IIC data read code  */
#define DELAY_TIME_1MS       (260000uL)                     /* Sampling loop */


/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/
uint8_t  g_w_byte;
uint8_t  g_config_io;
uint8_t  g_r_buffer = 0u;
uint8_t  port_x_state = 0u;

int32_t touch_read_request = 0;

/******************************************************************************
Private global variables and functions
******************************************************************************/






/******************************************************************************
* Function Name: RIIC0_PORT_Init
* Description  : Initialises channel 0 i2c port register for this board
* Arguments    : None
* Return Value : None
******************************************************************************/
static void RIIC0_PORT_Init(void)
{
    /* ---- P1_0 : SCL0 ---- */
    /* Port initialize */
    RZA_IO_RegWrite_16(&GPIO.PIBC1,  0, GPIO_PIBC1_PIBC10_SHIFT,   GPIO_PIBC1_PIBC10);
    RZA_IO_RegWrite_16(&GPIO.PBDC1,  1, GPIO_PBDC1_PBDC10_SHIFT,   GPIO_PBDC1_PBDC10);
    RZA_IO_RegWrite_16(&GPIO.PM1,    1, GPIO_PM1_PM10_SHIFT,       GPIO_PM1_PM10);
    RZA_IO_RegWrite_16(&GPIO.PMC1,   0, GPIO_PMC1_PMC10_SHIFT,     GPIO_PMC1_PMC10);
    RZA_IO_RegWrite_16(&GPIO.PIPC1,  1, GPIO_PIPC1_PIPC10_SHIFT,   GPIO_PIPC1_PIPC10);
    /* Port mode : Multiplex mode                     */
    /* Port function setting : 1st multiplex function */
    /* I/O control mode : Peripheral function         */
    /* Bidirectional mode : Enable                    */
    RZA_IO_RegWrite_16(&GPIO.PBDC1,  1, GPIO_PBDC1_PBDC10_SHIFT,   GPIO_PBDC1_PBDC10);
    RZA_IO_RegWrite_16(&GPIO.PFC1,   0, GPIO_PFC1_PFC10_SHIFT,     GPIO_PFC1_PFC10);
    RZA_IO_RegWrite_16(&GPIO.PFCE1,  0, GPIO_PFCE1_PFCE10_SHIFT,   GPIO_PFCE1_PFCE10);
    RZA_IO_RegWrite_16(&GPIO.PFCAE1, 0, GPIO_PFCAE1_PFCAE10_SHIFT, GPIO_PFCAE1_PFCAE10);
    RZA_IO_RegWrite_16(&GPIO.PIPC1,  1, GPIO_PIPC1_PIPC10_SHIFT,   GPIO_PIPC1_PIPC10);
    RZA_IO_RegWrite_16(&GPIO.PMC1,   1, GPIO_PMC1_PMC10_SHIFT,     GPIO_PMC1_PMC10);

    /* ---- P1_1 : SDA0 ---- */
    /* Port initialise */
    RZA_IO_RegWrite_16(&GPIO.PIBC1,  0, GPIO_PIBC1_PIBC11_SHIFT,   GPIO_PIBC1_PIBC11);
    RZA_IO_RegWrite_16(&GPIO.PBDC1,  1, GPIO_PBDC1_PBDC11_SHIFT,   GPIO_PBDC1_PBDC11);
    RZA_IO_RegWrite_16(&GPIO.PM1,    1, GPIO_PM1_PM11_SHIFT,       GPIO_PM1_PM11);
    RZA_IO_RegWrite_16(&GPIO.PMC1,   0, GPIO_PMC1_PMC11_SHIFT,     GPIO_PMC1_PMC11);
    RZA_IO_RegWrite_16(&GPIO.PIPC1,  1, GPIO_PIPC1_PIPC11_SHIFT,   GPIO_PIPC1_PIPC11);
    /* Port mode : Multiplex mode                     */
    /* Port function setting : 1st multiplex function */
    /* I/O control mode : Peripheral function         */
    /* Bidirectional mode : Enable                    */
    RZA_IO_RegWrite_16(&GPIO.PBDC1,  1, GPIO_PBDC1_PBDC11_SHIFT,   GPIO_PBDC1_PBDC11);
    RZA_IO_RegWrite_16(&GPIO.PFC1,   0, GPIO_PFC1_PFC11_SHIFT,     GPIO_PFC1_PFC11);
    RZA_IO_RegWrite_16(&GPIO.PFCE1,  0, GPIO_PFCE1_PFCE11_SHIFT,   GPIO_PFCE1_PFCE11);
    RZA_IO_RegWrite_16(&GPIO.PFCAE1, 0, GPIO_PFCAE1_PFCAE11_SHIFT, GPIO_PFCAE1_PFCAE11);
    RZA_IO_RegWrite_16(&GPIO.PIPC1,  1, GPIO_PIPC1_PIPC11_SHIFT,   GPIO_PIPC1_PIPC11);
    RZA_IO_RegWrite_16(&GPIO.PMC1,   1, GPIO_PMC1_PMC11_SHIFT,     GPIO_PMC1_PMC11);

    /* ---- P9_5 : EEPROM_WC ---- */
    /* Port initialise */
	PORT9.PMCn.BIT.PMCn5 = 0;
	PORT9.PIPCn.BIT.PIPCn5 = 0;
	PORT9.PMn.BIT.PMn5 = 0;
	//Disable EEPROM write
	PORT9.Pn.BIT.Pn5 = 1;
}



/******************************************************************************
* Function Name: R_RIIC_Init_CH0
* Description  : Initialises channel 0 i2c for this board
* Arguments    : None
* Return Value : None
******************************************************************************/
void R_RIIC_Init_CH0(void)
{
	uint32_t cks;
	uint32_t brl;
	uint32_t brh;

	RIIC0_PORT_Init();



	R_RIIC_Init(DEVDRV_CH_0);



}

/******************************************************************************
* Function Name: RIIC1_PORT_Init
* Description  : Initialises channel 1 port register for this board
* Arguments    : None
* Return Value : None
******************************************************************************/
static void RIIC1_PORT_Init(void)
{

	/* ---- P1_2 : SCL1 ---- */
    /* Port initialise */
    RZA_IO_RegWrite_16(&GPIO.PIBC1,  0, GPIO_PIBC1_PIBC12_SHIFT,   GPIO_PIBC1_PIBC12);
    RZA_IO_RegWrite_16(&GPIO.PBDC1,  1, GPIO_PBDC1_PBDC12_SHIFT,   GPIO_PBDC1_PBDC12);
    RZA_IO_RegWrite_16(&GPIO.PM1,    1, GPIO_PM1_PM12_SHIFT,       GPIO_PM1_PM12);
    RZA_IO_RegWrite_16(&GPIO.PMC1,   0, GPIO_PMC1_PMC12_SHIFT,     GPIO_PMC1_PMC12);
    RZA_IO_RegWrite_16(&GPIO.PIPC1,  1, GPIO_PIPC1_PIPC12_SHIFT,   GPIO_PIPC1_PIPC12);
    /* Port mode : Multiplex mode                     */
    /* Port function setting : 1st multiplex function */
    /* I/O control mode : Peripheral function         */
    /* Bidirectional mode : Enable                    */
    RZA_IO_RegWrite_16(&GPIO.PBDC1,  1, GPIO_PBDC1_PBDC12_SHIFT,   GPIO_PBDC1_PBDC12);
    RZA_IO_RegWrite_16(&GPIO.PFC1,   0, GPIO_PFC1_PFC12_SHIFT,     GPIO_PFC1_PFC12);
    RZA_IO_RegWrite_16(&GPIO.PFCE1,  0, GPIO_PFCE1_PFCE12_SHIFT,   GPIO_PFCE1_PFCE12);
    RZA_IO_RegWrite_16(&GPIO.PFCAE1, 0, GPIO_PFCAE1_PFCAE12_SHIFT, GPIO_PFCAE1_PFCAE12);
    RZA_IO_RegWrite_16(&GPIO.PIPC1,  1, GPIO_PIPC1_PIPC12_SHIFT,   GPIO_PIPC1_PIPC12);
    RZA_IO_RegWrite_16(&GPIO.PMC1,   1, GPIO_PMC1_PMC12_SHIFT,     GPIO_PMC1_PMC12);

    /* ---- P1_3 : SDA3 ---- */
    /* Port initialise */
    RZA_IO_RegWrite_16(&GPIO.PIBC1,  0, GPIO_PIBC1_PIBC13_SHIFT,   GPIO_PIBC1_PIBC13);
    RZA_IO_RegWrite_16(&GPIO.PBDC1,  1, GPIO_PBDC1_PBDC13_SHIFT,   GPIO_PBDC1_PBDC13);
    RZA_IO_RegWrite_16(&GPIO.PM1,    1, GPIO_PM1_PM13_SHIFT,       GPIO_PM1_PM13);
    RZA_IO_RegWrite_16(&GPIO.PMC1,   0, GPIO_PMC1_PMC13_SHIFT,     GPIO_PMC1_PMC13);
    RZA_IO_RegWrite_16(&GPIO.PIPC1,  1, GPIO_PIPC1_PIPC13_SHIFT,   GPIO_PIPC1_PIPC13);
    /* Port mode : Multiplex mode                     */
    /* Port function setting : 1st multiplex function */
    /* I/O control mode : Peripheral function         */
    /* Bidirectional mode : Enable                    */
    RZA_IO_RegWrite_16(&GPIO.PBDC1,  1, GPIO_PBDC1_PBDC13_SHIFT,   GPIO_PBDC1_PBDC13);
    RZA_IO_RegWrite_16(&GPIO.PFC1,   0, GPIO_PFC1_PFC13_SHIFT,     GPIO_PFC1_PFC13);
    RZA_IO_RegWrite_16(&GPIO.PFCE1,  0, GPIO_PFCE1_PFCE13_SHIFT,   GPIO_PFCE1_PFCE13);
    RZA_IO_RegWrite_16(&GPIO.PFCAE1, 0, GPIO_PFCAE1_PFCAE13_SHIFT, GPIO_PFCAE1_PFCAE13);
    RZA_IO_RegWrite_16(&GPIO.PIPC1,  1, GPIO_PIPC1_PIPC13_SHIFT,   GPIO_PIPC1_PIPC13);
    RZA_IO_RegWrite_16(&GPIO.PMC1,   1, GPIO_PMC1_PMC13_SHIFT,     GPIO_PMC1_PMC13);

}

/******************************************************************************
* Function Name: R_RIIC_Init_CH1
* Description  : Initialises channel 1 i2c for this board
* Arguments    : None
* Return Value : None
******************************************************************************/
void R_RIIC_Init_CH1(void)
{
	uint32_t cks;
	uint32_t brl;
	uint32_t brh;
	uint8_t  iic_dev_io;
	uint8_t  iic_w_value = 0xff;

	uint8_t buf[3] = {0,0,0};
	uint8_t buffer = 0;

	RIIC1_PORT_Init();


	R_RIIC_Init(DEVDRV_CH_1);


}


/******************************************************************************
* Function Name: R_RIIC_rza1h_rsk_init
* Description  : Manages the initialisation of the i2c channels
* Arguments    : None
* Return Value : None
******************************************************************************/
void Pekoe_RIIC_init(void)
{


	RIIC0_PORT_Init();
	Userdef_RIIC0_Init();

	RIIC1_PORT_Init();
	Userdef_RIIC1_Init();

//    printf("RIIC Initialize complete.\n");
//    printf("\n");
}

/********************************************
 * Function Name: R_RIIC_CAMERA_write
 * Description  : Configuration register for Camera
 ********************************************/
int32_t R_RIIC_CAMERA_write(uint32_t channel,
		                       uint8_t d_adr,
		                       uint16_t w_adr,
		                       uint8_t data)
{
    int32_t  ret;
    uint8_t  buf[3];
    uint8_t  buf2[3];
    uint8_t  buf3[3];


    buf[0] = (uint8_t)(d_adr | SAMPLE_RIIC_RWCODE_W);
    buf2[0] = (uint8_t)(w_adr  & 0x00ff);
    buf2[1] = (uint8_t)(data  & 0x00ff);
    buf3[0] = (uint8_t)(data  & 0xff);

    ret = R_RIIC_SendCond(channel, RIIC_TX_MODE_START);
    if (DEVDRV_SUCCESS == ret)
    {
    	ret = R_RIIC_Write(channel, buf, 1);
    }
    if (DEVDRV_SUCCESS == ret)
    {
        ret = R_RIIC_Write(channel, buf2, 2);
    }

    if (DEVDRV_SUCCESS == ret)
    {
        ret =R_RIIC_SendCond(channel, RIIC_TX_MODE_STOP);
    }
    if (DEVDRV_SUCCESS == ret)
    {
        ret = R_RIIC_DetectStop(channel);
    }

    return ret;
}

/* End of File */

