/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2014 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name     : bsc_userdef.c
* Device(s)     : RZ/A1H (R7S721001)
* Tool-Chain    : GNUARM-RZv13.01-EABI
* H/W Platform  : RSK+RZA1H CPU Board
* Description   : Common driver (User define function)
*******************************************************************************/
/*******************************************************************************
* History       : DD.MM.YYYY Version Description
*               : 18.06.2013 1.00
*               : 21.03.2014 2.00
*******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
/* Default  type definition header */
#include "r_typedefs.h"
/* Device driver header */
#include "dev_drv.h"
/* Common Driver header */
#include "devdrv_common.h"
/* I/O Register root header */
#include "iodefine.h"
/* Bus State Controller User header */
#include "bsc_userdef.h"
/* Interchangeable compiler specific header */
#include "compiler_settings.h"


// Burst read/burst write (burst length 1): CAS Latency = 2
#define SDRAM_MODE_CS3    (*(volatile uint16_t *)(0x3FFFE040))

/******************************************************************************
* Function Name: Userdef_BSC_CS3Init
* Description  : This is the user-defined function called by the R_BSC_Init
*              : function. The setting for initialisation of the BSC in the CS3
*              : space is required. In this sample code, the setting to use
*              : the SDRAM in the CS3 space is executed. Sets the BSC to
*              : connect the MT48LC16M16A2P-6A IT to the CS3 space with 16-bit
*              : bus width.
* Arguments    : none
* Return Value : none
******************************************************************************/
void Userdef_BSC_CS3Init(void)
{

volatile int32_t cnt;

	/* NOTE: Secure wait time according to SDRAM specification. */
	cnt = 100;
	while (cnt-- > 0)
	{
	        /* wait */
	}

	/* ==== CS3BCR settings ==== */
	BSC.CS3BCR.LONG = 0x00004C00uL;
		/* Idle Cycles between Write-read Cycles  */
		/* and Write-write Cycles : 0 idle cycles */
		/* Memory type :SDRAM                     */
		/* Data Bus Size : 16-bit                 */

	/* ==== CS3WCR settings ==== */
	BSC.CS3WCR.SDRAM.LONG = 0x00002492uL;
		/* Precharge completion wait cycles: 1 cycle     */
		/* Wait cycles between ACTV command              */
		/* and READ(A)/WRITE(A) command : 1 cycles       */
		/* CAS latency for Area 3 : 2 cycles             */
		/* Auto-precharge startup wait cycles : 2 cycles */
		/* Idle cycles from REF command/self-refresh     */
		/* Release to ACTV/REF/MRS command : 5 cycles    */

	/* ==== SDCR settings ==== */
//	BSC.SDCR = 0x00120812uL;
	BSC.SDCR.LONG = 0x00000811uL; /* only CS3 used, 13 bits Row / 9 bits column*/
		/* Row address for Area 2 : 13-bit    */
		/* Column Address for Area 2 : 10-bit */
		/* Refresh Control :Refresh           */
		/* RMODE :Auto-refresh is performed   */
		/* BACTV :Auto-precharge mode         */
		/* Row address for Area 3 : 13-bit    */
		/* Column Address for Area 3 : 10-bit */

	/* ==== RTCOR settings ==== */
	BSC.RTCOR.LONG = 0xA55A0020uL;
		/* 7.813usec /240nsec            */
		/*   = 32(0x20)cycles per refresh */

	/* ==== RTCSR settings ==== */
	BSC.RTCSR.LONG = 0xA55A0010uL;
		/* Initialization sequence start */
		/* Clock select CKIO-phy/16      */
		/* Refresh count :Once           */

	/* ==== Written in SDRAM Mode Register ==== */
	SDRAM_MODE_CS3 = 0x0000u;
		/* The writing data is arbitrary            */
		/* SDRAM mode register setting CS3 space    */
		/* Burst read (burst length 1)./Burst write */

}


/* End of File */

