/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2014 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name   : sample_riic_command.c
* $Rev: 1121 $
* $Date:: 2014-08-06 17:09:53 +0900#$
* Description : RIIC sample program
*******************************************************************************/


/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include <stdio.h>
#include "r_typedefs.h"
#include "dev_drv.h"                /* Device Driver common header */
#include "devdrv_riic.h"            /* RIIC Driver Header */
#include "sample_riic.h"
#include "command.h"
#include "iodefine.h"
#include "rza_io_regrw.h"

#ifdef PEKOE_BOARD
/******************************************************************************
Typedef definitions
******************************************************************************/


/******************************************************************************
Macro definitions
******************************************************************************/
#define SAMPLE_RIIC_DEVICE_CODE (0xA0)  /* EEPROM device code         : b'1010 */
#define SAMPLE_RIIC_DEVICE_ADDR (0x01)  /* EEPROM device address code : b'001  */

#define SAMPLE_RIIC_DATA_SIZE   (128)   /* Data receive RAM size */
#define SAMPLE_RIIC_PAGE_SIZE   (16)    /* Maximum write in page write mode */

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/

int32_t WriteDataToEEPROM(unsigned char *params, int size)
{
    int32_t  ret;
    uint8_t  w_offset,i, w_size;
    uint8_t *w_ptr;
    uint8_t check_sum = 0;

    for(i = 0; i < size-1; i++)
    	check_sum += params[i];
    check_sum = check_sum ^ 0xFF;
    params[size-1] = check_sum;

    w_offset = 0;
    while(1)
	{
		w_ptr = params + w_offset;
		w_size = SAMPLE_RIIC_PAGE_SIZE;

		if( (w_offset + SAMPLE_RIIC_PAGE_SIZE) > size)
		{
			w_size = size - w_offset;
		}
    	/* ==== Write to EEPROM ==== */
    	ret = Sample_RIIC_EepWrite(DEVDRV_CH_0,         /* RIIC channel        */
                               SAMPLE_RIIC_DEVICE_CODE, /* Device code         */
                               SAMPLE_RIIC_DEVICE_ADDR, /* Device address code */
                               w_offset,                /* Write start address */
                               w_size,   				/* Write data size     */
                               w_ptr);        			/* Write data          */

    	/* When the NACK is received at the time of data transmission */
    	if (DEVDRV_ERROR_RIIC_NACK == ret)
   		{
    		printf("error: EEPROM write error (NACK).\n");
    		return COMMAND_ERROR;
   		}

    	/* ==== Wait until writing has been completed inside the EEPROM ==== */
    	Sample_RIIC_EepAckPolling(DEVDRV_CH_0, SAMPLE_RIIC_DEVICE_CODE, SAMPLE_RIIC_DEVICE_ADDR);
    	w_offset += w_size;
    	if(w_offset >= size) break;;
	}

	return w_offset;
}

int32_t ReadDataFromEEPROM(unsigned char *params, int size)
{
    int32_t  ret, i;
    uint8_t check_sum = 0, rx_check_sum = 0;
    ret = Sample_RIIC_EepRead(DEVDRV_CH_0,                  /* RIIC channel        */
                              SAMPLE_RIIC_DEVICE_CODE,  /* Device code         */
                              SAMPLE_RIIC_DEVICE_ADDR,  /* Device address code */
                              0x0000,                   /* Read start address  */
                              size,    /* Read data size      */
                              params);       /* Read data           */

    for(i = 0; i < size-1; i++)
    	check_sum += params[i];
    check_sum = check_sum ^ 0xFF;
    rx_check_sum = params[size-1];
    if(check_sum != rx_check_sum)
    {
    	printf("error: EEPROM Config Data Error\n");
        return COMMAND_ERROR;
    }

    /* When the NACK is received at the time data transmission */
    if (DEVDRV_ERROR_RIIC_NACK == ret)
    {
        printf("error: EEPROM read error (NACK).\n");
        return COMMAND_ERROR;
    }

    return size;
}

/* End of File */

#endif
