#ifndef __BENCHMARK_H__
#define __BENCHMARK_H__

/* performance Counters */
#define GNU_TOOLS 	1

#define PMON_COUNTER_0		0
#define PMON_COUNTER_1		1
#define PMON_COUNTER_2		2
#define PMON_COUNTER_3		3

/* Performance Event IDs */

#define PMNC_SW_INCR			0x00 //counters:1,2,3,4 um:zero minimum:500 name:PMNC_SW_INCR : Software increment of PMNC registers
#define IFETCH_MISS				0x01 //counters:1,2,3,4 um:zero minimum:500 name:IFETCH_MISS : Instruction fetch misses from cache or normal cacheable memory
#define ITLB_MISS				0x02 //counters:1,2,3,4 um:zero minimum:500 name:ITLB_MISS : Instruction fetch misses from TLB
#define DCACHE_REFILL	 		0x03 //counters:1,2,3,4 um:zero minimum:500 name:DCACHE_REFILL : Data R/W operation that causes a refill from cache or normal cacheable memory
#define DCACHE_ACCESS 			0x04 //counters:1,2,3,4 um:zero minimum:500 name:DCACHE_ACCESS : Data R/W from cache
#define DTLB_REFILL 			0x05 //counters:1,2,3,4 um:zero minimum:500 name:DTLB_REFILL : Data R/W that causes a TLB refill
#define DREAD	 				0x06 //counters:1,2,3,4 um:zero minimum:500 name:DREAD : Data read architecturally executed (note: architecturally executed = for instructions that are unconditional or that pass 			 the condition code)
#define DWRITE 					0x07 //counters:1,2,3,4 um:zero minimum:500 name:DWRITE : Data write architecturally executed
#define INSTR_EXECUTED	 		0x08 //counters:1,2,3,4 um:zero minimum:500 name:INSTR_EXECUTED : All executed instructions
#define EXC_TAKEN 				0x09 //counters:1,2,3,4 um:zero minimum:500 name:EXC_TAKEN : Exception taken
#define EXC_EXECUTED 			0x0A //counters:1,2,3,4 um:zero minimum:500 name:EXC_EXECUTED : Exception return architecturally executed
#define CID_WRITE	 			0x0B //counters:1,2,3,4 um:zero minimum:500 name:CID_WRITE : Instruction that writes to the Context ID Register architecturally executed
#define PC_WRITE 				0x0C //counters:1,2,3,4 um:zero minimum:500 name:PC_WRITE : SW change of PC, architecturally executed (not by exceptions)
#define PC_IMM_BRANCH 			0x0D //counters:1,2,3,4 um:zero minimum:500 name:PC_IMM_BRANCH : Immediate branch instruction executed (taken or not)
#define PC_PROC_RETURN 			0x0E //counters:1,2,3,4 um:zero minimum:500 name:PC_PROC_RETURN : Procedure return architecturally executed (not by exceptions)
#define UNALIGNED_ACCESS	 	0x0F //counters:1,2,3,4 um:zero minimum:500 name:UNALIGNED_ACCESS : Unaligned access architecturally executed
#define PC_BRANCH_MIS_PRED 		0x10 //counters:1,2,3,4 um:zero minimum:500 name:PC_BRANCH_MIS_PRED : Branch mispredicted or  not predicted. Counts pipeline flushes because of misprediction
#define PC_BRANCH_MIS_USED	 	0x12 //counters:1,2,3,4 um:zero minimum:500 name:PC_BRANCH_MIS_USED : Branch or change in program flow that could have been predicted
#define WRITE_BUFFER_FULL 		0x40 //counters:1,2,3,4 um:zero minimum:500 name:WRITE_BUFFER_FULL : Any write buffer full cycle
#define L2_STORE_MERGED		 	0x41 //counters:1,2,3,4 um:zero minimum:500 name:L2_STORE_MERGED : Any store that is merged in L2 cache
#define L2_STORE_BUFF 			0x42 //counters:1,2,3,4 um:zero minimum:500 name:L2_STORE_BUFF : Any bufferable store from load/store to L2 cache
#define L2_ACCESS 				0x43 //counters:1,2,3,4 um:zero minimum:500 name:L2_ACCESS : Any access to L2 cache
#define L2_CACH_MISS	 		0x44 //counters:1,2,3,4 um:zero minimum:500 name:L2_CACH_MISS : Any cacheable miss in L2 cache
#define AXI_READ_CYCLES 		0x45 //counters:1,2,3,4 um:zero minimum:500 name:AXI_READ_CYCLES : Number of cycles for an active AXI read
#define AXI_WRITE_CYCLES 		0x46 //counters:1,2,3,4 um:zero minimum:500 name:AXI_WRITE_CYCLES : Number of cycles for an active AXI write
#define MEMORY_REPLAY	 		0x47 //counters:1,2,3,4 um:zero minimum:500 name:MEMORY_REPLAY : Any replay event in the memory subsystem
#define UNALIGNED_ACCESS_REPLAY 0x48 //counters:1,2,3,4 um:zero minimum:500 name:UNALIGNED_ACCESS_REPLAY : Unaligned access that causes a replay
#define L1_DATA_MISS	 		0x49 //counters:1,2,3,4 um:zero minimum:500 name:L1_DATA_MISS : L1 data cache miss as a result of the hashing algorithm
#define L1_INST_MISS 			0x4A //counters:1,2,3,4 um:zero minimum:500 name:L1_INST_MISS : L1 instruction cache miss as a result of the hashing algorithm
#define L1_DATA_COLORING 		0x4B //counters:1,2,3,4 um:zero minimum:500 name:L1_DATA_COLORING : L1 data access in which a page coloring alias occurs
#define L1_NEON_DATA	 		0x4C //counters:1,2,3,4 um:zero minimum:500 name:L1_NEON_DATA : NEON data access that hits L1 cache
#define L1_NEON_CACH_DATA	 	0x4D //counters:1,2,3,4 um:zero minimum:500 name:L1_NEON_CACH_DATA : NEON cacheable data access that hits L1 cache
#define L2_NEON 				0x4E //counters:1,2,3,4 um:zero minimum:500 name:L2_NEON : L2 access as a result of NEON memory access
#define L2_NEON_HIT 			0x4F //counters:1,2,3,4 um:zero minimum:500 name:L2_NEON_HIT : Any NEON hit in L2 cache
#define L1_INST 				0x50 //counters:1,2,3,4 um:zero minimum:500 name:L1_INST : Any L1 instruction cache access, excluding CP15 cache accesses
#define PC_RETURN_MIS_PRED	 	0x51 //counters:1,2,3,4 um:zero minimum:500 name:PC_RETURN_MIS_PRED : Return stack misprediction at return stack pop (incorrect target address)
#define PC_BRANCH_FAILED	 	0x52 //counters:1,2,3,4 um:zero minimum:500 name:PC_BRANCH_FAILED : Branch prediction misprediction
#define PC_BRANCH_TAKEN 		0x53 //counters:1,2,3,4 um:zero minimum:500 name:PC_BRANCH_TAKEN : Any predicted branch that 					    is taken
#define PC_BRANCH_EXECUTED 		0x54 //counters:1,2,3,4 um:zero minimum:500 name:PC_BRANCH_EXECUTED : Any taken branch that is executed
#define OP_EXECUTED	 			0x55 //counters:1,2,3,4 um:zero minimum:500 name:OP_EXECUTED : Number of operations executed  (in instruction or mutli-cycle instruction)
#define CYCLES_INST_STALL	 	0x56 //counters:1,2,3,4 um:zero minimum:500 name:CYCLES_INST_STALL : Cycles where no instruction available
#define CYCLES_INST 			0x57 //counters:1,2,3,4 um:zero minimum:500 name:CYCLES_INST : Number of instructions issued in a cycle
#define CYCLES_NEON_DATA_STALL  0x58 //counters:1,2,3,4 um:zero minimum:500 name:CYCLES_NEON_DATA_STALL : Number of cycles the processor waits on MRC data from NEON
#define CYCLES_NEON_INST_STALL 	0x59 //counters:1,2,3,4 um:zero minimum:500 name:CYCLES_NEON_INST_STALL : Number of cycles the processor waits on NEON instruction queue or NEON load queue
#define NEON_CYCLES 			0x5A //counters:1,2,3,4 um:zero minimum:500 name:NEON_CYCLES : Number of cycles NEON and integer processors are not idle
#define PMU0_EVENTS 			0x70 //counters:1,2,3,4 um:zero minimum:500 name:PMU0_EVENTS : Number of events from external input source PMUEXTIN[0]
#define PMU1_EVENTS 			0x71 //counters:1,2,3,4 um:zero minimum:500 name:PMU1_EVENTS : Number of events from external input source PMUEXTIN[1]
#define PMU_EVENTS 				0x72 //counters:1,2,3,4 um:zero minimum:500 name:PMU_EVENTS : Number of events from both exte  rnal input sources PMUEXTIN[0] and PMUEXTIN[1]
#define CPU_CYCLES 				0xFF //counters:0 um:zero minimum:500 name:CPU_CYCLES : Number of CPU cycles

unsigned int setUSERENRegister(void);
unsigned int ReadUSERENRegister(void);
void EnablePerfCounters(void);
void DisablePerfCounters(void);
void SelectPerfEvent(unsigned int counter, unsigned int event);
void StartPerfCounters(void);
void StopPerfCounters(void);
unsigned int GetPerfCount(unsigned int counter);
unsigned int GetCycleCount(void);

#endif
