#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "r_typedefs.h"                         /* Renesas data types header */
#include "compiler_settings.h"
#include "h264enc_api.h"

#define FIND_BITRATE
#define ENABLE_BENCHMARK
//#define WRITE_RECON_BUF
//#define WRITE_NALU

#define CTPL_H264E_VERSION	"1.1"

#ifdef ENABLE_BENCHMARK
#include "benchmark.h"
#endif

#define H264_STATE_MEM_SIZE	(1360*1024) //5000000 //3055000

#define MAX_VIDEO_WIDTH  640
#define MAX_VIDEO_HEIGHT 480
#define MAX_FRAME_SIZE	 (MAX_VIDEO_WIDTH*MAX_VIDEO_HEIGHT) + ((MAX_VIDEO_WIDTH*MAX_VIDEO_HEIGHT)/2)

#define MAX_OUT_SIZE	1520640

//#define printf

/* Input config parameters structure */
typedef struct
{
	char *infile;					//!< H.264 compressed output bitstream
	char *outfile;					//!< H.264 compressed output bitstream
	char *reconfile;

	/* Static parameters */
	int frame_width;
	int frame_height;
	int profile;
	int level;
	int chroma_format;
	int frame_content;
	int encoding_preset;
	int rc_enable;
	int search_range;
	int num_ref_frames;
	int subpel_me;
	int frame_count;

	/* Dynamic Parameters */
	int target_fr;
	int target_br;
	int disable_intra4x4;
	int disable_intra16x16;
	int disable_interskip;
	int disable_inter16x16;
	int disable_inter16x8;
	int disable_inter8x16;
	int disable_inter8x8;
	int disable_inter8x4;
	int disable_inter4x8;
	int disable_inter4x4;
	int force_I_frame;
	int disable_intra_in_inter;
	int i_qp;
	int p_qp;
	int initial_qp;
	int min_qp;
	int max_qp;
	int constrained_intra_prediction;
	int intra_period;
	int idr_period;
	int air_rate;
	int max_mbs_per_slice;
	int num_slice_groups;
	int slice_group_map_type;
	int df_send_parameters;
	int df_disable_idc;
	int df_alpha;
	int df_beta;
} h264e_input_config;

/* API data structure definitions */
tH264E_MEM_INFO mem_info;
tH264E_STATIC_PARAMS c_config;
tH264E_DYNAMIC_PARAMS d_config;
tH264E_STATUS_PARAMS s_params;
tH264E_INPUT_PARAMS  ip_params;
//tH264E_OUTPUT_PARAMS op_params;
tH264E_OUTPUT_PARAMS op_params  __attribute__((section(".BSS_H264_BS_INTERNAL_RAM"), aligned(32)));

unsigned char h264_state_buf[H264_STATE_MEM_SIZE];

#ifdef WRITE_RECON_BUF
unsigned char h264_recon_buf[MAX_FRAME_SIZE];
#endif

/* Application data structures */
h264e_input_config input_params;
unsigned char *h264e_state_memory;

/* Input/Output files */
char *f_in, *f_out;
#ifdef WRITE_NALU
char stream_file[MAX_OUT_SIZE];
#endif

int total_byte_cnt = 0;
unsigned int byte_count = 0;
unsigned int frm_count = 0;
unsigned int total_frm_count = 0;

/* Application function definitions */
void InitH264EncoderParams(h264e_input_config* params, tH264E_STATIC_PARAMS *c_params, tH264E_DYNAMIC_PARAMS *d_params);
void WriteNALUToFile(void);
void PrintConfigParams(h264e_input_config* params, tH264E_STATUS_PARAMS *s_params);
void ReadConfigParams(h264e_input_config* params, int width, int height, int frameRate, int bitRate);
unsigned char *ProcessH264Encoder(unsigned char *frameBuf, unsigned int *frameSize);

#ifdef FIND_BITRATE
float frame_rate;
float bit_rate, fps, ftime;
int target_fr;
#endif

#ifdef ENABLE_BENCHMARK
void StartBenchMark(void);
void StopBenchMark(void);
unsigned int tc,i;
unsigned int totalcycles;
unsigned int i_miss;
unsigned int d_miss;
unsigned int l2_miss;
unsigned int dtlb_miss;
double total_cycles;
#endif

int InitH264Encoder(int width, int height, int frameRate, int bitRate)
{
	int ret_val = 0;
	unsigned long memsize;

	tH264E_STATIC_PARAMS *c_params = &c_config;
	tH264E_DYNAMIC_PARAMS *d_params = &d_config;
	h264e_input_config* params = &input_params;

	if(width > MAX_VIDEO_WIDTH || height > MAX_VIDEO_HEIGHT)
		return -1;

#ifdef ENABLE_BENCHMARK
	i = setUSERENRegister();
	i = ReadUSERENRegister();
	if(i != 1)
	{
		printf("User Mode access for performance registers is not enabled\n");
	}
	EnablePerfCounters();
#endif

	ReadConfigParams(params, width, height, frameRate, bitRate);

	CTPL_H264_VENC_GetMemInfo(&mem_info);
//	printf("H.264 State Memory Info: Size = %ld Align = %d\n", mem_info.state_mem_size, mem_info.align);
	memsize = mem_info.state_mem_size + mem_info.align;
    if(memsize > sizeof(h264_state_buf))
    {
		printf("Insufficient Buffer for H.264 State Memory\n");
		return -1;
	}
	h264e_state_memory = &h264_state_buf[0];

#ifdef WRITE_RECON_BUF
	op_params.recon_frame = &h264_recon_buf[0];
#else
	op_params.recon_frame = NULL;
#endif

	if(c_params != H264_NULL && d_params != H264_NULL)
		InitH264EncoderParams(params, c_params, d_params);

	ret_val = CTPL_H264_VENC_Init(h264e_state_memory, c_params, d_params);

	if (ret_val < 0)
	{
		printf("Failed to initialize H.264 Encoder\n");
		return -1;
	}

	CTPL_H264_VENC_Status(h264e_state_memory, &s_params);

//	PrintConfigParams(params, &s_params);

#ifdef FIND_BITRATE
	total_byte_cnt = 0;
#endif

#ifdef ENABLE_BENCHMARK
	totalcycles = i_miss = d_miss = l2_miss = dtlb_miss = 0;
	total_cycles = 0.0;
#endif

	return ret_val;
}
extern int rx_interrupt_cnt, rx_task_cnt;
unsigned char *ProcessH264Encoder(unsigned char *frameBuf, unsigned int *frameSize)
{
	int ret_val, i;
	unsigned int ESPayloadSize;

#ifdef FIND_BITRATE
	byte_count = 0;
#endif

	ip_params.input_frame = frameBuf;

#ifdef ENABLE_BENCHMARK
	StartBenchMark();
	tc = 0;
#endif

	ret_val = CTPL_H264_VENC_Process(h264e_state_memory, &ip_params, &op_params);

#ifdef ENABLE_BENCHMARK
	StopBenchMark();
	total_cycles += (float)tc/1000000.0;
#endif

	frm_count++;
	total_frm_count++;

	ESPayloadSize = 0;
	for (i = 0; i < op_params.nalu_count; i++)
		ESPayloadSize += op_params.nalu_length[i] ;

	*frameSize = ESPayloadSize;

#if 0
	WriteNALUToFile();
#else
	total_byte_cnt += (int)ESPayloadSize;
#endif

//	printf("F(%3s) = %3d ", op_params.frame_type == VIDEO_P_FRAME ? "P" : op_params.idr_flag ? "IDR" : "I",frm_count);
//	printf("QP = %3d ", op_params.frame_qp);

#ifdef FIND_BITRATE
//	printf(" Bits - %5d", byte_count * 8);
#endif

//	printf("\n");
	if(!(frm_count % 100))
	{
//		printf("BitRate = Effective Frame Rate (FPS)                     : %f \n", 400 * ((float)frm_count/(float)total_cycles));
		bit_rate = (float) (((float) total_byte_cnt / (float) frm_count)) * 8 * (float)s_params.target_fr;
		fps = 400 * ((float)frm_count/(float)total_cycles);
		ftime = 1000/fps;
//		printf("Frames: %d BitRate = %f FPS = %.0f / %.0f ms\n", frm_count, bit_rate, 400 * ((float)frm_count/(float)total_cycles), 1000/(bit_rate, 400 * ((float)frm_count/(float)total_cycles)));
		printf("Frames =  %d, BitRate = %.0f bps, FPS = %.0f / %.0f ms per frame\n", total_frm_count, bit_rate, 400 * ((float)frm_count/(float)total_cycles), 1000/(bit_rate, 400 * ((float)frm_count/(float)total_cycles)));
		fflush(stdout);
//		printf("Total Bytes =  %d\n", total_byte_cnt);
//		printf("Debug: %d %d\n", rx_interrupt_cnt, rx_task_cnt);
//		total_byte_cnt = 0;
//		frm_count = 0;
//		total_cycles = 0.0;
	}

	return op_params.nalus_buffer;
}

void DeInitH264Encoder(void)
{
	printf("----------------------------------------------------------------------------\n");
	printf("Total Frames  : %4d\n", frm_count);
	printf("Total Bytes   : %d\n", total_byte_cnt);

#ifdef FIND_BITRATE
	frame_rate = (float) (((float) total_byte_cnt / (float) frm_count)) * 8 * (float)s_params.target_fr;
	target_fr = s_params.target_fr;
	printf("Average Bit Rate  %d FPS - %f bps\n", target_fr,frame_rate);
#endif

#ifdef ENABLE_BENCHMARK
	printf("Total Cycles consumed for processing %d frames : %f \n", frm_count, (float) (total_cycles));
	printf("Total Cycles consumed for processing 1 frame   : %f \n", (float) (total_cycles)/(float)frm_count);
	printf("Effective Frame Rate (FPS)                     : %f \n", 400 * ((float)frm_count/(float)total_cycles));
#endif

	printf("----------------------------------------------------------------------------\n\n");
}

#ifdef ENABLE_BENCHMARK
void StartBenchMark(void)
{
	SelectPerfEvent(PMON_COUNTER_0, IFETCH_MISS);
	SelectPerfEvent(PMON_COUNTER_1, DCACHE_REFILL);
	SelectPerfEvent(PMON_COUNTER_2, L2_CACH_MISS);
	SelectPerfEvent(PMON_COUNTER_3, DTLB_REFILL);
	StartPerfCounters();
}
void StopBenchMark(void)
{
	StopPerfCounters();
	tc= (GetCycleCount() >> 0);
	totalcycles += (GetCycleCount() >> 0);
	i_miss += GetPerfCount(PMON_COUNTER_0);
	d_miss += GetPerfCount(PMON_COUNTER_1);
	l2_miss += GetPerfCount(PMON_COUNTER_2);
	dtlb_miss += GetPerfCount(PMON_COUNTER_3);
}
#endif

void InitH264EncoderParams(h264e_input_config* params, tH264E_STATIC_PARAMS *c_params, tH264E_DYNAMIC_PARAMS *d_params)
{
	c_params->frame_width = params->frame_width;
	c_params->frame_height = params->frame_height;
	c_params->profile = (char)params->profile;
	c_params->level = (char)params->level;
	c_params->chroma_format = VIDEO_YUV_420SP;
	c_params->frame_content = VIDEO_PROGRESSIVE_FRAME;
	c_params->encoding_preset = (char)params->encoding_preset;
	c_params->rc_enable = (char)params->rc_enable;
	c_params->subpel_me = (char)params->subpel_me;
	c_params->search_range = (char)params->search_range;
	c_params->num_ref_frames = (char)params->num_ref_frames;
	c_params->frame_count = params->frame_count;
	c_params->air_rate = params->air_rate;

	d_params->target_fr = params->target_fr;
	d_params->target_br  = params->target_br;
	d_params->disable_intra4x4 = (char)params->disable_intra4x4;
	d_params->disable_intra16x16 = (char)params->disable_intra16x16;
	d_params->disable_inter16x16 = (char)params->disable_inter16x16;
	d_params->disable_inter16x8 = (char)params->disable_inter16x8;
	d_params->disable_inter8x16 = (char)params->disable_inter8x16;
	d_params->disable_inter8x8 = (char)params->disable_inter8x8;
	d_params->disable_inter8x4 = (char)params->disable_inter8x4;
	d_params->disable_inter4x8 = (char)params->disable_inter4x8;
	d_params->disable_inter4x4 = (char)params->disable_inter4x4;
	d_params->disable_intra_in_inter = (char)params->disable_intra_in_inter;
	d_params->force_I_frame = (char)params->force_I_frame;
	d_params->intra_period = params->intra_period;
	d_params->idr_period = params->idr_period;
	d_params->min_qp = (char)params->min_qp;
	d_params->max_qp = (char)params->max_qp;
	d_params->i_qp = (char)params->i_qp;
	d_params->p_qp = (char)params->p_qp;
	d_params->initial_qp = (char)params->initial_qp;
	d_params->max_mbs_per_slice = params->max_mbs_per_slice;
	d_params->num_slice_groups = (char)params->num_slice_groups;
	d_params->slice_group_map_type = (char)params->slice_group_map_type;
	d_params->constrained_intra_prediction = (char)params->constrained_intra_prediction;
	d_params->df_send_parameters = (char)params->df_send_parameters;
	d_params->df_disable_idc = (char)params->df_disable_idc;
	d_params->df_alpha = params->df_alpha;
	d_params->df_beta = params->df_beta;
}

void ReadConfigParams(h264e_input_config* params, int width, int height, int frameRate, int bitRate)
{
	params->infile = NULL ;
#ifdef WRITE_NALU
	params->outfile = (char *)&stream_file[0] ;
#endif
	params->reconfile = NULL ;

	params->frame_width 					= width ;
	params->frame_height 					= height ;
	params->frame_count 					= 100 ;
	params->profile 						= 0  ;
	params->level 							= 8  ;
	params->encoding_preset 				= 0  ;
	params->rc_enable 		 				= 1  ;
	params->search_range 					= 16 ;
	params->num_ref_frames 					= 1  ;
	params->subpel_me 						= 2  ;
	params->target_br 						= bitRate ;
	params->target_fr 						= frameRate ;
	params->disable_intra4x4 				= 0  ;
	params->disable_intra16x16 				= 0  ;
	params->disable_inter16x16 				= 0  ;
	params->disable_inter16x8 				= 1  ;
	params->disable_inter8x16 				= 1  ;
	params->disable_inter8x8 				= 1  ;
	params->disable_inter8x4 				= 1  ;
	params->disable_inter4x8 				= 1  ;
	params->disable_inter4x4 				= 1  ;
	params->force_I_frame 					= 0  ;
	params->disable_intra_in_inter 			= 0  ;
	params->i_qp							= 24 ;
	params->p_qp 							= 24 ;
	params->min_qp 							= 8  ;
	params->max_qp 							= 44 ;
	params->initial_qp 						= 0  ;
	params->constrained_intra_prediction	= 1  ;
	params->intra_period 					= 0  ;
	params->idr_period 						= frameRate * 3;
	params->air_rate 						= 0  ;
	params->max_mbs_per_slice 				= 0  ;
	params->num_slice_groups 				= 1  ;
	params->slice_group_map_type 			= 0  ;
	params->df_send_parameters 				= 1  ;
	params->df_disable_idc					= 0  ;
	params->df_alpha 						= 0  ;
	params->df_beta 						= 0  ;

	f_in = params->infile;
	f_out = params->outfile;
}

/*!
 ************************************************************************
 * \brief
 *    Prints the header of the protocol.
 * \par Input:
 *    struct inp_par *inp
 * \par Output:
 *    none
 ************************************************************************
 */
void PrintConfigParams(h264e_input_config* params, tH264E_STATUS_PARAMS *s_params)
{
	printf("--------------------------- ESL H.264 Encoder %4.4s ------------------------\n\n",CTPL_H264E_VERSION);
	printf(" Frames to be encoded I-P          : %d \n", params->frame_count);
	printf(" Image format                      : %dx%d \n",s_params->frame_width, s_params->frame_height);
	printf(" Profile/Level IDC                 : (%d,%d)\n",s_params->profile, s_params->level);
	printf(" Bit Rate                          : %d\n", s_params->target_br);
	printf(" Frame Rate                        : %d\n", s_params->target_fr);
	printf(" Rate Control                      : %d\n", s_params->rc_enable);
	printf(" Encoding Preset                   : %d\n", s_params->encoding_preset);
	printf(" Output File Format                : H.264/AVC Annex B Byte Stream Format \n");
	printf("\n");
	printf("----------------------------------------------------------------------------\n\n");
}

void WriteNALUToFile(void)
{
	int i,j, length;
	unsigned char * bs_buf;

	if(total_byte_cnt >= MAX_OUT_SIZE) return;

	bs_buf = op_params.nalus_buffer;
	for (i = 0; i < op_params.nalu_count; i++)
	{
		for(j=0; j<op_params.nalu_length[i]; j++)
			*f_out++ = bs_buf[j];

//		SendRTP(bs_buf, op_params.nalu_length[i]);

		bs_buf += op_params.nalu_length[i];

		byte_count +=(unsigned int)op_params.nalu_length[i];
	}
	total_byte_cnt += (int)byte_count;
}
