/***********************************************************************\
*                                                                       *
*                               NOTICE                                  *
*                                                                       *
*   This source code contains proprietary and confidential information, *
*   and is the property of Consilient Technologies Pvt Ltd.             *
*                                                                       *
*   This program has been distributed under license agreement and       *
*   any unauthorized use or distribution is prohibited.                 *
*                                                                       *
*   Copyright (c) 2013 Consilient Technologies Pvt Ltd                  *
*   All rights reserved.                                                *
*                                                                       *
*   Consilient Technologies Pvt Ltd                                     *
*   44-45, Residency (Cross) Road                                       *
*   Bangalore - 560 025, INDIA                                          *
*                                                                       *
*   URL      : http://www.consilient-tech.com                           *
*                                                                       *
\***********************************************************************/

#ifndef _H264E_API_H_
#define _H264E_API_H_

#define H264_NULL ((void *)0)

#define MAX_NALUS_PER_FRAME 	128
#define	MAX_BS_SIZE 		128000*2
/* max resoltion */
#define H264E_MAX_HORIZONTAL_SIZE	640
#define H264E_MAX_VERTICAL_SIZE		480


/* Frame Types */
#define VIDEO_P_FRAME                      0
#define VIDEO_B_FRAME                      1
#define VIDEO_I_FRAME                      2

/* Picture content type */
#define VIDEO_PROGRESSIVE_FRAME            0
#define VIDEO_INTERLACED_FRAME             1

/* Picture chroma format */
#define VIDEO_MONO_CHROME                  0
#define VIDEO_YUV_420P                     1
#define VIDEO_YUV_422P                     2
#define VIDEO_YUV_444P                     3
#define VIDEO_YUV_420SP                    4

/* Profile */
#define H264E_BASELINE_PROFILE             0
#define H264E_MAIN_PROFILE                 1
#define H264E_EXTENDED_PROFILE			   2
#define H264E_HIGH_PROFILE				   3

/* Levels */
#define H264E_LEVEL_10                     0
#define H264E_LEVEL_1B                     1
#define H264E_LEVEL_11                     2
#define H264E_LEVEL_12                     3
#define H264E_LEVEL_13                     4
#define H264E_LEVEL_20                     5
#define H264E_LEVEL_21                     6
#define H264E_LEVEL_22                     7
#define H264E_LEVEL_30                     8
#define H264E_LEVEL_31                     9
#define H264E_LEVEL_32                     10
#define H264E_LEVEL_40                     11
#define H264E_LEVEL_41                     12
#define H264E_LEVEL_42                     13
#define H264E_LEVEL_50                     14
#define H264E_LEVEL_51                     15

/* Encoding preset */
#define H264E_USER_DEFINED					0
#define H264E_HIGH_QUALITY					1
#define H264E_HIGH_SPEED					2
#define H264E_ULTRAHIGH_SPEED				3

/* Rate Control Enable/Disable*/
#define RC_DISABLE							0
#define RC_ENABLE							1

/* Subpel interpolation */
#define ENABLE_ALL_SUBPEL					0
#define DISABLE_ALL_SUBPEL					1
#define DISABLE_QPEL_SUBPEL					2

/* DF parameters */
#define FILTER_ALL_EDGES					0
#define DISABLE_FILTER_ALL_EDGES			1
#define DISABLE_FILTER_SLICE_EDGES			2

/* H.264 Encoder Data Structures */


/* State Memory Information */
typedef struct
{
    unsigned long state_mem_size;
    unsigned int align;
} tH264E_MEM_INFO;


/* Static Configuration Parameters */
typedef struct
{
    int frame_width;
    int frame_height;
    char profile;
    char level;
    char chroma_format;
    char frame_content;
    char encoding_preset;
    char rc_enable;
    char search_range;
    char num_ref_frames;
    char subpel_me;
    int frame_count;
    int air_rate;
} tH264E_STATIC_PARAMS;


/* Dynamic Configuration Parameters */
typedef struct
{
    int target_fr;
    int target_br;
    char disable_intra4x4;
    char disable_intra16x16;
    char disable_interskip;
    char disable_inter16x16;
    char disable_inter16x8;
    char disable_inter8x16;
    char disable_inter8x8;
    char disable_inter8x4;
    char disable_inter4x8;
    char disable_inter4x4;
    char force_I_frame;
    char disable_intra_in_inter;
    char i_qp;
    char p_qp;
    char initial_qp;
    char min_qp;
    char max_qp;
    char constrained_intra_prediction;
    int intra_period;
    int idr_period;
    int max_mbs_per_slice;
    char num_slice_groups;
    char slice_group_map_type;
    char df_send_parameters;
    char df_disable_idc;
    int df_alpha;
    int df_beta;
} tH264E_DYNAMIC_PARAMS;

/* Configuration Status Parameters */
typedef struct
{
    int frame_width;
    int frame_height;
    char profile;
    char level;
    int target_fr;
    int target_br;
    char chroma_format;
    char frame_content;
    char encoding_preset;
    char rc_enable;
} tH264E_STATUS_PARAMS;

/* Encoder Process Input Parameters */
typedef struct
{
    unsigned char *h264e_state_mem;
    unsigned char save_recon_bufs;
    unsigned char *input_frame;
} tH264E_INPUT_PARAMS;


/* Encoder Process Output Parameters */
typedef struct
{
    int frame_type;
    int frame_qp;
    int idr_flag;
    int nalu_count;
    int nalu_length[MAX_NALUS_PER_FRAME];
    unsigned char nalus_buffer[MAX_BS_SIZE];
    unsigned char *recon_frame;
} tH264E_OUTPUT_PARAMS;

void CTPL_H264_VENC_GetMemInfo(tH264E_MEM_INFO *);
int CTPL_H264_VENC_Init(unsigned char *, tH264E_STATIC_PARAMS *, tH264E_DYNAMIC_PARAMS *);
int CTPL_H264_VENC_Status(unsigned char *, tH264E_STATUS_PARAMS *);
int CTPL_H264_VENC_Control(unsigned char *, tH264E_DYNAMIC_PARAMS *);
int CTPL_H264_VENC_Process(unsigned char *, tH264E_INPUT_PARAMS *, tH264E_OUTPUT_PARAMS *);

#endif
