/****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "r_typedefs.h"                         /* Renesas data types header */
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "config.h"

/* Renesas includes. */
#include "r_typedefs.h"
#include "sio_char.h"

/* FreeRTOS+UDP includes. */
#include "FreeRTOS_UDP_IP.h"
#include "FreeRTOS_Sockets.h"

#define VERSION_NUMBER	"1.0.2"

#ifdef CAMERA_OV7670
#define CAMERA_USED		"OV7670"
#else
#define CAMERA_USED		"OV7740"
#endif

/****************************************************************************/

uint8_t ucMACAddress[ 6 ] = { 0x00, 0x11, 0x22, 0x33, 0x44, 0x55 };
//static uint8_t pcIPAddress[15] = "192.168.88.200";
int videoTaskStarted = 0;

/****************************************************************************/

/****************************************************************************/
void InitCameraModule(int cameraType, int videoResolution, int videoWidth, int videoHeight);
void InitIPStack(void);
int InitH264Encoder(int width, int height, int frameRate, int bitRate);
uint32_t CreateRTPSocket(uint8_t *pcIPAddress, int dstPort);
int InitMPEG2TS(void);
void StartVideoProcess(int frameRate);
int32_t ReadDataFromEEPROM(unsigned char *params, int size);
int32_t WriteDataToEEPROM(unsigned char *params, int size);
void InitLEDTickCount(void);
/****************************************************************************/
typedef struct {
	int32_t ipAddrMode;
	int cameraType;
	int videoResolution;
	int videoFrameRate;
	int videoBitRate;
	int videoWidth;
	int videoHeight;
	uint8_t ucIPAddress[16];
	uint8_t ucNetMask[16];
	uint8_t ucGatewayAddress[16];
	uint8_t ucDNSServerAddress[16];
	uint8_t ucRemoteIPAddress[16];
	uint16_t rtpPort;
	uint8_t configValid;
	uint8_t checkSum;
} APP_CONFIG;

APP_CONFIG config;

void SetDefaultConfigParams(void)
{
	config.ipAddrMode = DEFAULT_IP_ADDR_MODE;
	config.cameraType = DEFAULT_CAMERA_TYPE;
	config.videoResolution = DEFALUT_VIDEO_RESOLUTION;
	config.videoFrameRate = DEFAULT_FRAME_RATE;
	config.videoBitRate = DEFAULT_BIT_RATE;
	config.rtpPort= DEFAULT_REMOTE_RTP_PORT;
	config.configValid = 0x11;
    config.checkSum = 0;

	strcpy(config.ucIPAddress, DEFAULT_IP_ADDRESS);
	strcpy(config.ucNetMask, DEFAULT_NET_MASK);
	strcpy(config.ucGatewayAddress, DEFAULT_GW_ADDRESS);
	strcpy(config.ucDNSServerAddress, DEFAULT_DNS_ADDRESS);
	strcpy(config.ucRemoteIPAddress, DEFAULT_REMOTE_ADDRESS);

	if(config.videoResolution == 0)
	{
		config.videoWidth = 320;
		config.videoHeight = 240;
	}
	else
	{
		config.videoWidth = 640;
		config.videoHeight = 480;
	}
}

static void PrintConfigData(void)
{
	printf("Streaming Configuration:\n");

    if(config.ipAddrMode == 1)
    	printf("IP Address Mode:        DHCP\n");
    else
    {
    	printf("IP Address Mode:        STATIC\n");
    	printf("IP: %s  GW: %s, DNS: %s Mask: %s\n", config.ucIPAddress, config.ucGatewayAddress, config.ucDNSServerAddress, config.ucNetMask);
    }

	printf("Remote IP Address:      %s\n", config.ucRemoteIPAddress);
	printf("Remote RTP Port:        %d\n", config.rtpPort);

	printf("Video Resolution:       ");
	if(config.videoResolution == 0)
		printf("QVGA (320x240)\n");
	else
		printf("VGA (640x480)\n");

	printf("Video Frame Rate:       %d FPS\n", config.videoFrameRate);
	printf("Video Bit Rate:         %d bps\n", config.videoBitRate);
}

void ReadConfigData(void)
{
    char_t         temp[24];
    int dummy;

	printf("\n\nChoose Streaming Configuration:\n\n");

	/* IP Addressing Mode */
	while(1)
	{
		printf("IP Addressing mode (0 - STATIC,  1 - DHCP): ");
		scanf("%d", &config.ipAddrMode);

    	if(config.ipAddrMode != 0 && config.ipAddrMode != 1)
    	{
    		printf("Invalid IP Address Mode, Please enter either 0 or 1\n");
    		continue;
    	}

		if(config.ipAddrMode == 0)
		{
			printf("\nStatic IP address parameters:\n\n");

			printf("IP address: ");
			scanf("%s", (uint32_t *)config.ucIPAddress);

			printf("Gateway IP address: ");
			scanf("%s", (uint32_t *)config.ucGatewayAddress);

			printf("DNS IP address: ");
			scanf("%s", (uint32_t *)config.ucDNSServerAddress);

			printf("Net Mask address: ");
			scanf("%s", (uint32_t *)config.ucNetMask);
		}

		break;
	}

	/* Remote IP Address */
	printf("\nRemote IP address: ");
	scanf("%s", (uint32_t *)config.ucRemoteIPAddress);

	printf("\nRemote RTP Port (1000 to 8000): ");
	scanf("%d", &config.rtpPort);

	/* Video Resolution Selection */
    while(1)
    {
    	printf("\nVideo Resolution (0 - QVGA, 1 - VGA): ");
    	scanf("%d", &config.videoResolution);
    	if(config.videoResolution != 0 && config.videoResolution != 1)
    	{
    		printf("Invalid Video Resolution, Please enter either 0 or 1\n");
    		continue;
    	}
    	if(config.videoResolution == 0)
    	{
    		config.videoWidth = 320;
    		config.videoHeight = 240;
    	}
   		else
   		{
    		config.videoWidth = 640;
    		config.videoHeight = 480;
   		}

    	break;
    }

	/* Video Frame Rate Selection */
    while(1)
    {
    	printf("\nVideo Frame Rate (1 to 25): ");
    	scanf("%d", &config.videoFrameRate);
    	if(config.videoFrameRate < 1 || config.videoFrameRate > 25)
    	{
    		printf("Invalid Video Frame Rate, Please enter a valid frame rate \n");
    		continue;
    	}
    	break;
    }

	/* Video Bit Rate Selection */
   	printf("\nVideo Bit Rate in kbps (64 to 1024): ");
   	scanf("%d", &config.videoBitRate);
   	if(config.videoBitRate > 1024)
   	{
   		printf("Max bit rate supported 1024kbps, Selected 1024kbps\n");
   		config.videoBitRate = 1024;
   	}
   	config.videoBitRate = config.videoBitRate*1000;

   	config.configValid = 0x11;
    config.checkSum = 0;

   	printf("\nStreaming Configuration Done!\n\n");
}

void PrintCTPLBanner(void)
{
	printf("\n");
	printf("-----------------------------------------------------\n");
	printf("           Consilient Technologies Pvt. Ltd        \n");
	printf("           web:    www.consilient-tech.com         \n");
	printf("           E-mail: raghava@consilient-tech.com     \n");
	printf("-----------------------------------------------------\n");
	printf("            H.264 Streaming on RZ/A1L\n");
	printf("            Version %s for Camera %s\n", VERSION_NUMBER, CAMERA_USED);
	printf("-----------------------------------------------------\n");
}

void GetIpAddressBytes(uint8_t *str, uint8_t *value)
{
	int index = 0;

	value[0] = value[1] = value[2] = value[3] = 0;
	while (*str)
	{
//		if (isdigit((unsigned char)*str))
		if(*str != '.')
		{
				value[index] *= 10;
				value[index] += *str - '0';
		}
		else
			index++;

		str++;
	}
}

int main( void )
{
	int32_t retVal;
	uint8_t ip_values[4];

	PrintCTPLBanner();

	Pekoe_RIIC_init();

#ifdef PEKOE_BOARD
    retVal = ReadDataFromEEPROM((unsigned char *)&config, sizeof(config));
    if(retVal < 0 || config.configValid != 0x11)
    {
    	ReadConfigData();  //Invalid Configuration, Offer the user menu
	    WriteDataToEEPROM((unsigned char *)&config, sizeof(config));
    }
    else
    {
       	PrintConfigData();
    	retVal = IoGetcharDelay(4000);
    	if(retVal)
    	{
    		ReadConfigData();  //Key Stroke, Offer the user menu
    	    WriteDataToEEPROM((unsigned char *)&config, sizeof(config));
    	}
    }
#else
	ReadConfigData();  //Invalid Configuration, Offer the user menu
#endif

   	PrintConfigData();

   	GetIpAddressBytes(config.ucRemoteIPAddress, ip_values);
   	ucMACAddress[3] = ip_values[3];
   	ucMACAddress[4] = (config.rtpPort >> 8) & 0xff;
   	ucMACAddress[5] = config.rtpPort & 0xff;

   	printf("\nSetting up Ethernet ");
	printf("(MAC: %02x-%02x-%02x-%02x-%02x-%02x)...\n\n", ucMACAddress[0], ucMACAddress[1], ucMACAddress[2], ucMACAddress[3], ucMACAddress[4], ucMACAddress[5]);

#ifdef CAMERA_OV7670
    config.cameraType = 0;
#else
    config.cameraType = 1;
#endif

	InitCameraModule(config.cameraType, config.videoResolution, config.videoWidth, config.videoHeight);

	InitIPStack();

	InitH264Encoder(config.videoWidth, config.videoHeight, config.videoFrameRate, config.videoBitRate);

	CreateRTPSocket(config.ucRemoteIPAddress, config.rtpPort);

	InitMPEG2TS();

	videoTaskStarted = 0;

	vTaskStartScheduler();

	return 0;
}

/*-----------------------------------------------------------*/
void vApplicationMallocFailedHook( void )
{
	/* Called if a call to pvPortMalloc() fails because there is insufficient
	free memory available in the FreeRTOS heap.  pvPortMalloc() is called
	internally by FreeRTOS API functions that create tasks, queues, software
	timers, and semaphores.  The size of the FreeRTOS heap is set by the
	configTOTAL_HEAP_SIZE configuration constant in FreeRTOSConfig.h. */
	taskDISABLE_INTERRUPTS();
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle pxTask, signed char *pcTaskName )
{
	( void ) pcTaskName;
	( void ) pxTask;

	/* Run time stack overflow checking is performed if
	configCHECK_FOR_STACK_OVERFLOW is defined to 1 or 2.  This hook
	function is called if a stack overflow is detected. */
	taskDISABLE_INTERRUPTS();
	for( ;; );
}

/*-----------------------------------------------------------*/

void vApplicationIdleHook( void )
{
//volatile size_t xFreeHeapSpace;
volatile int xFreeHeapSpace;
	/* This is just a trivial example of an idle hook.  It is called on each
	cycle of the idle task.  It must *NOT* attempt to block.  In this case the
	idle task just queries the amount of FreeRTOS heap that remains.  See the
	memory management section on the http://www.FreeRTOS.org web site for memory
	management options.  If there is a lot of heap memory free then the
	configTOTAL_HEAP_SIZE value in FreeRTOSConfig.h can be reduced to free up
	RAM. */
	xFreeHeapSpace = xPortGetFreeHeapSize();

	/* Remove compiler warning about xFreeHeapSpace being set but never used. */
	( void ) xFreeHeapSpace;
}
/*-----------------------------------------------------------*/

void vAssertCalled( const char * pcFile, unsigned long ulLine )
{
volatile unsigned long ul = 0;

	( void ) pcFile;
	( void ) ulLine;

	taskENTER_CRITICAL();
	{
		/* Set ul to a non-zero value using the debugger to step out of this
		function. */
		while( ul == 0 )
		{
			portNOP();
		}
	}
	taskEXIT_CRITICAL();
}
/*-----------------------------------------------------------*/

void InitIPStack(void)
{
	FreeRTOS_IPInit(config.ipAddrMode, config.ucIPAddress, config.ucNetMask, config.ucGatewayAddress, config.ucDNSServerAddress, ucMACAddress );
}

void vApplicationTickHook( void )
{
#if 0
	#if mainCREATE_SIMPLE_BLINKY_DEMO_ONLY == 0
	{
		/* The full demo includes a software timer demo/test that requires
		prodding periodically from the tick interrupt. */
		vTimerPeriodicISRTests();

		/* Call the periodic queue overwrite from ISR demo. */
		vQueueOverwritePeriodicISRDemo();
	}
	#endif
#endif
}

void vApplicationIPNetworkEventHook( eIPCallbackEvent_t eNetworkEvent )
{
	uint32_t ulIPAddress, ulNetMask, ulGatewayAddress, ulDNSServerAddress;
	int8_t cBuffer[ 16 ];

    if( eNetworkEvent == eNetworkUp )
    {
        /* The network is up and configured.  Print out the configuration
        obtained from the DHCP server. */
        FreeRTOS_GetAddressConfiguration( &ulIPAddress,
                                          &ulNetMask,
                                          &ulGatewayAddress,
                                          &ulDNSServerAddress );

        /* Convert the IP address to a string then print it out. */
        FreeRTOS_inet_ntoa( ulIPAddress, cBuffer );
        printf( "IP Address: %s\r\n", cBuffer );

        /* Convert the net mask to a string then print it out. */
        FreeRTOS_inet_ntoa( ulNetMask, cBuffer );
        printf( "Subnet Mask: %s\r\n", cBuffer );

        /* Convert the IP address of the gateway to a string then print it out. */
        FreeRTOS_inet_ntoa( ulGatewayAddress, cBuffer );
        printf( "Gateway IP Address: %s\r\n", cBuffer );

        /* Convert the IP address of the DNS server to a string then print it out. */
        FreeRTOS_inet_ntoa( ulDNSServerAddress, cBuffer );
        printf( "DNS server IP Address: %s\r\n", cBuffer );

        vTaskDelay(500);

        //Network UP, Start the video send task

#ifdef PEKOE_BOARD
        LED_Init();
        InitLEDTickCount();
#endif
        printf("\nStart Video Streaming: rtp:\\\\%s:%d\n", config.ucRemoteIPAddress, config.rtpPort);
        printf("Video Properties: Width = %d Height = %d FrameRate = %d BitRate = %d\n\n", config.videoWidth, config.videoHeight, config.videoFrameRate, config.videoBitRate);
        printf("Streaming Statistics:\n");

        if(!videoTaskStarted)
        	StartVideoProcess(config.videoFrameRate);

        videoTaskStarted = 1;
    }
}

/* Called automatically when a reply to an outgoing ping is received. */
void vApplicationPingReplyHook( ePingReplyStatus_t eStatus, uint16_t usIdentifier )
{
	switch( eStatus )
	{
		case eSuccess	:
			printf("Ping Reply Success - ");
			break;

		case eInvalidChecksum :
			printf("Ping reply received with invalid checksum - ");
			break;

		case eInvalidData :
			printf("Ping reply received with invalid data - ");
			break;

		default :
			/* It is not possible to get here as all enums have their own
			case. */
			break;
	}

	printf("identifier %d\r\n", ( int ) usIdentifier );
}


void tm_memcpy(uint8_t *dst, uint8_t *src, uint32_t size)
{
	int i;
	for(i = 0; i < size; i++)
		*dst++ = *src++;
}

void tm_memset(uint8_t *dst, uint8_t value, uint32_t size)
{
	int i;
	for(i = 0; i < size; i++)
		*dst++ = value;
}

