#include <stdio.h>
#include <stdlib.h>

/* Scheduler include files. */
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

/* Renesas includes. */
#include "r_typedefs.h"
#include "sio_char.h"

/* FreeRTOS+UDP includes. */
#include "FreeRTOS_UDP_IP.h"
#include "FreeRTOS_Sockets.h"
#include "RtpSession.h"

#define	VIDEO_SEND_TASK_PRIORITY	( tskIDLE_PRIORITY + 1 )
#define	VIDEO_RECEIVE_TASK_PRIORITY	( tskIDLE_PRIORITY + 2 )
#define VIDEO_RTP_PORT		5004

xSocket_t xSocket;
struct freertos_sockaddr xBindAddress, xDestinationAddress;
RTPPACKET	RTPPacket ;

int prevTickValue;

/*****************************************************************************/
void VideoSendTask( void *pvParameters );
void VideoReceiveTask( void *pvParameters );
uint32_t CreateRTPSocket(uint8_t *pcIPAddress, int dstPort);
uint32_t SendRTP(uint8_t *Buffer, uint32_t Length);
uint32_t ReceiveRTP(uint8_t *Buffer, uint32_t Length);
unsigned char *ProcessH264Encoder(unsigned char *frameBuf, unsigned int *frameSize);
int InitMPEG2TS(void);
int ProcessMPEF2TS(unsigned char *payloadBuf, unsigned int payloadSize);
/*****************************************************************************/

#define printf

unsigned long GetRandom( void )
{
	return 0;
}

uint32_t CreateRTPSocket(uint8_t *pcIPAddress, int dstPort)
{
	uint32_t ulIPAddress;

	xSocket = FreeRTOS_socket( FREERTOS_AF_INET, FREERTOS_SOCK_DGRAM, FREERTOS_IPPROTO_UDP );
	if( xSocket == FREERTOS_INVALID_SOCKET )
	{
		printf("Failed to Create Socket\n");
		return -1;
	}

	xBindAddress.sin_port = VIDEO_RTP_PORT;
	xBindAddress.sin_port = FreeRTOS_htons( xBindAddress.sin_port );
	if( FreeRTOS_bind( xSocket, &xBindAddress, sizeof( &xBindAddress ) ) == 0 )
	{
		printf("Bind Successful\n");
	}
	else
		return -1;

	/* Remote Address To send data */
	ulIPAddress = FreeRTOS_inet_addr( pcIPAddress );
	xDestinationAddress.sin_addr = ulIPAddress;
	xDestinationAddress.sin_port = dstPort;
	xDestinationAddress.sin_port = FreeRTOS_htons( xDestinationAddress.sin_port );
}

void	SetRTPTimeStamp (FIXEDRTPHEADER* RtpPacketHdr, unsigned int TimeStamp)
{
	RtpPacketHdr->TimeStamp =  FreeRTOS_htonl( TimeStamp ) ;
}

void	SetRTPSequenceNumber (FIXEDRTPHEADER* RtpPacketHdr, unsigned short SequenceNumber)
{
	RtpPacketHdr->SequenceNumber = FreeRTOS_htons (SequenceNumber) ;
}

void InitRTPHeader ()
{
	unsigned int SSRC = 0xdeadbeef ;
	FIXEDRTPHEADER* RtpPacketHdr = &RTPPacket.RtpPacketHdr ;

	RtpPacketHdr->CSRCCount = 0x0 ;
	RtpPacketHdr->Extension = 0x0 ;
	RtpPacketHdr->Padding = FALSE ;          // Padding Flag
	RtpPacketHdr->Version = MAXVERSION ;
	RtpPacketHdr->PayloadType = MPEG2_PAYLOADTYPE;
	RtpPacketHdr->Marker = 0x1 ;

	RtpPacketHdr->SequenceNumber = 100 ;      // Sequence Number of Packet
	RTPPacket.SequenceNumber = 100 ;

	RtpPacketHdr->SSRC = FreeRTOS_htonl (SSRC) ;                // Synchronisation Source

	RTPPacket.TimeStamp = GetRandom () ;
	SetRTPTimeStamp (RtpPacketHdr, RTPPacket.TimeStamp);

#if 0
	gettimeofday (&RTPPacket.RtpCurrTime, NULL) ;
#else
	RTPPacket.RtpCurrTime = xTaskGetTickCount();
#endif

	return ;
}

void AddRTPHeader ()
{
	FIXEDRTPHEADER* RtpPacketHdr = &RTPPacket.RtpPacketHdr ;

	SetRTPSequenceNumber (RtpPacketHdr, RTPPacket.SequenceNumber);
	RTPPacket.SequenceNumber += 1 ;

	SetRTPTimeStamp (RtpPacketHdr, RTPPacket.TimeStamp);
	RTPPacket.TimeStamp += GetDiffMSec (RTPPacket.RtpCurrTime) * 90 ; //90 KHz timestamp

#if 0
	gettimeofday (&RTPPacket.RtpCurrTime, NULL) ;
#else
	RTPPacket.RtpCurrTime = xTaskGetTickCount();
#endif

	memcpy (RTPPacket.RtpDataBuf,RtpPacketHdr, RTP_FIXED_HEADER_LENGTH);

	return ;
}

int InitializeRTPChannel (void)
{
	RTPPacket.TSPacketCount = 0;
	RTPPacket.RtpDataBufLen = RTP_FIXED_HEADER_LENGTH ;

	/* Initialize RTP Fixed Header */
	InitRTPHeader ();

	return 0 ;
}

void SendTSPacket (unsigned char *TSDataBuf,int LastTsPacket)
{
	int	ret ;

	memcpy (&RTPPacket.RtpDataBuf[RTPPacket.RtpDataBufLen], TSDataBuf,188) ;
	RTPPacket.RtpDataBufLen += 188;

	RTPPacket.TSPacketCount++ ;

		printf ("Adding RTP Header..\n");
	if ((RTPPacket.TSPacketCount == 7) || (LastTsPacket))
	{
		 AddRTPHeader ();

		/* Send RTP Packet over UDP */
		ret = SendRTP(RTPPacket.RtpDataBuf, RTPPacket.RtpDataBufLen);
		if (ret <= 0)
			printf ("Error sending RTP packet ..\n") ;

		/* Reinitialize RTP structure members */
		RTPPacket.TSPacketCount = 0 ;
		RTPPacket.RtpDataBufLen = RTP_FIXED_HEADER_LENGTH ;
	}

	return ;

}

void CloseRTPChannel ()
{
	FreeRTOS_closesocket((xSocket_t)RTPPacket.SocketFD);

}

void InitLEDTickCount(void)
{
	prevTickValue = xTaskGetTickCount();
}

void BlinkLED(void)
{
	int diffTick;
	diffTick = GetDiffMSec(prevTickValue);
	if(diffTick >= 500)
	{
		LED_TOGGLE();
		prevTickValue = xTaskGetTickCount();
	}
}

uint32_t SendRTP(uint8_t *Buffer, uint32_t Length)
{
	portBASE_TYPE lReturned = 0;

	while(lReturned != Length)
		lReturned = FreeRTOS_sendto( xSocket, ( void * ) Buffer, Length, 0, &xDestinationAddress, sizeof( xDestinationAddress ) );

	BlinkLED();

	if(lReturned < Length)
	{
		printf("Packet Send Error = %d\n", lReturned);
		return -1;
	}

	return lReturned;
}

uint32_t ReceiveRTP(uint8_t *Buffer, uint32_t Length)
{
	struct freertos_sockaddr xClient;
	uint32_t xClientLength = sizeof( xClient );
	uint32_t lBytes;

	lBytes = FreeRTOS_recvfrom( xSocket, Buffer, Length, 0, &xClient, &xClientLength );

	return lBytes;
}


