#include <stdio.h>
#include <stdlib.h>

/* Scheduler include files. */
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

/* Renesas includes. */
#include "r_typedefs.h"
#include "sio_char.h"

/* FreeRTOS+UDP includes. */
#include "FreeRTOS_UDP_IP.h"
#include "FreeRTOS_Sockets.h"
#include "RtpSession.h"
#include "compiler_settings.h"
#include "config.h"

//#define SAVE_FRAME

#define	VIDEO_SEND_TASK_PRIORITY	( tskIDLE_PRIORITY + 1 )
#define	VIDEO_RECEIVE_TASK_PRIORITY	( tskIDLE_PRIORITY + 2 )
int taskDelay;
int frameCount = 0;

#ifdef SAVE_FRAME
#define FRAME_LENGTH  (MAX_WIDTH*MAX_HEIGHT) + (MAX_WIDTH*MAX_HEIGHT)/2
#define FRAME_SAVE_BUFFER_LENGTH	(30*1024*1024)
int frameSaveIndex = 0;

#pragma arm section zidata = "BSS_SDRAM_SECTION"
#pragma arm section rwdata = "BSS_SDRAM_SECTION"
unsigned char BSS_SDRAM_SECTION frameSaveBuffer[FRAME_SAVE_BUFFER_LENGTH];
#pragma arm section zidata
#pragma arm section rwdata

#endif


/*****************************************************************************/
void StartCameraCapture(void);
void VideoSendTask( void *pvParameters );
void VideoReceiveTask( void *pvParameters );
unsigned char *ProcessH264Encoder(unsigned char *frameBuf, unsigned int *frameSize);
int ProcessMPEG2TS(unsigned char *payloadBuf, unsigned int payloadSize);
uint32_t SendRTP(uint8_t *Buffer, uint32_t Length);
unsigned char *GetCameraFrame(void);
uint32_t ReceiveRTP(uint8_t *Buffer, uint32_t Length);
/*****************************************************************************/

void SetTaskDelay(int frameRate)
{
#if 0
	switch(frameRate)
	{
		case 5:
			taskDelay = ( 200 / portTICK_RATE_MS );
//			taskDelay = ( 166 / portTICK_RATE_MS );
			break;

		case 10:
			taskDelay = ( 100 / portTICK_RATE_MS );
			break;

		case 15:
			taskDelay = ( 66 / portTICK_RATE_MS );
			break;

		case 20:
			taskDelay = ( 50 / portTICK_RATE_MS );
			break;

		case 25:
			taskDelay = ( 40 / portTICK_RATE_MS );
			break;

		case 30:
			taskDelay = ( 33 / portTICK_RATE_MS );
			break;

		default:
			taskDelay = ( 200 / portTICK_RATE_MS );
			break;
	}
#else
	if(frameRate >= 1 && frameRate <= 30)
		taskDelay = ( (1000/frameRate) / portTICK_RATE_MS );

//	printf("taskDelay = %d\n", taskDelay);
#endif
}

void StartVideoProcess(int frameRate)
{
	SetTaskDelay(frameRate);

	xTaskCreate( VideoSendTask,					/* The function that implements the task. */
				( signed char * ) "VSEND", 		/* The text name assigned to the task - for debug only as it is not used by the kernel. */
				configMINIMAL_STACK_SIZE*10, 	/* The size of the stack to allocate to the task. */
				NULL, 							/* The parameter passed to the task - not used in this case. */
				VIDEO_SEND_TASK_PRIORITY,/* The priority assigned to the task. */
				NULL );							/* The task handle is not required, so NULL is passed. */
}

#ifdef SAVE_FRAME
void SaveFrame(unsigned char *frameBuf)
{
	int i;

	if((frameSaveIndex + FRAME_LENGTH) > FRAME_SAVE_BUFFER_LENGTH)
		return;

	for(i = 0; i < FRAME_LENGTH; i++)
		frameSaveBuffer[frameSaveIndex++] = frameBuf[i];
}
#endif

void VideoSendTask( void *pvParameters )
{
	unsigned char *frameBuf;
	uint8_t dummyBuf[10];
	portTickType xNextWakeTime;
	unsigned int payloadSize = 0;
	unsigned char *payloadBuf;
	int i;

	/* Initialise xNextWakeTime - this only needs to be done once. */
	xNextWakeTime = xTaskGetTickCount();

	StartCameraCapture();

#ifdef SAVE_FRAME
	frameSaveIndex = 0;
#endif

	//Dummy Packet
	for(i = 0; i < 1; i++)
	{
		vTaskDelayUntil( &xNextWakeTime, taskDelay );
		SendRTP(dummyBuf, sizeof(dummyBuf));
	}

	while(1)
	{
#if 1
		if(taskDelay < 200)
		    vTaskDelayUntil( &xNextWakeTime, taskDelay );
#else
	    vTaskDelayUntil( &xNextWakeTime, taskDelay );
#endif

	    frameBuf = GetCameraFrame();

#ifdef SAVE_FRAME
		SaveFrame(frameBuf);
#endif
		payloadBuf = ProcessH264Encoder(frameBuf, &payloadSize); //Encode one frame
		ProcessMPEG2TS(payloadBuf, payloadSize);
#if 0
		printf("Frame: %d %d\n", ++frameCount, payloadSize);
#endif
	};
}
